//
// C++ Interface: id3libinterface
//
// Description:
//
//
// Author: Roland Weigert <roweigert@t-online.de>, (C) 2009
//
// Copyright: See COPYING file that comes with this distribution
//
//

/**
 *@file id3libinterface.h
 *The class for id3libwriting is declared in this file.<br>
 *Include this file, if you want to write with id3lib<br>
 *There is no kind of readsupport with id3lib, since kyamo does all reading with taglib.
 *Taglib is oh so much faster, so this will not change at all.<br>
 *You also need the files globals.h and taginformation.h when you want to use this file.
 *
 *@author Roland Weigert
 *@date 04-03-2009
 *@todo Write all
 */

#ifndef _ID3LIBINTERFACE_H_
#define _ID3LIBINTERFACE_H_

#include <QString>
#include <QFile>
#include <QFileInfo>
#include <QBuffer>

//taglib-library-includes
#include <id3/tag.h>

#include "globals.h"
#include "taginformation.h"

/**
 *@brief The Interface class to id3libwriting
 *
 *All actions that use id3lib, are handled over this interfaceclass
 *There is no way to set single tag informations.
 *Reading and setting tags is done only over a complete Taginformation
 */
class id3libinterface
  {
  public:
    /**
    *@brief Constructor
    *
    *Takes over the filename, and initializes the tags, to a zerovalue
    *@PARAM QString that takes the filename
     */
    id3libinterface(const QString);
    /**
     *@brief Destructor
     */
    ~id3libinterface();
    /**
     *@brief Saves a tag
     *
     *Saves the given taginformation into the filename that was given in constructor.<br>
     *As default it saves both id3v1- and id3v2-tags, when no version is given.
     *@PARAM tag Contains all informations that should be saved.
     *@PARAM version Contains the tagversion the file will be saved if it is mp3(use tagversion::tagtype to set filetype)
     */
    bool save(const taginformation& tag,const int version=tagversion::v1tag|tagversion::v2tag);
  private:
    bool writemp3(const taginformation& tag,const int version=tagversion::v1tag|tagversion::v2tag);
    /**
     *@brief Store picture in this file
     *
     *@param picture The QImage that has to be included in file.
     *@param picposition Use Coverart::pt  defined in globals.h to choose wich picture to save.
     *@param filetosave The file that is used for saving.
     */
    void storepicture(const QImage& picture,const Coverart::pt picposition);
    /**Internal taginformation, for handing over to and from the real save-function*/
    taginformation tag;
    /**Only internal use of filename. It is automaticly set by constructor*/
    QString filename;
  };

#endif
