/***************************************************************************
 *   Copyright (C) 2007 by Roland Weigert   *
 *   roweigert@t-online.de             *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include "coverbox.h"

coverchooser::coverchooser(QWidget *parent,const QImage &cover):
    QDialog(parent)
{
  this->setWindowTitle(tr("Set Cover"));
  image=cover.copy();
  coverlabel=new QLabel(this);
  QImage scaled=cover.scaledToWidth(100);
  coverlabel->setPixmap(QPixmap::fromImage(scaled));
  coverlabel->setAlignment(Qt::AlignCenter);
  infotext=new QLabel(tr("Select the right position for this picture."),this);
  layout=new QVBoxLayout(this);
  layout->addWidget(coverlabel);
  layout->addWidget(infotext);
  buttonlayout=new QHBoxLayout(this);
  frontbutton=new QPushButton(tr("Front-Image"),this);
  mediabutton=new QPushButton(tr("Media-Image"),this);
  backbutton=new QPushButton(tr("Back-Image"),this);
  buttonlayout->addWidget(frontbutton);
  buttonlayout->addWidget(mediabutton);
  buttonlayout->addWidget(backbutton);
  layout->addLayout(buttonlayout);
  this->setModal(TRUE);
  this->setLayout(layout);
  this->show();
  connect (frontbutton,SIGNAL(clicked()),this,SLOT(frontslot()));
  connect (mediabutton,SIGNAL(clicked()),this,SLOT(mediaslot()));
  connect (backbutton,SIGNAL(clicked()),this,SLOT(backslot()));
}

coverchooser::coverchooser(QWidget *parent,const int setcovers):
    QDialog(parent)
{
  this->setWindowTitle(tr("Remove Cover"));
  image=QImage(":/icons/question.png");
  coverlabel=new QLabel(this);
  coverlabel->setPixmap(QPixmap::fromImage(image));
  coverlabel->setAlignment(Qt::AlignCenter);
  infotext=new QLabel(tr("Select the picture you want to remove."),this);
  layout=new QVBoxLayout(this);
  layout->addWidget(coverlabel);
  layout->addWidget(infotext);
  buttonlayout=new QHBoxLayout(this);
  frontbutton=new QPushButton(tr("Front-Image"),this);
  mediabutton=new QPushButton(tr("Media-Image"),this);
  backbutton=new QPushButton(tr("Back-Image"),this);
  if (!(setcovers&1))
    frontbutton->setEnabled(FALSE);
  if (!(setcovers&2))
    mediabutton->setEnabled(FALSE);
  if (!(setcovers&4))
    backbutton->setEnabled(FALSE);
  buttonlayout->addWidget(frontbutton);
  buttonlayout->addWidget(mediabutton);
  buttonlayout->addWidget(backbutton);
  layout->addLayout(buttonlayout);
  this->setModal(TRUE);
  this->setLayout(layout);
  this->show();
  connect (frontbutton,SIGNAL(clicked()),this,SLOT(frontslot()));
  connect (mediabutton,SIGNAL(clicked()),this,SLOT(mediaslot()));
  connect (backbutton,SIGNAL(clicked()),this,SLOT(backslot()));
}

void coverchooser::closeEvent(QCloseEvent *event)
{
  event->ignore();
  emit cancelsignal();
}

void coverchooser::frontslot()
{
  emit selectedas(Coverart::FrontCover,image);
}

void coverchooser::mediaslot()
{
  emit selectedas(Coverart::Media,image);
}

void coverchooser::backslot()
{
  emit selectedas(Coverart::BackCover,image);
}

coverbox::coverbox(QWidget *parent):
    QGroupBox(tr("CD-Covers"),parent),
    imagechooser(0),
    finder(0)
{
  this->setAcceptDrops(TRUE);
  dialoglayout=new QGridLayout;
  frontimage=new QLabel(tr("<center>No frontpicture</center>"),this);
  //frontimage=new QLabel(this);
  frontimage->setMinimumSize(120,120);
  backimage=new QLabel(tr("<center>No backpicture</center>"),this);
  //backimage=new QLabel(this);
  backimage->setMinimumSize(120,120);
  mediaimage=new QLabel(tr("<center>No mediapicture</center>"),this);
  //mediaimage=new QLabel(this);
  mediaimage->setMinimumSize(120,120);
  dialoglayout->addWidget(frontimage,0,0,4,1);
  //dialoglayout->addWidget(frontlabel,4,0);
  dialoglayout->addWidget(mediaimage,0,1,4,1);
  //dialoglayout->addWidget(medialabel,4,1);
  dialoglayout->addWidget(backimage,0,2,4,1);
  //dialoglayout->addWidget(backlabel,4,2);
  dialoglayout->setMargin(2);
  dialoglayout->setSpacing(2);
  loadbutton=new QPushButton(QIcon(":/icons/fileopen.png"),tr("&Load"),this);
  searchbutton=new QPushButton(QIcon(":/icons/internet.png"),tr("&Find"),this);
  searchbutton->setEnabled(FALSE);
  removebutton=new QPushButton(QIcon(":/icons/eraser.png"),tr("&Remove"),this);
  removebutton->setEnabled(FALSE);
  dialoglayout->addWidget(loadbutton,5,0);
  dialoglayout->addWidget(searchbutton,5,1);
  dialoglayout->addWidget(removebutton,5,2);
  this->setLayout(dialoglayout);
  this->adjustSize();
  connect (loadbutton,SIGNAL(clicked()),this,SLOT(loadslot()));
  connect (removebutton,SIGNAL(clicked()),this,SLOT(removeslot()));
  connect (searchbutton,SIGNAL(clicked()),this,SLOT(startsearch()));
}

coverbox::~coverbox()
{
  if (finder)
    delete finder;
  if (imagechooser)
    delete imagechooser;
}

void coverbox::setimage(const QImage newcover,int covertype)
{
  if (covertype==Coverart::FrontCover)
    {
      front=newcover.copy();
      QImage scaled=front.scaledToHeight(120);
      frontimage->setPixmap(QPixmap::fromImage(scaled));
      //frontlabel->setText(tr("<center>Frontpicture set</center>"));
    }
  else if (covertype==Coverart::BackCover)
    {
      back=newcover.copy();
      QImage scaled=back.scaledToHeight(120);
      backimage->setPixmap(QPixmap::fromImage(scaled));
      //backlabel->setText(tr("<center>Backpicture set</center>"));
    }
  else if (covertype==Coverart::Media)
    {
      media=newcover.copy();
      QImage scaled=media.scaledToHeight(120);
      mediaimage->setPixmap(QPixmap::fromImage(scaled));
      //medialabel->setText(tr("<center>Mediapicture set</center>"));
    }
  adjustSize();
  emit coverchange();
}

void coverbox::clearimages(bool supported)
{
  front=QImage();
  media=QImage();
  back=QImage();
  frontimage->clear();
  mediaimage->clear();
  backimage->clear();
  if (!supported)
    {
      frontimage->setText(tr("<center>Not supported</center>"));
      mediaimage->setText(tr("<center>Not supported</center>"));
      backimage->setText(tr("<center>Not supported</center>"));
    }
  else
    {
      frontimage->setText(tr("<center>No frontpicture</center>"));
      mediaimage->setText(tr("<center>No mediapicture</center>"));
      backimage->setText(tr("<center>No backpicture</center>"));
    }
}

void coverbox::dragEnterEvent(QDragEnterEvent *event)
{
  if (event->mimeData()->hasImage()||event->mimeData()->hasFormat("text/plain"))
    event->acceptProposedAction();
}

void coverbox::dropEvent(QDropEvent *event)
{
  QImage image;
  if (event->mimeData()->hasImage())
    {
      image=qvariant_cast<QImage>(event->mimeData()->imageData());
    }
  else if (event->mimeData()->hasFormat("text/plain"));
  {
    QString filename=event->mimeData()->text();
    filename=filename.remove("file://");
    if (filename.toLower().endsWith("gif")||filename.toLower().endsWith("jpg")||filename.toLower().endsWith("jpeg")||filename.toLower().endsWith("png")||filename.toLower().endsWith("bmp"))
      {
        image.load(filename);
      }
  }
  imagechooser=new coverchooser(this,image);
  connect (imagechooser,SIGNAL(selectedas(const int,const QImage &)),this,SLOT(setpicture(const int,const QImage &)));
  connect (imagechooser,SIGNAL(cancelsignal()),this,SLOT(deleteselector()));
}

void coverbox::setpicture(const int position,const QImage &cover)
{
  setimage(cover, position);
  if (imagechooser)
    {
      delete imagechooser;
      imagechooser=NULL;
    }
  //Check all pictures, and give free the removebutton,or disable it if none is set
  if (front.isNull()&&media.isNull()&&back.isNull())
    removebutton->setEnabled(FALSE);
  else
    removebutton->setEnabled(TRUE);
}

void coverbox::loadslot()
{
  QString filename = QFileDialog::getOpenFileName(this, tr("Select Picture"),QDir::homePath (),tr("Images (*.png *.xpm *.jpg *.gif *.bmp)"));
  QImage image;
  image.load(filename);
  if (!image.isNull())
    {
      imagechooser=new coverchooser(this,image);
      connect (imagechooser,SIGNAL(selectedas(const int,const QImage &)),this,SLOT(setpicture(const int,const QImage &)));
    }
}

void coverbox::removeslot()
{
  if (imagechooser)
    delete imagechooser;
  int setcovers=0;
  if (!front.isNull())
    setcovers+=1;
  if (!media.isNull())
    setcovers+=2;
  if (!back.isNull())
    setcovers+=4;
  imagechooser=new coverchooser(this,setcovers);
  connect (imagechooser,SIGNAL(selectedas(const int, const QImage&)),this,SLOT(removepicture(const int)));
  connect (imagechooser,SIGNAL(cancelsignal()),this,SLOT(stopremove()));
}

void coverbox::removepicture(const int position)
{
  if (position==Coverart::FrontCover)
    {
      front=QImage();
      frontimage->clear();
    }
  if (position==Coverart::Media)
    {
      media=QImage();
      mediaimage->clear();
    }
  if (position==Coverart::BackCover)
    {
      back=QImage();
      backimage->clear();
    }
  if (imagechooser)
    {
      delete imagechooser;
      imagechooser=NULL;
    }
  if (front.isNull()&&media.isNull()&&back.isNull())
    removebutton->setEnabled(FALSE);
  else
    removebutton->setEnabled(TRUE);
  emit coverchange();
}

void coverbox::stopremove()
{
  if (imagechooser)
    delete imagechooser;
  imagechooser=NULL;
}

void coverbox::deleteselector()
{
  if (imagechooser)
    {
      delete imagechooser;
    }
  imagechooser=NULL;
}


const QImage coverbox::getfrontimage()
{
  return front;
}

const QImage coverbox::getmediaimage()
{
  return media;
}

const QImage coverbox::getbackimage()
{
  return back;
}

void coverbox::startsearch()
{
  if (finder)
    delete finder;

  finder=new coverfinder(searchstring);
  connect (finder,SIGNAL(imagesignal(const int,const QImage)),this,SLOT(setpicture(const int,const QImage)));
  connect (finder,SIGNAL(cancelsignal()),this,SLOT(cancelfinder()));
}

void coverbox::setsearchstring(const QString string)
{
  searchstring=string;
  searchstring.replace(' ','+');
  if (searchstring.length()<2)
    searchbutton->setEnabled(FALSE);
  else
    searchbutton->setEnabled(TRUE);
}

void coverbox::cancelfinder()
{
  if (finder)
    {
      delete finder;
      finder=NULL;
    }
}
