/***************************************************************************
 *   Copyright (C) 2007 by Roland Weigert   *
 *   roweigert@t-online.de             *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

/**
 *@file coverbox.h
 *
 *This file holds the class for the coverbox<br>
 *It handles all events for coverpictures
 *@author Roland Weigert
 *@date 27-12-2007
 */

#ifndef _COVERBOX_H_
#define _COVERBOX_H_

#include <QGroupBox>
#include <QLayout>
#include <QLabel>
#include <QPushButton>
#include <QImage>
#include <QDropEvent>
#include <QStringList>
#include <QDialog>
#include <QFileDialog>
#include <QDir>
#include <QCloseEvent>

#include "global-stuff/globals.h"
#include "coverbox/coverfinder.h"


/**
*@brief This class is needed to choose the right position for an image
*/
class coverchooser : public QDialog
  {
    Q_OBJECT
  public:
    /**
    *@brief Constructor
    *@param cover The Image that needs positioninfo
    */
    coverchooser(QWidget *parent,const QImage &cover);
    /**
    *@brief Constructor
    *Second Constructor, that constructs a dialog for erasing of picturees
    */
    coverchooser(QWidget *parent,const int setpictures);
    /**
    *@brief Destructor
    */
  private:
    QVBoxLayout *layout;
    QHBoxLayout *buttonlayout;
    QLabel *coverlabel;
    QLabel *infotext;
    QPushButton *frontbutton;
    QPushButton *mediabutton;
    QPushButton *backbutton;
    QImage image;
  private slots:
    void frontslot();
    void mediaslot();
    void backslot();
    void closeEvent(QCloseEvent *event);
  signals:
    /**
    *@brief This Signal is emitted, when user has chosen a position for the picure
    *Sends the Picture and the pictureposition.
    */
    void selectedas(const int covertype,const QImage &cover);
    /**
    *@brief This signal is emitted, when user closes the dialog.
    */
    void cancelsignal();
  };

class coverbox : public QGroupBox
  {
    Q_OBJECT
  public:
    /**
    *@brief Constructor
    */
    coverbox(QWidget *parent);
    /**
    *@brief Destructor
    */
    ~coverbox();
    /**
    *@brief Get the frontimage,if one is set
    */
    const QImage getfrontimage();
    /**
    *@brief Get the mediaimage,if one is set
    */
    const QImage getmediaimage();
    /**
    *@brief Get the backimage,if one is set
    */
    const QImage getbackimage();
    /**
    *@brief The setterfunction for images
    *@param image The image you want to set
    *@param covertype The position of the cover
    */
    void setimage(const QImage image,int covertype);
    /**
    *@brief Clear all images out of this widget
    *@param supported Sets weather the widget should display picture not set or not supported
    */
    void clearimages(bool supported);
    /**
    *@brief Set the string that is used for searching by coverfinder
    *@param string The new searchstring (QString::null deactivates findbutton)
    */
    void setsearchstring(const QString string);
  private:
    void dragEnterEvent(QDragEnterEvent *event);
    void dropEvent(QDropEvent *event);
    QLabel *frontimage;
    QLabel *frontlabel;
    QLabel *mediaimage;
    QLabel *medialabel;
    QLabel *backimage;
    QLabel *backlabel;
    QPushButton *loadbutton;
    QPushButton *searchbutton;
    QPushButton *removebutton;
    QGridLayout *dialoglayout;
    QImage front;
    QImage media;
    QImage back;
    coverchooser *imagechooser;
    coverfinder *finder;
    QString searchstring;
  private slots:
    void removeslot();
    void removepicture(const int position);
    void stopremove();
    void setpicture(const int position,const QImage &cover);
    void loadslot();
    void deleteselector();
    void startsearch();
    void cancelfinder();
  signals:
    void coverchange();

  };

#endif
