/***************************************************************************
 *   Copyright (C) 2007 by Roland Weigert   *
 *   roweigert@t-online.de             *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include "coverfinder.h"

coverfinder::coverfinder(const QString text):
    searchstring(text),
    col(0),
    row(0)
{
  scroller=new QScrollArea(this);
  container=new QWidget(scroller);
  pictureslayout=new QGridLayout(scroller);
  container->setLayout(pictureslayout);
  layout= new QVBoxLayout(this);
  this->setWindowTitle(tr("Select matching Image."));
  layout->addWidget(scroller);
  buttonlayout=new QHBoxLayout;
  frontbutton=new QPushButton(tr("Frontcover"),this);
  mediabutton=new QPushButton(tr("Mediacover"),this);
  backbutton=new QPushButton(tr("Backcover"),this);
  frontbutton->setEnabled(FALSE);
  mediabutton->setEnabled(FALSE);
  backbutton->setEnabled(FALSE);
  cancelbutton=new QPushButton(QIcon(":/icons/cancel.png"),tr("Done"),this);
  buttonlayout->addWidget(frontbutton);
  buttonlayout->addWidget(mediabutton);
  buttonlayout->addWidget(backbutton);
  buttonlayout->addWidget(cancelbutton);
  layout->addLayout(buttonlayout);
  this->setLayout(layout);
  scroller->setWidget(container );
  scroller->setWidgetResizable( TRUE );
  this->setMinimumSize(600,400);
  this->show();
  fetchlist();
  connect (this,SIGNAL(gotnewimage(const QImage)),this,SLOT(appendnewimage(const QImage)));
  connect (frontbutton,SIGNAL(clicked()),this,SLOT(setfront()));
  connect (mediabutton,SIGNAL(clicked()),this,SLOT(setmedia()));
  connect (backbutton,SIGNAL(clicked()),this,SLOT(setback()));
  connect (cancelbutton,SIGNAL(clicked()),this,SIGNAL(cancelsignal()));
}

coverfinder::~coverfinder()
{
  foundcovers.clear();
}

void coverfinder::closeEvent(QCloseEvent *event)
{
  emit cancelsignal();
  event->accept();
}

void coverfinder::fetchlist()
{
  runningqueries.clear();
  foundcovers.clear();
  QHttp *googlequery=new QHttp(this);
  googlequery->setHost("images.google.com",80);
  QString querytext=QString("/images?hl=en&q=%1&btnG=Search+Images&gbv=1&").arg(searchstring);
  linkitem newlink;
  newlink.url=googlequery;
  newlink.id=googlequery->get(querytext);
  runningqueries.append(newlink);
  connect (googlequery,SIGNAL(requestFinished(int,bool)),this,SLOT(parselinks(int,bool)));
}

void coverfinder::parselinks(int id,bool error)
{
//Find the query with read-state, get the data, than delete it
  bool found=FALSE;
  QList<linkitem>::Iterator querywalkingpointer=runningqueries.begin();
  while (querywalkingpointer!=runningqueries.end()&&!found)
    {
      if ((*querywalkingpointer).id==id)
        found=TRUE;
      else
        querywalkingpointer++;
    }
  if (found)
    {
      if (!error)
        {
          QByteArray receiveddata=(*querywalkingpointer).url->readAll();
          QString result=QString::fromUtf8(receiveddata);
          runningqueries.erase(querywalkingpointer);
          while (result.contains("imgurl"))
            {
              result=result.right(result.length()-result.indexOf("imgurl"));
              int position=result.indexOf("&");
              QString url=result.left(position);
              url=url.remove("imgurl=http://");
              //Ok, lets set up a new imagequery, and sit htere waiting for the image
              int slashposition=url.indexOf("/");
              QString server=url.left(slashposition);
              QString link=url.right(url.length()-slashposition);
              QHttp *imagequery=new QHttp(this);
              imagequery->setHost(server,80);
              QString querytext=QString(link);
              linkitem newlink;
              newlink.url=imagequery;
              newlink.id=imagequery->get(querytext);
              runningqueries.append(newlink);
              connect (imagequery,SIGNAL(requestFinished(int,bool)),this,SLOT(catchimage(int,bool)));
              result=result.right(result.length()-position);
            }
        }
      else
        {
          emit queryerror(tr("something went wrong with the google-query!"));
          runningqueries.erase(querywalkingpointer);
        }
    }
}

void coverfinder::catchimage(int id,bool error)
{
//Find the query with read-state, get the data, than delete it
  bool found=FALSE;
  QList<linkitem>::Iterator querywalkingpointer=runningqueries.begin();
  while (querywalkingpointer!=runningqueries.end()&&!found)
    {
      if ((*querywalkingpointer).id==id)
        found=TRUE;
      else
        querywalkingpointer++;
    }
  if (found)
    {
      QByteArray receiveddata=(*querywalkingpointer).url->readAll();
      runningqueries.erase(querywalkingpointer);
      QImage imagebuffer;
      if (imagebuffer.loadFromData(receiveddata))
        {
          emit gotnewimage(imagebuffer);
        }
    }
}

void coverfinder::appendnewimage(const QImage image)
{
  coverobject *newcover=new coverobject(image);
  connect (newcover,SIGNAL(selected(coverobject *)),this,SLOT(deselctothers(coverobject *)));
  foundcovers.append(newcover);
  pictureslayout->addWidget(newcover,row,col);
  col++;
  if (col==2)
    {
      col=0;
      row++;
    }
}

void coverfinder::setfront()
{
  QList<coverobject *>::Iterator picturewalkingpointer=foundcovers.begin();
  bool found=FALSE;
  while (!found&&picturewalkingpointer!=foundcovers.end())
    {
      if ((*picturewalkingpointer)->ischecked())
        {
          emit imagesignal(Coverart::FrontCover,(*picturewalkingpointer)->getimage());
          found=TRUE;
        }
      else
        picturewalkingpointer++;
    }
}

void coverfinder::setmedia()
{
  QList<coverobject *>::Iterator picturewalkingpointer=foundcovers.begin();
  bool found=FALSE;
  while (!found&&picturewalkingpointer!=foundcovers.end())
    {
      if ((*picturewalkingpointer)->ischecked())
        {
          emit imagesignal(Coverart::Media,(*picturewalkingpointer)->getimage());
          found=TRUE;
        }
      else
        picturewalkingpointer++;
    }
}

void coverfinder::setback()
{
  QList<coverobject *>::Iterator picturewalkingpointer=foundcovers.begin();
  bool found=FALSE;
  while (!found&&picturewalkingpointer!=foundcovers.end())
    {
      if ((*picturewalkingpointer)->ischecked())
        {
          emit imagesignal(Coverart::BackCover,(*picturewalkingpointer)->getimage());
          found=TRUE;
        }
      else
        picturewalkingpointer++;
    }
}

void coverfinder::deselctothers(coverobject *selectedobject)
{
  frontbutton->setEnabled(TRUE);
  mediabutton->setEnabled(TRUE);
  backbutton->setEnabled(TRUE);
  QList<coverobject *>::Iterator picturewalkingpointer=foundcovers.begin();
  while (picturewalkingpointer!=foundcovers.end())
    {
      if (selectedobject!=(*picturewalkingpointer))
        {
          (*picturewalkingpointer)->deselect();
        }
      picturewalkingpointer++;
    }
}

coverobject::coverobject(const QImage &cover):originalpic(cover.copy())
{
  layout=new QHBoxLayout;
  leftlayout=new QVBoxLayout;
  sizelabel=new QLabel(QString::number(originalpic.width())+"x"+QString::number(originalpic.height()));
  check=new QCheckBox(tr("Select"));
  picturelabel=new QLabel;
  QImage scaled=originalpic.scaledToWidth(100);
  picturelabel->setPixmap(QPixmap::fromImage(scaled));
  leftlayout->addWidget(sizelabel);
  leftlayout->addWidget(check);
  layout->addLayout(leftlayout);
  layout->addWidget(picturelabel);
  this->setLayout(layout);
  connect (check,SIGNAL(clicked()),this,SLOT(select()));
}

coverobject::~coverobject()
{
  delete picturelabel;
  delete sizelabel;
  delete check;
  delete leftlayout;
  delete layout;
}

void coverobject::select()
{
  this->check->setChecked(TRUE);
  emit selected(this);
}

void coverobject::deselect()
{
  this->check->setChecked(FALSE);
}

const QImage & coverobject::getimage()
{
  return originalpic;
}

const bool coverobject::ischecked() const
  {
    return check->isChecked();
  }

