/***************************************************************************
 *   Copyright (C) 2007 by Roland Weigert   *
 *   roweigert@t-online.de             *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

/**
 *@file coverfinder.h
 *
 *This file holds the class that searches covers in Internet
 *@author Roland Weigert
 *@date 18-11-2007
 */

#ifndef _COVERFINDR_H_
#define _COVERFINDR_H_

#include <QString>
#include <QDialog>
#include <QScrollArea>
#include <QLabel>
#include <QLayout>
#include <QPushButton>
#include <QCheckBox>
#include <QGroupBox>
#include <QImage>
#include <QHttp>
#include <QList>
#include <QCloseEvent>

#include "global-stuff/globals.h"

//Prototypes
class coverobject;
struct linkitem;

class coverfinder : public QWidget
  {
    Q_OBJECT
  public:
    coverfinder(QString searchkeywords);
    ~coverfinder();
  private:
    QScrollArea *scroller;
    QWidget *container;
    QVBoxLayout *layout;
    QGridLayout *pictureslayout;
    QHBoxLayout *buttonlayout;
    QPushButton *frontbutton;
    QPushButton *mediabutton;
    QPushButton *backbutton;
    QPushButton *cancelbutton;
    QList<linkitem> runningqueries;
    QList<coverobject *> foundcovers;
    QString searchstring;
    int col,row;
  private slots:
    void parselinks(int,bool);
    void catchimage(int,bool);
    void appendnewimage(const QImage);
    void setfront();
    void setmedia();
    void setback();
    void deselctothers(coverobject *);
    void closeEvent(QCloseEvent *event);
  private:
    void fetchlist();
  signals:
    void queryerror(const QString errortext);
    void gotnewimage(const QImage image);
    void imagesignal(const int position,const QImage);
    void cancelsignal();
  };

class coverobject : public QGroupBox
  {
    Q_OBJECT
  public:
    coverobject(const QImage &);
    ~coverobject();
    const QImage & getimage();
    const bool ischecked() const;
  private:
    QHBoxLayout *layout;
    QVBoxLayout *leftlayout;
    QImage originalpic;
    QLabel *picturelabel;
    QLabel *sizelabel;
    QCheckBox *check;
    private slots:
    void select();
    public slots:
    void deselect();
    signals:
    void selected(coverobject *);
  };

struct linkitem
  {
    QHttp *url;
    int id;
  };

#endif
