//
// C++ Interface: mysqlinterface
//
// Description:
//
//
// Author: Roland Weigert <roweigert@t-online.de>, (C) 2008
//
// Copyright: See COPYING file that comes with this distribution
//
//

/**
 * @file mysqlinterface.h
 *
 * This file holds the class for the mysql-interface.
 * It is a direct derivate from databaseinterface, so it gets all its API-calls.
 * Also it is directly cotrolled over a databasethread. Make no changes to the API,
 * otherwise the compatibility with the dbtread-class will break.
 * @author Roland Weigert
 * @date 03-10-2008
 * @todo Write all
 */

#ifndef _MYSQLINTERFACE_H_
#define _MYSQLINTERFACE_H_

#include <QObject>
#include <QString>
#include <QStringList>
#include <QList>
#include <QSqlDatabase>

#include "databaseinterface.h"

class mysqlinterface : public databaseinterface
  {
    Q_OBJECT
  public:
    mysqlinterface(QString &dbname, QString &user, QString &pass);
    ~mysqlinterface();
  public slots:
    virtual void checkdbinsert(const dbtrackdescription curtrack){};
    /**
     * @brief getdbcover
     * Catch a coverfile from db. The id is from a prior dblookup.<br>
     * Connect to the gotdbcover signal to get the picture back from the db.<br>
     * If something goes wrong, or a non existing id is given the signal dbcovererror is emitted.
     * @param id The id of the cover
     */
    virtual void getdbcover(const long id){};
    /**
     * @brief getdbbytracknumberslot
     * Find out the albumid of the current track given by the information in curtrack.
     * Than fetch the Track from the db that has right albumid and tracknumber;
     * Connect to gotdbartistandtrack-signal to get the result.
     * @param tracknumber The number of the searched track
     * @param track The dbtrackdescription of the current track
     */
    virtual void getdbbytracknumber(const int tracknumber,const dbtrackdescription track){};
    virtual void getdbbytrackname (const QString trackname, const dbtrackdescription track){};
    virtual void getdbbyartistname(const QString artistname, const dbtrackdescription track){};
    virtual void getdbbyalbumname(const QString albumname, const dbtrackdescription track){};
    virtual void filldbartists(QCompleter *artists){};
    virtual void fillbalbums(QCompleter *albums){};
    virtual void filldbtracks(QCompleter *tracks){};
    virtual void filldbgenres(QCompleter *genres){};
    /**
     * @brief checkdbfileinsert
     * Checks if a file is already known in the Files table.<br>
     * If The file is not yet in the table gotdbfileisnew-signal is emitted.<br>
     * You can call dbfileinsert than with this filename, and the read out tags.<br>
     * If the file is already known, but the modificationtime is newer than in the db the signal gotdbfileneedsupdate is emitted.<br>
     * You can call dbfileupdate than.<br>
     * @param filename The filename to check
     * @param modified The Modificationdate of the file in the Format YYYY-MM-DD HH:MM:SS
     */
    virtual void checkdbfileinsert( const QString filename,const QString modified){};
    /**
     * @brief dbfileinsert
     * Inserts a file into the database that is not yet in it.<br>
     * The QStringList insertslist is checked, before inserting, so it makes no sense to call this slot without checking if the file is in the db with checkdbfileinsert before.<br>
     * @param filename The filename to insert
     * @param tags The tags of the file
     */
    virtual void dbfileinsert(const QString filename,dbtrackdescription tags){};
    /**
     * @brief dbfileupdate
     * Updates a file in the database that has a newer timestamp than in db.<br>
     * The QStringList updatelist is checked, before updating, so it makes no sense to call this slot without checking if the file is in the db with checkdbfileinsert before.<br>
     * @param filename The filename to update
     * @param tags The tags of the file
     */
    virtual void dbfileupdate(const QString filename,dbtrackdescription tags){};
    /**
     * @brief Check if the db is already started and connected.
     * You can start a db with startdb, if it is not already running.
     */
    virtual bool isdbconnected(){};
  private:
    /**
    * @brief Start a not running db
    * Give username and pass to start the db.
    * The db will be generated with the given userdata, if it is not existing already.
    * The user is asked for a db-pass with create-abilities therefor.
    */
    virtual void startdb(QString &dbname, QString &user, QString &pass){};
    QSqlDatabase *db;

  };

#endif
