/***************************************************************************
 *   Copyright (C) 2007 by Roland Weigert   *
 *   roweigert@t-online.de             *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include "freedbinterface.h"

freedbinterface::freedbinterface(QString server):
    servername(server),
    connection(new QHttp),
    tracks(0)

{
  connection->setHost(servername,80);
}

freedbinterface::~freedbinterface()
{
  delete connection;
}

void freedbinterface::getservers()
{
  QString freedbquery=QString("/~cddb/cddb.cgi/?cmd=sites&hello=%1+localhost+%2+%3&proto=6").arg(USERNAME,APPNAME,VERSION);
  connection->get(freedbquery);
  bufferlist.clear();
  connect (connection,SIGNAL(done(bool)),this,SLOT(parseservers(bool)));
}

void freedbinterface::getdisc(cdinterface &discinformation,const QString freedb_discid)
{
  disc=discinformation.gettoc();
  tracks=discinformation.gettracks();
  discid=freedb_discid;
  QString freedbquery=QString("cddb query %1 %2").arg(discid,QString::number(tracks));
  int counter=0;
  while (counter<=tracks-1)
    {
      freedbquery=QString("%1 %2")
                  .arg(freedbquery,QString::number(disc[counter].frameoffset));
      counter++;
    }
  int discseconds=(disc[tracks].frameoffset-disc[0].frameoffset)/75;
  freedbquery=QString("%1 %2").arg(freedbquery,QString::number(discseconds));
  freedbquery=QString("/~cddb/cddb.cgi/?cmd=%1").arg(freedbquery);
  freedbquery=QString("%1&hello=%2+localhost+%3+%4&proto=6").arg(freedbquery,USERNAME,APPNAME,VERSION);
  freedbquery.replace(' ','+');
  bufferlist.clear();
  genrelist.clear();
  connection->get(freedbquery);
  connect (connection,SIGNAL(done(bool)),this,SLOT(parsediscs(bool)));
}

void freedbinterface::abortfetch()
{}

void freedbinterface::parseservers(bool errorstatus)
{
  disconnect (connection,SIGNAL(done(bool)),this,SLOT(parseservers(bool)));
  QStringList serverlist;
  if (!errorstatus)
    {
      QByteArray receiveddata=connection->readAll();
      bufferlist=chopresult(receiveddata);
      QStringList::ConstIterator walkingpointer=bufferlist.begin();
      while (walkingpointer!=bufferlist.end())
        {
          if ((*walkingpointer).contains("http"))
            {
              QString newserver=(*walkingpointer).left((*walkingpointer).indexOf("http"));
              newserver.remove(' ');
              serverlist.append(newserver);
            }
          walkingpointer++;
        }
      emit gotserverlist(serverlist);
    }
}

void freedbinterface::parsediscs(bool errorstatus)
{
  disconnect (connection,SIGNAL(done(bool)),this,SLOT(parsediscs(bool)));
  if (!errorstatus&&bufferlist.isEmpty())
    {
      QByteArray receiveddata=connection->readAll();
      //Parse out all information on first result, and fetch the corresponding discs one by one
      if (bufferlist.isEmpty())
        {
          bufferlist=chopresult(receiveddata);
          QStringList::ConstIterator walkingpointer=bufferlist.begin();
          while (walkingpointer!=bufferlist.end())
            {
              if ((*walkingpointer).contains(discid))
                {
                  QString newgenre=(*walkingpointer).left((*walkingpointer).indexOf(discid));
                  //Single findings have a different format, so we have to remove the 200 in front
                  if (newgenre.startsWith("202"))
                    {
                      emit queryerror(tr("No result from Freedb"));
                    }
                  else
                    {
                      if (newgenre.startsWith("200"))
                        newgenre.remove("200");
                      newgenre.remove(' ');
                      freedbdiscquery newquery;
                      newquery.discid=discid.trimmed();
                      newquery.genre=newgenre.trimmed();
                      if (!newquery.discid.isEmpty()&&!newquery.genre.isEmpty())
                        genrelist.append(newquery);
                    }
                }
              else if (bufferlist.first().startsWith("211")&&(*walkingpointer).contains('/'))
                {
                  //Landed here? Should be inexact matches
                  QString newgenre=(*walkingpointer).left((*walkingpointer).indexOf(' '));
                  QString newdiscid=(*walkingpointer).right((*walkingpointer).length()-((*walkingpointer).indexOf(' ')+1));
                  newdiscid=newdiscid.left(newdiscid.indexOf(' '));
                  freedbdiscquery newquery;
                  newquery.discid=newdiscid.trimmed();
                  newquery.genre=newgenre.trimmed();
                  if (!newquery.discid.isEmpty()&&!newquery.genre.isEmpty())
                    genrelist.append(newquery);
                }
              walkingpointer++;
            }
          if (genrelist.count())
            {
              genrewalkingpointer=genrelist.begin();
              QString querystring=QString("/~cddb/cddb.cgi/?cmd=cddb+read+%1+%2&hello=%3+localhost").arg((*genrewalkingpointer).genre,(*genrewalkingpointer).discid,USERNAME);
              querystring=QString("%1+%2+%3&proto=6").arg(querystring,APPNAME,VERSION);
              connection->get(querystring);
              connect (connection,SIGNAL(done(bool)),this,SLOT(parsedisc(bool)));
            }
        }
    }
  else if (!genrelist.isEmpty())
    {
      genrewalkingpointer++;
      if (genrewalkingpointer!=genrelist.end())
        {
          //Pull out the next genre from list, if there is still one to go
          QString querystring=QString("/~cddb/cddb.cgi/?cmd=cddb+read+%1+%2&hello=%3+localhost").arg((*genrewalkingpointer).genre,(*genrewalkingpointer).discid,USERNAME);
          querystring=QString("%1+%2+%3&proto=6").arg(querystring,APPNAME,VERSION);
          connection->get(querystring);
          connect (connection,SIGNAL(done(bool)),this,SLOT(parsedisc(bool)));
        }
      else
        emit  gotdisc (albumresult);
    }
}

void freedbinterface::parsedisc(bool errorstatus)
{
  disconnect (connection,SIGNAL(done(bool)),this,SLOT(parsedisc(bool)));
  if (!errorstatus)
    {
      bufferlist.clear();
      QByteArray receiveddata=connection->readAll();
      bufferlist=chopresult(receiveddata);
      //Check through all lines in buffer, if we have a disc length, than append a new disc for the return list
      freedbalbum newalbum;
      QStringList::ConstIterator walkingpointer=bufferlist.begin();
      bool discfound=FALSE;
      while (walkingpointer!=bufferlist.end())
        {
          if ((*walkingpointer).contains("Disc length"))
            discfound=TRUE;
          else if ((*walkingpointer).startsWith("DTITLE="))
            {
              QString buffer=(*walkingpointer);
              buffer.remove("DTITLE=");
              newalbum.fdb_artist=(buffer.left(buffer.indexOf("/"))).trimmed();
              newalbum.fdb_album=(buffer.right(buffer.length()-(buffer.indexOf("/")+1))).trimmed();
            }
          else if ((*walkingpointer).startsWith("DGENRE="))
            {
              QString buffer=(*walkingpointer);
              newalbum.fdb_genre=buffer.remove("DGENRE=").trimmed();
            }
          else if ((*walkingpointer).startsWith("DYEAR="))
            {
              QString buffer=(*walkingpointer);
              newalbum.fdb_release=buffer.remove("DYEAR=").toInt();
            }
          else if ((*walkingpointer).startsWith("DISCID="))
            {
              QString buffer=(*walkingpointer);
              newalbum.fdb_discid=buffer.remove("DISCID=").trimmed();
            }
          else if ((*walkingpointer).startsWith("TTITLE"))
            {
              QString buffer=(*walkingpointer);
              buffer.remove("TTITLE");
              freedbtrack newtrack;
              newtrack.fdb_number=(buffer.left(buffer.indexOf("="))).toInt()+1;
              newtrack.fdb_title=(buffer.right(buffer.length()-(buffer.indexOf("=")+1))).trimmed();
              newalbum.fdb_tracklist.append(newtrack);
            }
          walkingpointer++;
        }
      if (discfound)
        albumresult.append(newalbum);
    }
  parsediscs(FALSE);
}

const QStringList freedbinterface::chopresult(const QByteArray &convertarray)
{
  QStringList lines;
  QString buffer;
  //find new lines, and make Qstrings with them
  QByteArray copyarray=convertarray;
  while (copyarray.contains('\n'))
    {
      QByteArray bufferarray=copyarray.left(copyarray.indexOf('\n'));
      copyarray=copyarray.remove(0,copyarray.indexOf('\n')+1);
      buffer=QString::fromUtf8(bufferarray.data());
      lines.append(buffer);
      //qDebug("This is the result from freedb : %s",qPrintable(buffer));
    }
  return lines;
}
