//
// C++ Interface: communicationinterface
//
// Description:
//
//
// Author: Roland Weigert <roweigert@t-online.de>, (C) 2009
//
// Copyright: See COPYING file that comes with this distribution
//
//

/**
 * @file communicationinterface.h
 *
 * This file holds the class for all appinternal communication.<br>
 * This way the signals dont need to pass through several classes, to go to their destination.
 * To be flexible enough all data is passed as QVariant with two QStrings holding the sender and the type.<br>
 * For example the encoder sends that it needs a new progresswidget for track 03
 * @author Roland Weigert
 * @date 23-02-2009
 * @todo Write all
 */

#ifndef _COMMUNICATIONINTERFACE_H_
#define _COMMUNICATIONINTERFACE_H_

#include <QObject>
#include <QVariant>
#include <QString>
#include <QList>
#include <QStringList>
#include <QMutex>

#include "globals.h"


/*
To solve the  problem with other treads, we have to lock the data into a mutex.
This means, that the data cant be sent within the signals. It needs to be fetched.
At the start of the fetch the mutex for the data must be locked.
Otherwise it always leads into problems, because more than one thread tries to access the same data at the same time.
-->Programm-shutdown, or strange behaviour.
*/

//prototypedeclaration;
class messageobject;

/**
 *@brief communicationinterface-class
 *
 */
class communicationinterface : public QObject
  {
    Q_OBJECT
//The class is empty, cause we only need the pass through of signals
  signals:
    void requestsignal(QString sender,QString type, QVariant);
    void requestsignal(QString sender,QString type, QList<QVariant>);
  public slots:
    void requestslot(QString sender,QString type, QVariant);
    void requestslot(QString sender,QString type, QList<QVariant>);
  };

class advancedcommunicationinterface : public QObject
  {
    Q_OBJECT
  public:
    advancedcommunicationinterface();
    ~advancedcommunicationinterface();
    void sendmessage(messageobject *object);
    void deletemessage(const long id);
  private:
    long currentid;
    QList<messageobject *> messagelist;
  signals:
    void requestsignal(QStringList destination,messageobject *);
  };

class messageobject : public QMutex
  {
  public:
    messageobject(const QString sender, const QStringList destination, QString type, QVariant data);
    void setmessageid(const long id);
    const long getmessageid();
    const QString getsender();
    const QStringList getdestination();
    const QString gettype();
    const QVariant getdata();
    const int confirm(const QString confirmer);
  private:
    QString sender;
    QStringList destination;
    QString type;
    QVariant data;
    long id;
  };

/**
 * Since we need some custom structures to transfer all kind of communication, we declare them here, and tell Qt that there are new types, that QVariant needs to handle. <br>
 * If we come in need to transfer a new kind of data, we can declare a new type here, than we can use it everywhere in the app, if this header is included.
 */

struct comm_progress
  {
    int track;
    int value;
    QString filename;//Use this only on finished-events. Let this empty, if you don' t need it.
  };
Q_DECLARE_METATYPE(comm_progress)
#endif
