/***************************************************************************
 *   Copyright (C) 2007 by Roland Weigert   *
 *   roweigert@t-online.de             *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include "fileoperator.h"

extern communicationinterface *comm_if;
extern advancedcommunicationinterface *adv_comm_if;
extern rcfile *settingsfile;

fileoperator::fileoperator(const int mode,const int overwritemode,const QStringList list,const QString basedirectory,const QStringList list2 ):
    deletemode(mode),
    ovmode(overwritemode),
    ignorelist(list),
    taskrunning(FALSE),
    basedir(basedirectory),
    partcount(0),
    filterworks(FALSE),
    nodeletelist(list2)
{
  connect (this,SIGNAL(filedone(const QString)),this,SLOT(nextjob(const QString)));
  connect(this,SIGNAL(nottagged(const QString)),this,SLOT(nottaggedslot(const QString)));
  connect(this,SIGNAL(nowrite(const QString, const QString)),this,SLOT(nowriteslot(const QString, const QString)));
  connect(this,SIGNAL(nodelete(const QString)),this,SLOT(nodeleteslot(const QString)));
}

fileoperator::~fileoperator()
{}

void fileoperator::setdeletemode(const int mode)
{
  deletemode=mode;
}

void fileoperator::setovmode(const int mode)
{
  ovmode=mode;
}

void fileoperator::setignorelist(const QStringList list)
{
  for(int i=0;i<list.count();i++)
  {
	  ignorelist.append(list.at(i));
	  qDebug("element in list: %s",qPrintable(list.at(i)));
  }
  qDebug("Crash in fileoperator");
  //ignorelist.clear();
  //ignorelist<<list;
}

void fileoperator::setbasedir(const QString directory)
{
  basedir=directory;
}

void fileoperator::deletefile(const QString filename)
{
  struct fileentry newentry=
    {
      filename,QString::null,fileoperations::remove,fileoperations::waiting
    };
  locklist.lock();
  joblist.append(newentry);
  locklist.unlock();
  if (!taskrunning)
    startjob();
}

void fileoperator::copyfile(const QString source, const QString destination)
{
  struct fileentry newentry=
    {
      source,destination,fileoperations::copyname,fileoperations::waiting
    };
  locklist.lock();
  joblist.append(newentry);
  locklist.unlock();
  if (!taskrunning)
    startjob();
}

void fileoperator::movefile(const QString source, const QString destination)
{
  struct fileentry newentry=
    {
      source,destination,fileoperations::movename,fileoperations::waiting
    };
  locklist.lock();
  joblist.append(newentry);
  locklist.unlock();
  if (!taskrunning)
    startjob();
}

void fileoperator::copyfile(const QString source)
{
  struct fileentry newentry=
    {
      source,QString::null,fileoperations::copytag,fileoperations::waiting
    };
  locklist.lock();
  joblist.append(newentry);
  locklist.unlock();
  if (!taskrunning)
    startjob();
}

void fileoperator::movefile(const QString source)
{
  struct fileentry newentry=
    {
      source,QString::null,fileoperations::movetag,fileoperations::waiting
    };
  locklist.lock();
  joblist.append(newentry);
  locklist.unlock();
  if (!taskrunning)
    startjob();
}

void fileoperator::retry(const QString filename)
{
  //Find the right job,and restart it
  locklist.lock();
  QList<fileentry>::Iterator jobindex=joblist.begin();
  bool found=false;
  fileentry actualjob;
  while (jobindex!=joblist.end()&&!found)
    {
      if (!filename.compare((*jobindex).source))
        {
          actualjob=*jobindex;
          found=true;
        }
      else
        jobindex++;
    }
  locklist.unlock();
  if (!actualjob.source.isNull())
    processjob(actualjob);
}

void fileoperator::resume(const QString filename)
{
  //Find the right job,and resume after it
  locklist.lock();
  QList<fileentry>::Iterator jobindex=joblist.begin();
  bool found=false;
  fileentry actualjob;
  while (jobindex!=joblist.end()&&!found)
    {
      if (!filename.compare((*jobindex).source))
        {
          jobindex=joblist.erase(jobindex);
          found=true;
          if (jobindex!=joblist.end()&&(*jobindex).status!=fileoperations::done)
            actualjob=*jobindex;
          else
            taskrunning=FALSE;
        }
      else
        jobindex++;
    }
  locklist.unlock();
  if (!actualjob.source.isNull())
    processjob(actualjob);
}

void fileoperator::startjob()
{
//This function is called, when no job is running atm, it starts the first job.
  locklist.lock();
  QList<fileentry>::Iterator jobindex=joblist.begin();
  locklist.unlock();
  if (jobindex!=joblist.end())
    processjob(*jobindex);
}

void fileoperator::nextjob(const QString readyfile)
{
  locklist.lock();
  QList<fileentry>::Iterator jobindex=joblist.begin();
  bool found=FALSE;
  while (jobindex!=joblist.end()&&!found)
    {
      if (!readyfile.compare((*jobindex).source))
        {
          found=TRUE;
          jobindex=joblist.erase(jobindex);
          if (jobindex!=joblist.end())
            {
              if ((*jobindex).status!=fileoperations::done)
                processjob(*jobindex);
            }
          else
            taskrunning=FALSE;
          jobindex=joblist.end();
        }
      else jobindex++;
    }
  locklist.unlock();
}

void fileoperator::processjob(fileentry &job)
{
  actualjob=&job;
  run();//Calls the run Method, so the Thread gets started
}

void fileoperator::run()
{
  QStringList receivelist;
  receivelist<<"main";
  messageobject *message=new messageobject("fileoperator", receivelist, QString("fileprogress"), QVariant (actualjob->source));
  adv_comm_if->sendmessage(message);
  taskrunning=TRUE;
  switch (actualjob->jobtype)
    {
    case fileoperations::remove:
    {
      QFileInfo filetodelete(actualjob->source);
      if (filetodelete.exists())
        {
          if (filetodelete.isWritable())
            {
              if (filetodelete.isDir())
                {
                  deleterecursivedir(actualjob->source);
                }
              else
                {
                  if (QFile::remove(actualjob->source))
                    {
                      qDebug("Deleting successfull");
                    }
                  else
                    qDebug("Delete file not successfull");
                }
              if (deletemode!=fileoperations::fileonly)
                {
                  //Extract the path out of the file, and start the directorycleaner
                  QDir motherdir=filetodelete.dir();
                  deletedir(motherdir.absolutePath());
                }
              emit filedone(actualjob->source);
            }
          else
            {
              actualjob->status=fileoperations::failed;
              emit nodelete(actualjob->source);
            }
        }
      else
        {
          //File seems already deleteted, by someone else,so emit fildeone-signal, to continue with next job
          emit filedone(actualjob->source);
        }
      break;
    }
    case fileoperations::copyname:
    {
      if (actualjob->source.compare(actualjob->destination))
        {
          //Only if the name changes, do the operations
          QFileInfo checkname(actualjob->destination);
          if (checkname.exists())
            {
              ovcheck(actualjob->source,actualjob->destination);
            }
          else
            {
              bool ok=QFile::copy(actualjob->source,actualjob->destination);
              if (ok)
                emit filedone(actualjob->source);
              else
                {
                  actualjob->status=fileoperations::failed;
                  emit nowrite(actualjob->source,actualjob->destination);
                }
            }
        }
      else
        emit filedone(actualjob->source);
      break;
    }
    case fileoperations::copytag:
    {
      actualjob->destination=generatenameandpath(actualjob->source);
      if (!actualjob->destination.isNull())
        {
          //looks good, we have created the path, and the filename
          if (actualjob->source.compare(actualjob->destination))
            {
              //Only if the name changes, do the operations
              QFileInfo checkname(actualjob->destination);
              if (checkname.exists())
                {
                  bool written=FALSE;
                  written=ovcheck(actualjob->source,actualjob->destination);
                  if (!written)
                    {
                      actualjob->status=fileoperations::failed;
                      emit nowrite(actualjob->source,actualjob->destination);
                    }
                }
              else
                {
                  bool ok=QFile::copy(actualjob->source,actualjob->destination);
                  if (ok)
                    emit filedone(actualjob->source);
                  else
                    {
                      actualjob->status=fileoperations::failed;
                      emit nowrite(actualjob->source,actualjob->destination);
                    }
                }
            }
          else
            emit filedone(actualjob->source);
        }
      else
        {
          actualjob->status=fileoperations::failed;
          emit nottagged(actualjob->source);
        }
      break;
    }
    case fileoperations::movename:
    {
      QFileInfo checkname(actualjob->destination);
      if (checkname.exists())
        {
          bool written=FALSE;
          written=ovcheck(actualjob->source,actualjob->destination);
          if (!written)
            {
              actualjob->status=fileoperations::failed;
              emit nowrite(actualjob->source,actualjob->destination);
            }
          else
            {
              if (QFile::remove(actualjob->source))
                {
                  emit filedone(actualjob->source);
                }
              else
                emit nodelete(actualjob->source);
            }
        }
      else
        {
          QDir dircheck(actualjob->source);
          if (dircheck.rename(actualjob->source,actualjob->destination))
            {
              emit filedone(actualjob->source);
            }
          else
            {
              actualjob->status=fileoperations::failed;
              QFileInfo checkname(actualjob->source);
              if (checkname.exists()&&!checkname.isWritable())
                emit nodelete(actualjob->source);
              else
                emit nowrite(actualjob->source,actualjob->destination);
            }
        }
      break;
    }
    case fileoperations::movetag:
    {
      actualjob->destination=generatenameandpath(actualjob->source);
      if (!actualjob->destination.isNull())
        {
          //looks good, we have created the path, and the filename
          if (actualjob->source.compare(actualjob->destination))
            {
              //Only if the name changes, do the operations
              QFileInfo checkname(actualjob->destination);
              if (checkname.exists())
                {
                  bool written=FALSE;
                  written=ovcheck(actualjob->source,actualjob->destination);
                  if (!written)
                    {
                      actualjob->status=fileoperations::failed;
                      emit nowrite(actualjob->source,actualjob->destination);
                    }
                  else
                    {
                      //Now remove the original if possible
                      QFileInfo filetodelete(actualjob->source);
                      if (filetodelete.exists())
                        {
                          if (filetodelete.isWritable())
                            {
                              QFile::remove(actualjob->source);
                              if (deletemode!=fileoperations::fileonly)
                                {
                                  //Extract the path out of the file, and start the directorycleaner
                                  QDir motherdir=filetodelete.dir();
                                  deletedir(motherdir.absolutePath());
                                }
                              emit filedone(actualjob->source);
                            }
                          else
                            {
                              actualjob->status=fileoperations::failed;
                              emit nodelete(actualjob->source);
                            }
                        }
                      else
                        {
                          //File seems already deleteted, by someone else,so emit fildeone-signal, to continue with next job
                          emit filedone(actualjob->source);
                        }
                    }
                }
              else
                {
                  bool ok=QFile::copy(actualjob->source,actualjob->destination);
                  if (ok)
                    {
                      //Now first remove the original
                      QFileInfo filetodelete(actualjob->source);
                      if (filetodelete.exists())
                        {
                          if (filetodelete.isWritable())
                            {
                              QFile::remove(actualjob->source);
                              if (deletemode!=fileoperations::fileonly)
                                {
                                  //Extract the path out of the file, and start the directorycleaner
                                  QDir motherdir=filetodelete.dir();
                                  deletedir(motherdir.absolutePath());
                                }
                              emit filedone(actualjob->source);
                            }
                          else
                            {
                              actualjob->status=fileoperations::failed;
                              emit nodelete(actualjob->source);
                            }
                        }
                      else
                        {
                          //File seems already deleteted, by someone else,so emit fildeone-signal, to continue with next job
                          emit filedone(actualjob->source);
                        }
                    }
                  else
                    {
                      actualjob->status=fileoperations::failed;
                      emit nowrite(actualjob->source,actualjob->destination);
                    }
                }
            }
          else
            emit filedone(actualjob->source);
        }
      else
        {
          actualjob->status=fileoperations::failed;
          emit nottagged(actualjob->source);
        }
      break;
    }
    }
  //emit fileready (actualjob->source);
  //comm_progress transfer={0,0,QString(actualjob->source)};
  //QVariant variant;
  //variant.setValue(transfer);
  //comm_if->requestslot(QString("fileoperator"),QString("fileinwork"),variant);
}

/*########################################################################################################*
 *#Here follow the internal helperfunctions needed, to process files and dirs                            #*
 *########################################################################################################*/

void fileoperator::deletedir(const QString path)
{
  //First check if this directory is on the nodeletelist
  bool found=FALSE;
  QStringList::Iterator nodeletewp=nodeletelist.begin();
  while (nodeletewp!=nodeletelist.end()&&!found)
    {
      if (path.compare(*nodeletewp))
        nodeletewp++;
      else
        found=TRUE;
    }
  if (!found)
    {
      QDir checkdir(path);
      if (checkdir.count()==2)
        {

          checkdir.rmdir(checkdir.absolutePath());
          if (deletemode==fileoperations::directoryrecursive)
            {
              checkdir.cdUp();
              deletedir(checkdir.absolutePath());
            }
        }
      else if (checkdir.count()<=(2+ignorelist.count()))
        {
          //Check all existing files,in the directory, if they match the files on the ignorelist, and remove all entries that match.
          bool deletedfile=FALSE;
          QStringList filelist=checkdir.entryList();
          QStringList::Iterator fileindex=filelist.begin();
          while (fileindex!=filelist.end())
            {
              bool found=FALSE;
              QStringList::Iterator ignoreindex=ignorelist.begin();
              while (ignoreindex!=ignorelist.end()&&!found)
                {
                  if ((*ignoreindex).compare((*fileindex)))
                    ignoreindex++;
                  else
                    found=TRUE;
                }
              if (found)
                {
                  QString processfile=QString("%1/%2").arg(path,(*fileindex));
                  QFileInfo checkfile(processfile);
                  if (checkfile.exists()&&checkfile.isWritable())
                    {
                      QFile::remove(processfile);
                      deletedfile=TRUE;
                    }
                }
              fileindex++;
            }
          //If a file was deleted, we retry to clear the same diretory again
          if (deletedfile)
            deletedir(path);
        }
    }
}

void fileoperator::deleterecursivedir(const QString filename)
{
  QDir removabledir(filename);
  QStringList dirresult=removabledir.entryList(QDir::Dirs);
  for (int i=0;i< dirresult.count();i++)
    {
      if (dirresult.at(i).compare("..")&&dirresult.at(i).compare("."))
        deleterecursivedir(QString("%1/%2").arg(filename,dirresult.at(i)));
    }
  dirresult=removabledir.entryList(QDir::Files);
  for (int i=0;i< dirresult.count();i++)
    {
      if (removabledir.remove(QString("%1/%2").arg(filename,dirresult.at(i))))
        {
          qDebug("Deleting successfull");
        }
      else
        qDebug("Delete file not successfull");
    }
  //Now everything should be empty, delete the directory itself
  removabledir.rmdir(filename);
}

bool fileoperator::setfilter(const QString string)
{
  filterworks=FALSE;
  filterparts.clear();
  filterstring=string;
  //First remove all spaces and slashes from the end
  while (filterstring.endsWith("/")||filterstring.endsWith(" "))
    {
      filterstring=filterstring.left(filterstring.length()-1);
    }
  QString buffer(string);
  while (buffer.contains('#')||buffer.contains('/'))
    {
      if (buffer.startsWith('#'))
        {
          //find the next #, and check if the word matches our mnemnomics
          if (buffer.length()>1)
            {
              int position=buffer.indexOf('#',1);
              QString checkword=buffer.left(position+1);
              if (position!=-1)
                {
                  if (!checkword.compare("#GENRE#")||
                      !checkword.compare("#BAND#")||
                      !checkword.compare("#ALBUM#")||
                      !checkword.compare("#SONG#")||
                      !checkword.compare("#TRACK#")||
                      !checkword.compare("#YEAR#"))
                    {
                      buffer=buffer.right(buffer.length()-checkword.length());
                      filterparts.append(checkword);
                      partcount++;
                      filterworks=TRUE;
                    }
                  else
                    {
                      checkword=checkword.left(checkword.length()-1);
                      filterparts.append(checkword);
                      buffer=buffer.right(buffer.length()-checkword.length());
                      partcount++;
                    }
                }
              else
                {
                  position=buffer.indexOf('/',1);
                  if (position!=-1)
                    {
                      QString checkword=buffer.left(position+1);
                      filterparts.append(checkword);
                      buffer=buffer.right(buffer.length()-checkword.length());
                      partcount++;
                    }
                  else
                    {
                      checkword=buffer;
                      filterparts.append(checkword);
                      partcount++;
                      buffer=QString::null;
                    }
                }
            }
          else
            {
              //Put this single file sign on the partslist
              filterparts<<"#";
              buffer=buffer.right(buffer.length()-1);
              partcount++;
            }
        }
      else if (buffer.startsWith('/'))
        {
          filterparts.append("/");
          partcount++;
          buffer=buffer.right(buffer.length()-1);
        }
      else
        {
          //Find the next # or /
          int position1=(buffer.indexOf('/'));
          int position2=(buffer.indexOf('#'));
          if (position1>-1&&(position2==-1||(position2>position1)))
            {
              QString newpart=buffer.left(position1);
              partcount++;
              filterparts.append(newpart);
              buffer=buffer.right(buffer.length()-newpart.length());
            }
          else if (position2>-1&&(position1==-1||(position1>position2)))
            {
              QString newpart=buffer.left(position2);
              partcount++;
              filterparts.append(newpart);
              buffer=buffer.right(buffer.length()-newpart.length());
            }
        }
    }
  return filterworks;
}

const QString fileoperator::generatenameandpath(const QString filename)
{
  mediafile filetoread(filename);
  if (filetoread.issupportedfile())
    {
      filetoread.readtags(tagversion::v2tag);
      QString artist(replaceunsuported(filetoread.getartistname()));
      QString album(replaceunsuported(filetoread.getalbumname()));
      QString song(replaceunsuported(filetoread.getsongname()));
      int track=filetoread.gettracknumber();
      QString genre(replaceunsuported(filetoread.getgenrename()));
      int release=filetoread.getreleasedate();
      QString newname=basedir;
      QString newpart;
      QStringList::Iterator filterpartindex=filterparts.begin();
      //Create the path also,if it does not exist
      bool missing=FALSE;
      while (filterpartindex!=filterparts.end())
        {
          if (!(*filterpartindex).compare("/"))
            {
              newname+="/"+newpart;
              //Check if the dir exists already
              QFileInfo checkdir(newname);
              if (!checkdir.exists())
                {
                  QDir newdir;
                  newdir.mkpath(newname);
                }
              newpart=QString::null;
            }
          else if (!(*filterpartindex).compare("#GENRE#"))
            {
              if (genre.count())
                newpart+=genre;
              else
                missing=TRUE;
            }
          else if (!(*filterpartindex).compare("#SONG#"))
            {
              if (song.count())
                newpart+=song;
              else
                missing=TRUE;
            }
          else if (!(*filterpartindex).compare("#BAND#"))
            {
              if (artist.count())
                newpart+=artist;
              else
                missing=TRUE;
            }
          else if (!(*filterpartindex).compare("#YEAR#"))
            {
              newpart+=QString::number(release);
            }
          else if (!(*filterpartindex).compare("#ALBUM#"))
            {
              if (album.count())
                newpart+=album;
              else
                missing=TRUE;
            }
          else if (!(*filterpartindex).compare("#TRACK#"))
            {
              if (track<10)
                newpart+="0";
              newpart+=QString::number(track);
            }
          else
            newpart+=(*filterpartindex);
          filterpartindex++;
        }
      if (!newpart.isNull())
        {
          //append the old filetype
          QString filetype=filename.right(4);
          newname+="/"+newpart+filetype;
        }
      if (!missing)
        {
          if (filetoread.canhandlepicture()&&settingsfile->getsav_picture())
            {
              const QList<coverart> coverlist=filetoread.getpictures();
              QList<coverart>::ConstIterator coverindex=coverlist.begin();
              QImage saveimage;
              while (coverindex!=coverlist.end()&&!saveimage.isNull())
                {
                  if ((*coverindex).picposition==Coverart::FrontCover)
                    saveimage=(*coverindex).image;
                  //Write a pic into the directory, the file goes to
                  coverindex++;
                }
              //Take the first stored image, if no frontimage is found
              if (coverlist.count()&&saveimage.isNull())
                saveimage=(*coverlist.begin()).image;
              //Get the foldername out of the newname
              QString folder=newname.left(newname.lastIndexOf("/"));
              QString format;
              switch (settingsfile->getsav_picturetype())
                {
                case 0:
                  format="bmp";
                  break;
                case 1:
                  format="jpg";
                  break;
                case 2:
                  format="gif";
                  break;
                case 3:
                  format="png";
                  break;
                };
              saveimage=saveimage.scaled(settingsfile->getsav_picturesize(),settingsfile->getsav_picturesize(),Qt::KeepAspectRatio,Qt::SmoothTransformation);
              saveimage.save(QString("%1/%2.%3").arg(folder,settingsfile->getsav_picturename(),format));
            }
          return newname;
        }
      else
        return QString::null;
    }
  else
    return QString::null;
}

bool fileoperator::ovcheck(const QString source,const QString destination)
{
  bool writeok=FALSE;
  QFileInfo checkname(destination);
  switch (ovmode)
    {
    case fileoperations::keepnew:
    {
      if (checkname.isWritable())
        {
          QFile::remove(destination);
          bool ok=QFile::copy(source,destination);
          if (ok)
            {
              writeok=TRUE;
              emit filedone(source);
            }
        }
      break;
    }
    case fileoperations::checkbitrate:
    {
      //Compare the bitrates, and take the better file if writable
      mediafile newfile(source),oldfile(destination);
      if (newfile.issupportedfile()&&oldfile.issupportedfile())
        {
          newfile.readtags();
          oldfile.readtags();
          if (newfile.getbitrate()>oldfile.getbitrate())
            {
              if (checkname.isWritable())
                {
                  QFile::remove(destination);
                  bool ok=QFile::copy(source,destination);
                  if (ok)
                    {
                      writeok=TRUE;
                      emit filedone(source);
                    }
                }
            }
          else
            {
              //We dont need to delete the old file, but still the transfer was good, so tell it to the caller.
              writeok=TRUE;
            }
        }
      break;
    }
    case fileoperations::ask:
    {
      int result=QMessageBox::question(0,tr("File already esists"),tr("The file %1 already exists. Do you want to overwrite it with the file %2 ?").arg(destination,source),QMessageBox::Yes,QMessageBox::No);
      if (result==QMessageBox::Yes)
        {
          if (checkname.isWritable())
            {
              QFile::remove(destination);
              bool ok=QFile::copy(source,destination);
              if (ok)
                {
                  writeok=TRUE;
                  emit filedone(source);
                }
            }
        }
      else
        {
          writeok=TRUE;
          emit filedone(source);
        }
      break;
    }
    }
  return writeok;
}

QString fileoperator::replaceunsuported(const QString text)
{
  QString returntext=text;
  QStringList removesigns;
  removesigns<<"/"<<"?"<<":"<<"|"<<"!"<<"#"<<"*"<<".";
  QStringList::Iterator removewp=removesigns.begin();
  while (removewp!=removesigns.end())
    {
      returntext=returntext.replace((*removewp),"_");
      removewp++;
    }
  return returntext;
}

void fileoperator::nottaggedslot(const QString filename)
{
  int result=QMessageBox::question(0,tr("Not propper tagged"),tr("The file %1 is not propper tagged. Do you want to skip it?").arg(filename),QMessageBox::Yes,QMessageBox::No);
  if (result==QMessageBox::Yes)
    resume(filename);
  else
    {
      QMessageBox::information(0,tr("Not propper tagged"),tr("This option is not yet ready, skipping file!").arg(filename),QMessageBox::Ok);
      resume(filename);
    }
}

void fileoperator::nowriteslot(const QString sourcefile, const QString destinationfile)
{
  int result=QMessageBox::warning(0,tr("Not writable"),tr("The file %1 could not be written.Check your privileges!").arg(destinationfile),tr("Skip"),tr("Retry"));
  if (result==0)
    resume(sourcefile);
  else if (result==1)
    retry(sourcefile);
}

void fileoperator::nodeleteslot(const QString filename)
{
  int result=QMessageBox::warning(0,tr("Not deletable"),tr("The file %1 could not be deleted.Check your privileges!").arg(filename),tr("Ignore"),tr("Retry"));
  if (result==0)
    resume(filename);
  else if (result==1)
    {
      QDir removetest(filename);
      if (removetest.remove(filename))
        emit filedone(filename);
      else
        nodeleteslot(filename);

    }
}
