//
// C++ Interface: userinterface
//
// Description:
//
//
// Author: Roland Weigert <roweigert@t-online.de>, (C) 2009
//
// Copyright: See COPYING file that comes with this distribution
//
//

/**
 *@file view.h
 *
 *This file holds theabstract masterclass, for all tabs in the programm:
 *settings,ripper,tagger,progresswidget
 *It includes the communicationinterface, and forces you to implement it, cause everything is pure virtual.
 *Since all tabviews should be scrollareas,it is direct derivated from QScrollArea
 *@author Roland Weigert
 *@date 28-01-2009
 */

#ifndef _USERINTERFACE_H_
#define _USERINTERFACE_H_

#include <QScrollArea>
#include <QWidget>
#include <QObject>

#include "global-stuff/communicationinterface.h"

/**
 * @brief view-class
 * Masterclass for all tabbed views
 */
class view : public QScrollArea
  {
    Q_OBJECT
  public:
    /**
     * @brief Construcotr
     */
    view(QWidget *parent);
    /**
     * @brief get the buttons in the toolbar that are active for this view
     */
    const int getactivebuttons();
    /**
     * @brief Set the activbuttons for this view
     */
    void setactivebuttons(const int value);
  private slots:
    /**
    * Reimplement this two functions, so they do the right stuff for your view
    */
    virtual void requestslot(QString sender,QString type, QVariant){};
    virtual void requestslot(QString sender,QString type, QList<QVariant>){};
  private:
    /**
     * This holds the buttons ored together, that should be active, when this view is front
     * For example: None when the progresswidget is in front.
     */
    int activebuttons;
  };

namespace validactions
  {
  enum actions
  {
    open=1,
    save=2,
    guess=4,
    rip=8,
    upload=16,
    cdinfo=32,
    organise=64
  };
};

#endif
