/***************************************************************************
 *   Copyright (C) 2007 by Roland Weigert   *
 *   roweigert@t-online.de             *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/


#include <QApplication>
#include <QTranslator>
#include <QStringList>
#include <QString>
#include <QSplashScreen>
#include <QPixmap>


#include "mainwindow.h"
#include "mediafile.h"
#include "taginformation.h"
#include "globals.h"

#include "global-stuff/fileoperator.h"
#include "global-stuff/communicationinterface.h"
#include "database/databasethread.h"
#include "settings/rcfile.h"

using namespace std;

rcfile *settingsfile=new rcfile;
fileoperator *fileoperationhandle=NULL;
databasethread *db_thread=new databasethread;
communicationinterface *comm_if=new communicationinterface();
advancedcommunicationinterface *adv_comm_if=new advancedcommunicationinterface();
QSplashScreen *splash=NULL;

int main(int argc, char *argv[])
{
  QApplication app(argc, argv);
  QTranslator translator;
  translator.load("kyamoqt4_"+QLocale::system().name(),":/translations");
  QPixmap pixmap( ":/icons/logo-big.png" );
  splash = new QSplashScreen( pixmap );
  splash->show();
  app.installTranslator(&translator);
  app.setWindowIcon(QIcon(":/icons/logo.png"));
  splash->showMessage("Loading settings");
  //Save a new settingsfile, if loading fails
  settingsfile->load();
  //Gather the list of dirctories that should not get deleted. They can be reseted later on dirchange
  QStringList nodeletelist;
  nodeletelist<<settingsfile->gettempdir()<<settingsfile->getworkingdir()<<settingsfile->getdestinationdir();
  fileoperationhandle=new fileoperator(settingsfile->getsav_deletemode(),settingsfile->getsav_ovmode(),settingsfile->getsav_ignorelist(),settingsfile->getdestinationdir(),nodeletelist);
  fileoperationhandle->setfilter(settingsfile->getsav_filter());
  //Start the databaseinterfacee-thread
  mainwindow appwindow;
  splash->finish( &appwindow );
  /*cdparanoiainterface *testinterface=new cdparanoiainterface;
    testinterface->readtrack(1,"/home/roland/Testverzeichnis");
    testinterface->readtrack(2,"/home/roland/Testverzeichnis");
    testinterface->readtrack(3,"/home/roland/Testverzeichnis");
    for (long i=0;i<1000000;i++)
    qApp->processEvents();
    testinterface->cancelall();
    lameinterface *test=new lameinterface(3);
    test->encodetrack(1,"/home/roland/Testverzeichnis",192,audiomode::Stereo,TRUE,FALSE);
    test->encodetrack(2,"/home/roland/Testverzeichnis",192,audiomode::Stereo,TRUE,FALSE);
    test->encodetrack(3,"/home/roland/Testverzeichnis",192,audiomode::Stereo,TRUE,FALSE);
    test->cancelall();
    */
  int result=app.exec();
  settingsfile->save();
  delete splash;
  return result;
}

