/***************************************************************************
 *   Copyright (C) 2007 by Roland Weigert   *
 *   roweigert@t-online.de             *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include "mainwindow.h"

extern databasethread *db_thread;
extern communicationinterface *comm_if;
extern advancedcommunicationinterface *adv_comm_if;
extern fileoperator *fileoperationhandle;
extern QSplashScreen *splash;

mainwindow::mainwindow():
    mainwidget(0),
    tagdialog(0),
    ripdialog(0),
    playdialog(0),
    progressdialog(0)
{
  splash->showMessage("Setting up mainwindow");
  createactions();
  createmenu();
  createtoolbar();
  createmainwidget();
  statusBar()->showMessage(tr("Ready"));
  setCentralWidget(mainwidget);
  this->resize(800,800);
  this->show();
  tabchanged(0);
  db_thread->testthreading();
  connect(mainwidget,SIGNAL(currentChanged(int)),this,SLOT(tabchanged(int)));
  connect(comm_if,SIGNAL(requestsignal(QString, QString, QVariant)),this,SLOT(requestslot(QString, QString, QVariant)));
  connect(comm_if,SIGNAL(requestsignal(QString, QString, QList<QVariant>)),this,SLOT(requestslot(QString, QString, QList<QVariant>)));
  connect(adv_comm_if,SIGNAL(requestsignal(QStringList,messageobject *)),this,SLOT(requestslot(QStringList,messageobject *)));
}

void mainwindow::createactions()
{
  //Set all actions and deactivate them
  setnewpathaction = new QAction(QIcon(":/icons/fileopen.png"), tr("&Open..."), this);
  setnewpathaction->setShortcut(tr("Ctrl+O"));
  setnewpathaction->setStatusTip(tr("Open a directory"));
  saveselectedaction = new QAction(QIcon(":/icons/filesave.png"), tr("&Save"), this);
  saveselectedaction->setShortcut(tr("Ctrl+S"));
  saveselectedaction->setStatusTip(tr("Save the selected files to disk"));
  saveselectedaction->setEnabled(false);
  organiseaction=new QAction(QIcon(":/icons/wizard.png"), tr("O&rganise"), this);
  organiseaction->setShortcut(tr("Ctrl+r"));
  organiseaction->setStatusTip(tr("Organise files based on tags"));
  organiseaction->setEnabled(false);
  guessaction=new QAction(QIcon(":/icons/guess.png"), tr("&Guess"), this);
  guessaction->setShortcut(tr("Ctrl+g"));
  guessaction->setStatusTip(tr("Guess tags out of filenames with help through musicbrainz or discogs"));
  guessaction->setEnabled(false);
  ripaction= new QAction(QIcon(":/icons/rip.png"), tr("&Rip..."), this);
  ripaction->setShortcut(tr("Ctrl+R"));
  ripaction->setStatusTip(tr("Rip disc"));
  ripaction->setEnabled(false);
  uploadaction= new QAction(QIcon(":/icons/upload.png"), tr("&Upload..."), this);
  uploadaction->setShortcut(tr("Ctrl+U"));
  uploadaction->setStatusTip(tr("Upload discinfo"));
  uploadaction->setEnabled(false);
  cdinfoaction= new QAction(QIcon(":/icons/cdinfo.png"), tr("&CDinfo..."), this);
  cdinfoaction->setShortcut(tr("Ctrl+C"));
  cdinfoaction->setStatusTip(tr("Get CDinfo"));
  connect (cdinfoaction,SIGNAL(triggered(bool)),this,SLOT(cdinfoslot(bool)));
  connect (ripaction,SIGNAL(triggered(bool)),this,SLOT(ripslot(bool)));
  connect (setnewpathaction,SIGNAL(triggered(bool)),this,SLOT(openslot(bool)));
  connect (saveselectedaction,SIGNAL(triggered(bool)),this,SLOT(saveslot(bool)));
  connect (organiseaction,SIGNAL(triggered(bool)),this,SLOT(organiseslot(bool)));
  hideallactions();
}

void mainwindow::createmenu()
{
  filemenu = menuBar()->addMenu(tr("&File"));
  filemenu->addAction(setnewpathaction);
  filemenu->addAction(saveselectedaction);
  filemenu->addAction(cdinfoaction);
  filemenu->addAction(ripaction);
  toolsmenu = menuBar()->addMenu(tr("&Tools"));
  toolsmenu->addAction(organiseaction);
  toolsmenu->addAction(guessaction);
  toolsmenu->addAction(uploadaction);
}

void mainwindow::createtoolbar()
{
  filetoolbar = addToolBar(tr("File"));
  filetoolbar->addAction(setnewpathaction);
  filetoolbar->addAction(saveselectedaction);
  filetoolbar->addAction(cdinfoaction);
  filetoolbar->addAction(ripaction);
  toolstoolbar = addToolBar(tr("Tools"));
  toolstoolbar->addAction(organiseaction);
  toolstoolbar->addAction(guessaction);
  toolstoolbar->addAction(uploadaction);
}

void mainwindow::createmainwidget()
{
  mainwidget=new QTabWidget(this);
  mainwidget->setTabPosition(QTabWidget::West);
  splash->showMessage("Setting up tagger");
  tagdialog=new tagger(this);
  splash->showMessage("Setting up ripper");
  ripdialog=new ripper(this);
  playdialog=new player(this);
  settingsdialog=new settings(this);
  mainwidget->addTab(tagdialog,QIcon(":/icons/tagger.xpm"),tr("Tagger"));
  mainwidget->addTab(ripdialog,QIcon(":/icons/cdaudio.png"),tr("Ripper"));
  mainwidget->addTab(playdialog,QIcon(":/icons/audio.png"),tr("Player"));
  mainwidget->addTab(settingsdialog,QIcon(":/icons/settings.png"),tr("Settings"));
  connect(ripdialog,SIGNAL(needprogress()),this,SLOT(createprogresswidget()));
  connect(tagdialog,SIGNAL(needprogress()),this,SLOT(createprogresswidget()));
}

void mainwindow::createprogresswidget()
{
  if (!progressdialog)
    {
      //Store the activewidget in lastactivewidget
      lastactivewidget=mainwidget->currentWidget();
      //Create a new tab, that shows all running processes, and handles the abortbuttons
      progressdialog=new progresswidget(this);
      int  index=mainwidget->addTab(progressdialog,QIcon(":/icons/clock.png"),tr("Progress"));
      mainwidget->setCurrentIndex(index);
      mainwidget->setTabEnabled(mainwidget->indexOf(tagdialog),FALSE);
      mainwidget->setTabEnabled(mainwidget->indexOf(ripdialog),FALSE);
      mainwidget->setTabEnabled(mainwidget->indexOf(playdialog),FALSE);
      mainwidget->setTabEnabled(mainwidget->indexOf(settingsdialog),FALSE);
      connect (ripdialog,SIGNAL(alldone()),this,SLOT(progressready()));
    }
}

void mainwindow::progressready()
{
  if (progressdialog)
    {
      delete progressdialog;
      progressdialog=0;
      mainwidget->setTabEnabled(mainwidget->indexOf(tagdialog),TRUE);
      mainwidget->setTabEnabled(mainwidget->indexOf(ripdialog),TRUE);
      mainwidget->setTabEnabled(mainwidget->indexOf(playdialog),TRUE);
      mainwidget->setTabEnabled(mainwidget->indexOf(settingsdialog),TRUE);
      mainwidget->setCurrentWidget(lastactivewidget);
      lastactivewidget=0;
    }
}

void mainwindow::hideallactions()
{
  setnewpathaction->setVisible(false);
  organiseaction->setVisible(false);
  guessaction->setVisible(false);
  ripaction->setVisible(false);
  uploadaction->setVisible(false);
  cdinfoaction->setVisible(false);
  saveselectedaction->setVisible(false);
}

void mainwindow::showactions(int value)
{
  hideallactions();
  if (value&validactions::open)
    setnewpathaction->setVisible(true);
  if (value&validactions::save)
    saveselectedaction->setVisible(true);
  if (value&validactions::guess)
    guessaction->setVisible(true);
  if (value&validactions::rip)
    ripaction->setVisible(true);
  if (value&validactions::upload)
    uploadaction->setVisible(true);
  if (value&validactions::cdinfo)
    cdinfoaction->setVisible(true);
  if (value&validactions::organise)
    organiseaction->setVisible(true);
}

void mainwindow::enableaction(int value)
{
  if (value&validactions::open)
    setnewpathaction->setEnabled(true);
  if (value&validactions::save)
    saveselectedaction->setEnabled(true);
  if (value&validactions::guess)
    guessaction->setEnabled(true);
  if (value&validactions::rip)
    ripaction->setEnabled(true);
  if (value&validactions::upload)
    uploadaction->setEnabled(true);
  if (value&validactions::cdinfo)
    cdinfoaction->setEnabled(true);
  if (value&validactions::organise)
    organiseaction->setEnabled(true);
}

void mainwindow::disableaction(int value)
{
  if (value&validactions::open)
    setnewpathaction->setEnabled(false);
  if (value&validactions::save)
    saveselectedaction->setEnabled(false);
  if (value&validactions::guess)
    guessaction->setEnabled(false);
  if (value&validactions::rip)
    ripaction->setEnabled(false);
  if (value&validactions::upload)
    uploadaction->setEnabled(false);
  if (value&validactions::cdinfo)
    cdinfoaction->setEnabled(false);
  if (value&validactions::organise)
    organiseaction->setEnabled(false);
}

void mainwindow::tabchanged(int value)
{
  view *actualwidget=static_cast<view *>(mainwidget->currentWidget());
  showactions(actualwidget->getactivebuttons());
}

void mainwindow::cdinfoslot(bool)
{
  QVariant data(0);
  comm_if->requestslot(QString("main"),QString("discinfo"), data);
}

void mainwindow::ripslot(bool)
{
  QVariant data(0);
  comm_if->requestslot(QString("main"),QString("rip"), data);
}

void mainwindow::openslot(bool)
{
  QVariant data(0);
  comm_if->requestslot(QString("main"),QString("dirchange"), data);
}

void mainwindow::saveslot(bool)
{
  QVariant data(0);
  comm_if->requestslot(QString("main"),QString("save"), data);
}

void mainwindow::organiseslot(bool)
{
  QVariant data(0);
  comm_if->requestslot(QString("main"),QString("organise"), data);
}

void mainwindow::fileoperatormessageslot(QString text)
{
  QString status=QString(tr("Processing file %1")).arg(text);
  statusBar()->showMessage(status,1500);
}

void mainwindow::requestslot(QStringList destination,messageobject *message)
{
  if (destination.contains("main"))
    {
      message->lock();
      if (!message->getsender().compare("fileoperator"))
        {
          if (!message->gettype().compare("fileprogress"))
            {
              QString status=QString(tr("Processing file %1")).arg(message->getdata().toString());
              statusBar()->showMessage(status,1500);
            }
        }
      if (!message->getsender().compare("tagger"))
        {
          if (!message->gettype().compare("openbutton"))
            {
              if (!message->getdata().toBool())
                disableaction(validactions::open);
              else
                enableaction(validactions::open);
            }
        }
      message->unlock();
      if (!message->confirm("main"))
        adv_comm_if->deletemessage(message->getmessageid());
      message->unlock();
    }
}

void mainwindow::requestslot(QString sender,QString type, QVariant data)
{
  if (!sender.compare("ripper"))
    {
      if (data.canConvert<int>())
        {
          comm_progress variantdata=data.value<comm_progress>();
          if (!type.compare("startenable"))
            {
              enableaction(validactions::rip);
            }
          if (!type.compare("startdisable"))
            {
              disableaction(validactions::rip);
            }
        }
    }
  if (!sender.compare("tagger"))
    {
      if (data.canConvert<int>())
        {
          if (!type.compare("filesselected"))
            {
              enableaction(validactions::save|validactions::organise);//validactions::guess|
            }
          if (!type.compare("nofilesselected"))
            {
              disableaction(validactions::save|validactions::guess|validactions::organise);
            }
        }
      if (data.canConvert<comm_progress>())
        {
          comm_progress variantdata=data.value<comm_progress>();
          if (!type.compare("saveprocess"))
            {
              QString status=QString(tr("Saving file %1 of %2")).arg(QString::number(variantdata.track ),QString::number(variantdata.value));
              statusBar()->showMessage(status);
            }
          if (!type.compare("finished"))
            {
              statusBar()->showMessage(tr("All files saved"));
            }
          qApp->processEvents();
        }
    }
  if (!sender.compare("fileoperator"))
    {
      if (data.canConvert<comm_progress>())
        {
          comm_progress variantdata=data.value<comm_progress>();
          if (!type.compare("fileinwork"))
            {
              QString status=QString(tr("Processing file %1")).arg(variantdata.filename);
              statusBar()->showMessage(status,1500);
            }
          qApp->processEvents();
        }
    }
}

void mainwindow::requestslot(QString sender,QString type, QList<QVariant> data)
{}
