/***************************************************************************
 *   Copyright (C) 2007 by Roland Weigert   *
 *   roweigert@t-online.de             *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
/**
 *@file mainwindow.h
 *
 *This file holds the class for the mainwindow, with its menubar,tabwidget and statusbar<br>
 *All other widgets are included over other headers.
 *@author Roland Weigert
 *@date 07-02-2009
 *@todo Take care of close-event
 */

#ifndef _MAINWINDOW_H_
#define _MAINWINDOW_H_

#include <QApplication>
#include <QMainWindow>
#include <QToolBar>
#include <QAction>
#include <QMenuBar>
#include <QStatusBar>
#include <QTabWidget>
#include <QIcon>
#include <QSplashScreen>

#include "global-stuff/view.h"
#include "global-stuff/communicationinterface.h"
#include "global-stuff/fileoperator.h"
#include "tagger/tagger.h"
#include "ripper/ripper.h"
#include "player/player.h"
#include "settings/settingsdialog.h"
#include "progresswidget.h"

#include "database/databasethread.h"

/**
*@brief the applications mainwindow
*
*It initialises all other widgets, and links them into the gui
*/
class mainwindow : public QMainWindow
  {
    Q_OBJECT
  public:
    /**
    *@brief Constructor
    *
    *Initialises the mainwindow, creates all subwidgets and makes all connections.
    */
    mainwindow();
  private slots:
    void createprogresswidget();
    void progressready();
    void tabchanged(int);
    void cdinfoslot(bool);
    void ripslot(bool);
    void openslot(bool);
    void saveslot(bool);
    void organiseslot(bool);
    void fileoperatormessageslot(QString);
    void requestslot(QString sender,QString type, QVariant data);
    void requestslot(QString sender,QString type, QList<QVariant> data);
    void requestslot(QStringList destination,messageobject *);
  signals:
    void encodeprogress(const int value,const int tracknumber);
    void encodeready(const QString savename, const int tracknumber);
    void ripperprogress(const int value,const int tracknumber);
    void ripperready(const QString savename, const int tracknumber);
  private:
    /**
    *@brief Create all actions
    *
    *This actions will then be loaded into the menubar with createmenu()<br>
    *and into the Toolbar with createToolBar().
    */
    void createactions();
    /**
    *@brief Create all Menu, out of the prebuilt actions
    */
    void createmenu();
    /**
    *@brief Create toolbar, out of the prebuilt actions
    */
    void createtoolbar();
    /**
    *@brief Create the maintabwidget, and let it init its own subwidgets
    */
    void createmainwidget();
    void hideallactions();
    void showactions(int);
    void enableaction(int);
    void disableaction(int);
    QMenu *filemenu;
    QMenu *toolsmenu;
    QToolBar *filetoolbar;
    QToolBar *toolstoolbar;
    QAction *setnewpathaction;
    QAction *organiseaction;
    QAction *guessaction;
    QAction *ripaction;
    QAction *uploadaction;
    QAction *cdinfoaction;
    QAction *saveselectedaction;
    QTabWidget *mainwidget;
    tagger *tagdialog;
    ripper *ripdialog;
    player *playdialog;
    settings *settingsdialog;
    progresswidget *progressdialog;
    QWidget *lastactivewidget;

  };

#endif
