//
// C++ Implementation: player
//
// Description:
//
//
// Author: Roland Weigert <roweigert@t-online.de>, (C) 2009
//
// Copyright: See COPYING file that comes with this distribution
//
//

#include "player/player.h"

extern rcfile *settingsfile;
extern advancedcommunicationinterface *adv_comm_if;

player::player(QWidget *parent):
    view(parent),
    playerconnect(NULL),
    sliderdown(false),
    playingitem(NULL),
    isplaying(0)
{
  setactivebuttons(0);
  splitview=new QSplitter(Qt::Horizontal,this);
  createcontrols();
  splitview->addWidget(leftbox);
  splitview->addWidget(controlsbox);
  splitview->setStretchFactor(0,1);
  splitview->setStretchFactor(1,1);
  dialoglayout=new QVBoxLayout(this);
  dialoglayout->addWidget(splitview);
  box=new QWidget(this);
  box->setLayout(dialoglayout);
  this->setWidget(box );
  this->setWidgetResizable( TRUE );
  timeslider->setEnabled(false);
  stopbutton->setEnabled(false);
  prevbutton->setEnabled(false);
  nextbuton->setEnabled(false);
  connect(timeslider,SIGNAL(sliderPressed()),this,SLOT(seekstart()));
  connect(playbutton,SIGNAL(clicked()),this,SLOT(playpause()));
  connect(stopbutton,SIGNAL(clicked()),this,SLOT(stop()));
  connect(prevbutton,SIGNAL(clicked()),this,SLOT(prev()));
  connect(nextbuton,SIGNAL(clicked()),this,SLOT(next()));
  connect(playlist,SIGNAL(removeactionsignal(const QList<QTreeWidgetItem *>)),this,SLOT(removeactionslot(const QList<QTreeWidgetItem *>)));
  connect(playlist,SIGNAL(clearactionsignal()),this,SLOT(clearactionslot()));
  connect(playlist,SIGNAL(itemDoubleClicked(QTreeWidgetItem *,int)),this,SLOT(play(QTreeWidgetItem *,int)));
  connect(adv_comm_if,SIGNAL(requestsignal(QStringList,messageobject *)),this,SLOT(requestslot(QStringList,messageobject *)));
}


player::~player()
{}

void player::createcontrols()
{
  //Leftbox
  leftbox=new QGroupBox(splitview);
  playlist=new playlistwidget(leftbox);
  timelabel=new QLabel(tr("Stopped"),leftbox);
  lengthlabel=new QLabel(tr("Stopped"),leftbox);
  timeslider=new QSlider(Qt::Horizontal,leftbox);
  QSlider(Qt::Horizontal,leftbox);
  leftlayout=new QGridLayout(leftbox);
  leftlayout->addWidget(playlist,0,0,5,6);
  leftlayout->addWidget(timelabel,5,0,1,1);
  leftlayout->addWidget(timeslider,5,1,1,4);
  leftlayout->addWidget(lengthlabel,5,5,1,1);
  leftbox->setLayout(leftlayout);
  //Controlsbox
  controlsbox=new QGroupBox(splitview);
  coverartlabel=new QLabel("No Cover",controlsbox);
  coverartlabel->setMinimumSize(200,200);
  taglabel=new QLabel(controlsbox);
  taglabel->setMinimumWidth(250);
  coverartlabel->setAlignment(Qt::AlignCenter);
  playbutton=new QPushButton(QIcon(":/icons/player_play.png"),tr("Play"),controlsbox);
  stopbutton=new QPushButton(QIcon(":/icons/player_stop.png"),tr("Stop"),controlsbox);
  prevbutton=new QPushButton(QIcon(":/icons/previous.png"),tr("Prev"),controlsbox);
  nextbuton=new QPushButton(QIcon(":/icons/next.png"),tr("Next"),controlsbox);
  controlslayout=new QGridLayout(controlsbox);
  controlslayout->addWidget(coverartlabel,0,0,1,2);
  controlslayout->addWidget(taglabel,1,0,1,2);
  controlslayout->addWidget(playbutton,2,0,1,1);
  controlslayout->addWidget(stopbutton,2,1,1,1);
  controlslayout->addWidget(prevbutton,3,0,1,1);
  controlslayout->addWidget(nextbuton,3,1,1,1);
  controlsspacer=new QSpacerItem(0,0,QSizePolicy::Expanding,QSizePolicy::Expanding);
  controlslayout->addItem(controlsspacer,4,0,1,2);
  controlsbox->setLayout(controlslayout);
}

const QString player::reformat(const QString original,int size)
{
  QString result=original;
  if (result.length()>size)
    {
      float i=1.0;
      while (i<(static_cast<float>(result.length()))/static_cast<float>(size))
        {
          if (i)
            result.insert(i*size+i*4,"<br>");
          else
            result.insert(size,"<br>");
          i++;
        }
    }
  return result;
}

playlistitem * player::append(QStringList list)
{
  playlistitem *firstitem=NULL;
  for (int i=0;i<list.count();i++)
    {
      playlistitem *newitem=new playlistitem(list.at(i),playlist);
      if (newitem->getfilename().isNull())
        delete newitem;
      else if (!firstitem)
        firstitem=newitem;
    }
  if (!playingitem)
    {
      playingitem=firstitem;
      playingitem->setactive(true);
      playbutton->setEnabled(true);
    }
  if (playlist->topLevelItemCount()>1)
    {
      nextbuton->setEnabled(true);
      prevbutton->setEnabled(true);
    }
  else
    {
      nextbuton->setEnabled(false);
      prevbutton->setEnabled(false);
    }
  return firstitem;
}

void player::play(QStringList list)
{
//Delete out the playlist
  playlist->clear();
  playingitem=append(list);
  if (playingitem)
    starttrack(playingitem);
}

void player::play(QTreeWidgetItem *item,int)
{
playingitem->setactive(false);
if(playerconnect)
{
delete playerconnect;
playerconnect=NULL;
}
playingitem=static_cast<playlistitem *>(item);
starttrack(playingitem);
}

void player::starttrack(playlistitem *item)
{
  mediafile actualfile(item->getfilename());
  if (actualfile.issupportedfile())
    {
      if (playerconnect)
        delete playerconnect;
      playerconnect=new playerlink(item->getfilename(),this);
      if(playerconnect->started())
      {
	      isplaying=1;
      stopbutton->setEnabled(true);
      timeslider->setEnabled(true);
      static_cast<playlistitem *>(item)->setactive(true);
      connect(playerconnect,SIGNAL(actualtime(const float)),this,SLOT(processtracking(const float)));
      connect(playerconnect,SIGNAL(finished()),this,SLOT(finished()));
      actualfile.readtags(tagversion::v2tag);
//After reading tags, the songlength is available
      timeslider->setRange(0,actualfile.getlength());
      timeslider->setSliderPosition(0);
      int seconds=0;
      int minutes=0;
      minutes=actualfile.getlength()/60;
      seconds=actualfile.getlength()-minutes*60;
      QString minutestring,secondsstring;
      if (seconds<10)
        secondsstring="0"+QString::number(seconds);
      else
        secondsstring=QString::number(seconds);
      if (minutes<10)
        minutestring="0"+QString::number(minutes);
      else
        minutestring=QString::number(minutes);
      QString timestring=QString("%1 : %2").arg(minutestring,secondsstring);
      lengthlabel->setText(timestring);
      QString htmllabel;
      htmllabel="<table width='300px' cellspacing='5' cellpadding='5'>"
                "<tr><td>"+tr("Artist ")+"</td><td><b>"+reformat(actualfile.getartistname(),20)+"</b></td></tr>"
                "<tr><td>"+tr("Album ")+"</td><td><b>"+reformat(actualfile.getalbumname(),20)+"</b></td></tr>"
                "<tr><td>"+tr("Song ")+"</td><td><b>"+reformat(QString::number(actualfile.gettracknumber())+" . "+actualfile.getsongname(),20)+"</b></td></tr>"
                "<tr><td>"+tr("Genre ")+"</td><td><b>"+reformat(actualfile.getgenrename(),20)+"</b></td></tr>"
                "<tr><td>"+tr("Release ")+"</td><td><b>"+QString::number(actualfile.getreleasedate())+"</b></td></tr>"
		      "</table><p align='center'>"+tr("Playing File:<br><b>")+reformat(item->getfilename(),30)+"</b></p>";
      taglabel->setText(htmllabel);
      if (actualfile.canhandlepicture())
        {
          bool coverset=false;
          const QList<coverart> coverlist=actualfile.getpictures();
          for (int i=0;i<coverlist.count();i++)
            {
              if (coverlist.at(i).picposition==Coverart::FrontCover)
                {
                  QImage scaled;
                  scaled=coverlist.at(i).image.scaledToWidth(200);
                  coverartlabel->setPixmap(QPixmap::fromImage(scaled));
                  coverset=true;
                }
            }
          if (!coverset)
            {
              if (coverlist.count())
                {
                  //Take the first cover in the file, if no frontcover is in
                  QImage scaled;
                  scaled=coverlist.at(0).image.scaledToWidth(200);
                  coverartlabel->setPixmap(QPixmap::fromImage(scaled));
                }
              else
                {
                  coverartlabel->clear();
                  coverartlabel->setText(tr("No Cover"));
                }
            }
        }
      else
        {
          coverartlabel->clear();
          coverartlabel->setText(tr("No Cover"));
        }
      playbutton->setIcon(QIcon(":/icons/player_pause.png"));
      playbutton->setText(tr("Pause"));
    }
    else
	    QMessageBox::critical(this,"K-YAMO-Information",tr("The Selected player is not in the given searchpathes!<br>"
			    "Check your <b>Searchpathsettings</b>."));
    }
  else
    {
      qDebug("File not supported or deleted.Setting filename to QString::null to notify caller");
      finished();
    }
}

void player::clearinfos()
{
  lengthlabel->setText(tr("Stopped"));
  taglabel->setText(tr("Stopped"));
  coverartlabel->clear();
}

void player::processtracking(const float time)
{
  timeslider->setSliderPosition(static_cast<int>(time));
  int seconds=0;
  int minutes=0;
  minutes=static_cast<int>(time)/60;
  seconds=static_cast<int>(time)-minutes*60;
  QString minutestring,secondsstring;
  if (seconds<10)
    secondsstring="0"+QString::number(seconds);
  else
    secondsstring=QString::number(seconds);
  if (minutes<10)
    minutestring="0"+QString::number(minutes);
  else
    minutestring=QString::number(minutes);
  QString timestring=QString("%1 : %2").arg(minutestring,secondsstring);
  timelabel->setText(timestring);
}

void player::finished()
{
  timeslider->setEnabled(false);
  stopbutton->setEnabled(false);
  timeslider->setSliderPosition(0);
  if (playlist->itemBelow(playingitem))
    {
      playingitem->setactive(false);
      playingitem=static_cast<playlistitem *>(playlist->itemBelow(playingitem));
      starttrack(playingitem);
    }
  else
    {
      playbutton->setIcon(QIcon(":/icons/player_play.png"));
      playbutton->setText(tr("Play"));
      isplaying=0;
    }
}

void player::seekstart()
{
  disconnect(playerconnect,SIGNAL(actualtime(const float)),this,SLOT(processtracking(const float)));
  connect(timeslider,SIGNAL(sliderReleased()),this,SLOT(seekend()));
  connect(timeslider,SIGNAL(valueChanged(int)),this,SLOT(timechange(int)));
  sliderdown=true;
  timerpos=0;
}

void player::seekend()
{
  disconnect(timeslider,SIGNAL(sliderReleased()),this,SLOT(seekend()));
  sliderdown=false;
  playerconnect->seek(timerpos);
  connect(playerconnect,SIGNAL(actualtime(const float)),this,SLOT(processtracking(const float)));
}

void player::playpause()
{
  if (!isplaying&&playingitem)
    {
      starttrack(playingitem);
    }
  else if (isplaying==1)
    {
      playerconnect->pause();
      isplaying=2;
      playbutton->setIcon(QIcon(":/icons/player_play.png"));
      playbutton->setText(tr("Play"));
    }
  else if (isplaying==2)
    {
      playerconnect->pause();
      isplaying=1;
      playbutton->setIcon(QIcon(":/icons/player_pause.png"));
      playbutton->setText(tr("Pause"));
    }
}

void player::stop()
{
  if (playerconnect)
    delete playerconnect;
  playerconnect=NULL;
  isplaying=0;
  playbutton->setIcon(QIcon(":/icons/player_play.png"));
  playbutton->setText(tr("Play"));
  timelabel->setText(tr("Stopped"));
  stopbutton->setEnabled(false);
}

void player::prev()
{
  if (playlist->itemAbove(playingitem))
    {
      playingitem->setactive(false);
      playingitem=static_cast<playlistitem *>(playlist->itemAbove(playingitem));
      playingitem->setactive(true);
      if (playerconnect&&isplaying)
        {
          delete playerconnect;
          playerconnect=NULL;
          isplaying=0;
          starttrack(playingitem);
        }
    }
}

void player::next()
{
  if (playlist->itemBelow(playingitem))
    {
      playingitem->setactive(false);
      playingitem=static_cast<playlistitem *>(playlist->itemBelow(playingitem));
      playingitem->setactive(true);
      if (playerconnect&&isplaying)
        {
          delete playerconnect;
          playerconnect=NULL;
          isplaying=0;
          starttrack(playingitem);
        }
    }
}

void player::timechange(int value)
{
  timerpos=value;
  int seconds=0;
  int minutes=0;
  minutes=(timerpos)/60;
  seconds=(timerpos)-minutes*60;
  QString minutestring,secondsstring;
  if (seconds<10)
    secondsstring="0"+QString::number(seconds);
  else
    secondsstring=QString::number(seconds);
  if (minutes<10)
    minutestring="0"+QString::number(minutes);
  else
    minutestring=QString::number(minutes);
  QString timestring=QString("%1 : %2").arg(minutestring,secondsstring);
  timelabel->setText(timestring);
}

void player::playactionslot()
{
  //QTreeWidgetItem *contextitem=playlist->itemAt(this->mapFromGlobal(storedpoint));
  playingitem->setactive(false);
  if (isplaying)
    {
      if (playerconnect)
        delete playerconnect;
      playerconnect=NULL;
    }
//  playingitem=static_cast<playlistitem *>(contextitem);
  playingitem->setactive(true);
  isplaying=0;
  starttrack(playingitem);
}

void player::clearactionslot()
{
  if (playerconnect)
    {
      delete playerconnect;
      playerconnect=NULL;
      playingitem=NULL;
    }
  playlist->clear();
  isplaying=0;
  playbutton->setIcon(QIcon(":/icons/player_play.png"));
  playbutton->setText(tr("Play"));
  timelabel->setText(tr("Stopped"));
  stopbutton->setEnabled(false);
  playbutton->setEnabled(false);
  timeslider->setEnabled(false);
  prevbutton->setEnabled(false);
  nextbuton->setEnabled(false);
  clearinfos();
}

void player::removeactionslot(const QList<QTreeWidgetItem *> list)
{
  bool activechanged=false;
  for (int i=0;i<list.count();i++)
    {
      qDebug("Nuts?");
      qDebug("Playing : %s  Comparing  %s",qPrintable(playingitem->getfilename()),qPrintable(static_cast<playlistitem *>(list.at(i))->getfilename()));
      if (static_cast<playlistitem *>(list.at(i))==playingitem)
        {
          activechanged=true;
          if (playlist->itemBelow(playingitem))
            {
              playingitem->setactive(false);
              playingitem=static_cast<playlistitem *>(playlist->itemBelow(playingitem));
              playingitem->setactive(true);
            }
          else
            {
              playingitem=NULL;
            }
        }
      delete list.at(i);
    }
  if (activechanged&&playerconnect&&isplaying==1)
    {
      delete playerconnect;
      playerconnect=NULL;
      isplaying=0;
      if (playlist->topLevelItemCount())
        starttrack(playingitem);
      else
        {
          playerconnect=NULL;
          isplaying=0;
          playbutton->setIcon(QIcon(":/icons/player_play.png"));
          playbutton->setText(tr("Play"));
          timelabel->setText(tr("Stopped"));
          stopbutton->setEnabled(false);
          playbutton->setEnabled(false);
          timeslider->setEnabled(false);
          prevbutton->setEnabled(false);
          nextbuton->setEnabled(false);
        }
    }
}

void player::requestslot(QStringList destination,messageobject *message)
{
  if (destination.contains("player"))
    {
      message->lock();
      if (!message->getsender().compare("filetree"))
        {
          if (!message->gettype().compare("play"))
            {
              QStringList list=message->getdata().value<QStringList>();
              play(list);
            }
          else if (!message->gettype().compare("append"))
            {
              QStringList list=message->getdata().value<QStringList>();
              append(list);
            }
        }
      if (!message->confirm("player"))
        adv_comm_if->deletemessage(message->getmessageid());
      message->unlock();
    }
}

