//
// C++ Interface: player
//
// Description:
//
//
// Author: Roland Weigert <roweigert@t-online.de>, (C) 2009
//
// Copyright: See COPYING file that comes with this distribution
//
//

/**
 *@file player.h
 *
 *This file holds the class for the playersubwindow.<br>
 *It has its own qlistview with all the files to play.
 *@author Roland Weigert
 *@date 08-03-2009
 *@todo Write all
 */

#ifndef _PLAYER_H_
#define _PLAYER_H_

#include <QScrollArea>
#include <QTreeWidget>
#include <QTreeWidgetItem>
#include <QBrush>
#include <QPushButton>
#include <QDir>
#include <QSplitter>
#include <QString>
#include <QGroupBox>
#include <QLayout>
#include <QLabel>
#include <QImage>
#include <QList>
#include <QWidget>
#include <QObject>
#include <QDropEvent>
#include <QMessageBox>
#include <QMenu>
#include <QContextMenuEvent>

#include "global-stuff/view.h"
#include "tag-interfaces/mediafile.h"
#include "settings/rcfile.h"
#include "global-stuff/communicationinterface.h"
#include "player/playerlink.h"
#include "player/playlistwidget.h"

//Prototypedeclaration
class playlistitem;

/**
 *@brief A class handling the playerinterfaces, and giving a gui to the user
 *
 *@todo Write all
 */
class player : public view
  {
    Q_OBJECT
  public:
    /**
    *@brief Constructor
    *
    *Creates a player object, that will take care of itself.
     */
    player(QWidget *parent);
    /**
     *@brief Destructor
     *
     *Deletes all used widgets.
     */
    ~player();
  private:
    /**
    *@brief Create all editable boxes for the tagger-gui.
     */
    void createcontrols();
    const QString reformat(const QString,int size);
    void clearinfos();
    QVBoxLayout *dialoglayout;
    QWidget *box;
    QSplitter *splitview;
    //Leftbox
    QGroupBox *leftbox;
    playlistwidget *playlist;
    QLabel *timelabel;
    QLabel *lengthlabel;
    QSlider *timeslider;
    QGridLayout *leftlayout;
    //Controlsbox
    QGroupBox *controlsbox;
    QLabel *coverartlabel;
    QLabel *taglabel;
    QPushButton *playbutton;
    QPushButton *stopbutton;
    QPushButton *prevbutton;
    QPushButton *nextbuton;
    QSpacerItem *controlsspacer;
    QGridLayout  *controlslayout;
    playerlink *playerconnect;
    //############################
    bool sliderdown;
    int isplaying;
    int timerpos;
    playlistitem *playingitem;
  private slots:
    playlistitem * append(QStringList);
    void play(QStringList);
    void play(QTreeWidgetItem *,int);
    void starttrack(playlistitem *item);
    void requestslot(QStringList,messageobject *);
    void processtracking(const float);
    void finished();
    void playpause();
    void stop();
    void prev();
    void next();
    void playactionslot();
    void clearactionslot();
    void removeactionslot(const QList<QTreeWidgetItem *> list);
    //#######Seekingpart#########
    void seekstart();
    void seekend();
    void timechange(int);
  };


#endif
