//
// C++ Implementation: playlistwidget
//
// Description:
//
//
// Author: Roland Weigert <roweigert@t-online.de>, (C) 2009
//
// Copyright: See COPYING file that comes with this distribution
//
//

#include "player/playlistwidget.h"

extern advancedcommunicationinterface *adv_comm_if;

playlistwidget::playlistwidget(QWidget *parent):
    QTreeWidget(parent),
    contextmenu(new QMenu(this))
{
  setSelectionMode(QAbstractItemView::ExtendedSelection);
  setColumnCount(3);
  setColumnWidth(0,150);
  setColumnWidth(1,150);
  setColumnWidth(2,150);
  setAlternatingRowColors(true);
  setIconSize(QSize(32,32));
  setRootIsDecorated(false);
  QStringList headerlist;
  headerlist<<tr("Song")<<tr("Artist")<<tr("Album");
  setHeaderLabels(headerlist);
  this->setAcceptDrops(TRUE);
}

void playlistwidget::contextMenuEvent ( QContextMenuEvent * event )
{
  contextmenu->clear();
  storedpoint=event->pos();
  QList<QTreeWidgetItem *> selection=selectedItems () ;
  if (selection.count())
    {
      if (selection.count()>1)
        {
          QAction *removeaction=contextmenu->addAction ( QIcon(":/icons/edit_delete.png"),QString(tr("Remove from playlist")));
          QAction *clearaction=contextmenu->addAction ( QIcon(":/icons/edit_shred.png"),QString(tr("Clear playlist")));
          connect(removeaction,SIGNAL(triggered( bool )),this,SLOT(removeactionslot()));
          connect(clearaction,SIGNAL(triggered( bool )),this,SIGNAL(clearactionsignal()));
          contextmenu->popup(mapToGlobal(storedpoint));
        }
      else
        {
          QAction *playaction=contextmenu->addAction ( QIcon(":/icons/player_play.png"),QString(tr("Play this song")));
          QAction *removeaction=contextmenu->addAction ( QIcon(":/icons/edit_delete.png"),QString(tr("Remove from playlist")));
          QAction *clearaction=contextmenu->addAction ( QIcon(":/icons/edit_shred.png"),QString(tr("Clear playlist")));
          connect(removeaction,SIGNAL(triggered( bool )),this,SLOT(removeactionslot()));
          connect(clearaction,SIGNAL(triggered( bool )),this,SIGNAL(clearactionsignal()));
          connect(playaction,SIGNAL(triggered( bool )),this,SLOT(playactionslot()));
          contextmenu->popup(mapToGlobal(storedpoint));
        }
    }
  else
    {
      //Show contextmenu for empty selection
      if (topLevelItemCount())
        {
          QAction *clearaction=contextmenu->addAction ( QIcon(":/icons/edit_shred.png"),QString(tr("Clear playlist")));
          connect(clearaction,SIGNAL(triggered( bool )),this,SIGNAL(clearactionsignal()));
          contextmenu->popup(mapToGlobal(storedpoint));
        }
    }
}

void playlistwidget::playactionslot()
{
  //Its the easiest way to use this existing signal
  emit itemDoubleClicked(currentItem(),0);
}

void playlistwidget::removeactionslot()
{
  QList<QTreeWidgetItem *> selection=selectedItems () ;
  emit removeactionsignal(selection);
}

void playlistwidget::mousePressEvent(QMouseEvent *event)
{
  QTreeWidget::mousePressEvent(event);
  if (event->button() == Qt::LeftButton)
    dragStartPosition = event->pos();
}

void playlistwidget::mouseMoveEvent(QMouseEvent *event)
{
  if ((event->pos()-dragStartPosition).manhattanLength()>QApplication::startDragDistance())
    {
      QList<QTreeWidgetItem *> selection=selectedItems ();
      QString filelist;
      if (selection.count()&&event->buttons()==Qt::LeftButton)
        {
          for (int i=0;i<selection.count();i++)
            {
              filelist+=QString::number(static_cast<playlistitem *>(selection.at(i))->getindex())+"\n";
            }
          mediafile actualfile(static_cast<playlistitem *>(selection.at(0))->getfilename());
          QPixmap cover;
          if (actualfile.issupportedfile())
            {
              actualfile.readtags(tagversion::v2tag);
              if (actualfile.canhandlepicture())
                {
                  const QList<coverart> coverlist=actualfile.getpictures();
                  for (int i=0;i<coverlist.count();i++)
                    {
                      if (coverlist.at(i).picposition==Coverart::FrontCover)
                        cover=QPixmap::fromImage(coverlist.at(i).image);
                    }
                  if (cover.isNull())
                    {
                      if (coverlist.count())
                        cover=QPixmap::fromImage(coverlist.at(0).image);
                      else
                        cover=QPixmap(":/icons/audio.png");
                    }
                }
              else
                cover=QPixmap(":/icons/audio.png");
            }
          QMimeData *mimedata=new QMimeData;
          mimedata->setText(filelist);
          QDrag *drag=new QDrag(this);
          drag->setMimeData(mimedata);
          drag->setPixmap(cover.scaled(64,64,Qt::KeepAspectRatio));
          drag->start();
        }
    }
}

void playlistwidget::dragEnterEvent(QDragEnterEvent *event)
{
  QStringList formatlist=event->mimeData()->formats();
  for (int i=0;i<formatlist.count();i++)
    qDebug(qPrintable(formatlist.at(i)));
  if (event->mimeData()->hasUrls()||event->mimeData()->hasText())
    event->acceptProposedAction();
}

void playlistwidget::dragMoveEvent(QDragMoveEvent *event)
{
//This function is only needed, to get the drag and drop working in the QTreeWidget
}


void playlistwidget::dropEvent(QDropEvent *event)
{
  if (event->mimeData()->hasUrls())
    {
      QList<QUrl> list=event->mimeData()->urls();
      for (int i=0;i<list.count();i++)
        {
          QString path=list.at(i).path();
          QFileInfo checkfile(path);
          if (checkfile.isDir())
            getallfiles(path);
          else if (path.toLower().endsWith("mp3")||path.toLower().endsWith("ogg")||path.toLower().endsWith("ape")||path.toLower().endsWith("flac"))
            {
              QStringList appendlist;
              appendlist<<path;
              QStringList receivelist;
              receivelist<<"player";
              messageobject *message=new messageobject(QString("filetree"), receivelist, QString("append"), QVariant (path));
              adv_comm_if->sendmessage(message);
            }
        }
    }
  else if (event->mimeData()->hasText())
    {
      QPoint storedpoint=event->pos();
      int newpos=indexOfTopLevelItem(itemAt(storedpoint));
      //find all files in the \n-terminated list
      QString filelist=event->mimeData()->text();
      QStringList realfiles;
      while (filelist.contains("\n"))
        {
          realfiles<<filelist.left(filelist.indexOf("\n"));
          filelist=filelist.right(filelist.length()-(filelist.indexOf("\n")+1));
        }
      QList<QTreeWidgetItem *> movelist;
      for (int i=0;i<realfiles.count();i++)
        {
          for (int j=0;j<topLevelItemCount();j++)
            {
              if (static_cast<playlistitem *>(topLevelItem(j))->getindex()==realfiles.at(i).toInt())
                {
                  QTreeWidgetItem * Item=takeTopLevelItem(j);
                  movelist<<Item;
                }
              if (newpos!=-1)
                insertTopLevelItems(newpos,movelist);
              else
                addTopLevelItems(movelist);

            }
        }
    }
}

void playlistwidget::getallfiles(QString subdir)
{
  QStringList files;
  QDir readindir(subdir);
  QStringList dirresult=readindir.entryList(QDir::Dirs|QDir::NoDotAndDotDot);
  for (int i=0;i< dirresult.count();i++)
    {
      if (subdir.endsWith("/"))
        getallfiles(QString("%1%2").arg(subdir,dirresult.at(i)));
      else
        getallfiles(QString("%1/%2").arg(subdir,dirresult.at(i)));
    }
  QStringList extensionfilters;
  extensionfilters<<"*.mp3"<<"*.ogg";//<<"*.flac"<<"*.ape";
  readindir.setNameFilters(extensionfilters);
  dirresult=readindir.entryList(QDir::Files|QDir::NoDotAndDotDot);
  for (int i=0;i< dirresult.count();i++)
    {
      if (subdir.endsWith("/"))
        files<<(QString("%1%2").arg(subdir,dirresult.at(i)));
      else
        files<<(QString("%1/%2").arg(subdir,dirresult.at(i)));
    }
  if (files.count())
    {
      QStringList receivelist;
      receivelist<<"player";
      messageobject *message=new messageobject(QString("filetree"), receivelist, QString("append"), QVariant (files));
      adv_comm_if->sendmessage(message);
      qApp->processEvents();
    }
}

//############## Playlistitem-part ############################################################

int playlistitem::index=0;

playlistitem::playlistitem(const QString file,QTreeWidget *parent):
    QTreeWidgetItem(parent),
    filename(file),
    pl_index(index++)
{
  QString artist,album,song;
  int track=0;
  mediafile actualfile(filename);
  if (actualfile.issupportedfile())
    {
      actualfile.readtags(tagversion::v2tag);
      artist=actualfile.getartistname();
      album=actualfile.getalbumname();
      song=actualfile.getsongname();
      track=actualfile.gettracknumber();
      if (artist.isNull())
        artist=QString("Unknown");
      if (album.isNull())
        album=QString("Unknown");
      if (song.isNull())
        song=QString("Unknown");
      this->setText(0,QString("%1.%2").arg(QString::number(track),song));
      this->setText(1,artist);
      this->setText(2,album);
      if (actualfile.canhandlepicture())
        {
          const QList<coverart> coverlist=actualfile.getpictures();
          for (int i=0;i<coverlist.count();i++)
            {
              if (coverlist.at(i).picposition==Coverart::FrontCover)
                cover=QIcon(QPixmap::fromImage(coverlist.at(i).image));
            }
          if (cover.isNull())
            {
              if (coverlist.count())
                cover=QIcon(QPixmap::fromImage(coverlist.at(0).image));
              else
                cover=QIcon(":/icons/audio.png");
            }
        }
      else
        cover=QIcon(":/icons/audio.png");
      setIcon(0,cover);
    }
  else
    {
      qDebug("File not supported or deleted.Setting filename to QString::null to notify caller");
      filename=QString::null;
    }
}

playlistitem::~playlistitem()
{}

const QString playlistitem::getfilename() const
  {
    return filename;
  }

const QIcon playlistitem::getcover()
{
  return cover;
}

const int playlistitem::getindex()
{
  return pl_index;
}

void playlistitem::setactive(bool value)
{
  if (value)
    {
      QBrush blue(Qt::blue);
      QBrush white(Qt::white);
      for (int i=0;i<=3;i++)
        {
          setBackground(i,blue);
          setForeground(i,white);
        }
    }
  else
    {
      QBrush transparent(Qt::transparent);
      QBrush black;
      for (int i=0;i<=3;i++)
        {
          setBackground(i,transparent);
          setForeground(i,black);
        }
    }
}
