//
// C++ Interface: playlistwidget
//
// Description:
//
//
// Author: Roland Weigert <roweigert@t-online.de>, (C) 2009
//
// Copyright: See COPYING file that comes with this distribution
//
//
/**
 *@file playlistwidget.h
 *The class for the filetree that shows and handles files and their status.<br>
 *
 *@author Roland Weigert
 *@date 20-02-2009
 *@todo All from scratch
 */

/**
 * If the watcher brings memorywarnings, you have to start this command as root:<br>
 * echo 16384 > /proc/sys/fs/inotify/max_user_watches
 * This enables more watchable files in the kernel.
 */

#ifndef _PLAYLISTWIDGET_H_
#define _PLAYLISTWIDGET_H_

#include <QTreeWidget>
#include <QTreeWidgetItem>
#include <QDir>
#include <QUrl>
#include <QFileSystemWatcher>
#include <QList>
#include <QStringList>
#include <QIcon>
#include <QDropEvent>
#include <QObject>
#include <QApplication>
#include <QMessageBox>
#include <QMenu>
#include <QContextMenuEvent>
#include <QApplication>

#include "global-stuff/communicationinterface.h"
#include "global-stuff/globals.h"
#include "tag-interfaces/mediafile.h"

//Prototype
class playlistitem;

class playlistwidget : public QTreeWidget
  {
    Q_OBJECT
  public:
    playlistwidget(QWidget *parent);
    void mousePressEvent(QMouseEvent *event);
    void mouseMoveEvent(QMouseEvent *event);
  private:
    void dragEnterEvent(QDragEnterEvent *event);
    void dropEvent(QDropEvent *event);
    void dragMoveEvent(QDragMoveEvent *event);
    void getallfiles(QString dirname);
    QMenu *contextmenu;
    QPoint storedpoint;
    QPoint dragStartPosition;
    int fileindex;
  private slots:
    void playactionslot();
    void removeactionslot();
    //This needs to be reimplemented for the contextmenu to work
    void contextMenuEvent ( QContextMenuEvent * event );
  signals:
    void clearactionsignal();
    void removeactionsignal(const QList<QTreeWidgetItem *>);
  };

class playlistitem : public QTreeWidgetItem
  {
  public:
    playlistitem(const QString filename,QTreeWidget *parent);
    ~playlistitem();
    const QString getfilename() const;
    void setactive(bool);
    const QIcon getcover();
    const int getindex();
  private:
    QString filename;
    QIcon cover;
    int pl_index;
    static int index;
  };

#endif
