/***************************************************************************
 *   Copyright (C) 2007 by Roland Weigert   *
 *   roweigert@t-online.de             *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include "progresswidget.h"

extern advancedcommunicationinterface *adv_comm_if;

progresswidget::progresswidget(QWidget *parent):
    view(parent),
    containerwidget(new QWidget(this)),
    layout(new QVBoxLayout(this)),
    cancelallbutton(new QPushButton(QIcon(":/icons/cancel.png"),tr("Cancel all"),this)),
    processcounter(1)
{
  layout->addStretch(10);
  layout->addWidget(cancelallbutton);
  containerwidget->setLayout(layout);
  this->setWidget(containerwidget);
  this->setWidgetResizable( TRUE );
  this->show();
  connect(cancelallbutton,SIGNAL(clicked()),this,SLOT(cancelall()));
  connect(adv_comm_if,SIGNAL(requestsignal(QStringList,messageobject *)),this,SLOT(requestslot(QStringList,messageobject *)));
}

progresswidget::~progresswidget()
{
for (int i=0;i<deletemelist.count();i++)
delete deletemelist.at(i);
}

void progresswidget::encodeprocess(const int value,const int tracknumber)
{
  //Check the encoderslist, if this tracknumber is already on the list
  bool found=FALSE;
  QList<progressobject *>::Iterator progresswalkingpointer=activeencoderslist.begin();
  while (progresswalkingpointer!=activeencoderslist.end()&&!found)
    {
      if ((*progresswalkingpointer)->gettracknumber()==tracknumber)
        {
          found=TRUE;
          (*progresswalkingpointer)->setprogress(value);
        }
      else
        progresswalkingpointer++;
    }
  if (!found)
    {
      QString text=QString(tr("Encoding track Nr.%1")).arg(QString::number(tracknumber));
      progressobject *newobject=new progressobject(text,processcounter,tracknumber);
      connect (newobject,SIGNAL(cancelsignal(const int)),this,SLOT(cancelprocess(const int)));
      activeencoderslist.append(newobject);
      layout->insertWidget(0,newobject);
      processcounter++;
    }
}

void progresswidget::ripprocess(const int value,const int tracknumber)
{
  //Check the ripperslist, if this tracknumber is already on the list
  bool found=FALSE;
  QList<progressobject *>::Iterator progresswalkingpointer=activeripperslist.begin();
  while (progresswalkingpointer!=activeripperslist.end()&&!found)
    {
      if ((*progresswalkingpointer)->gettracknumber()==tracknumber)
        {
          found=TRUE;
          (*progresswalkingpointer)->setprogress(value);
        }
      else
        progresswalkingpointer++;
    }
  if (!found)
    {
      QString text=QString(tr("Ripping track Nr.%1")).arg(QString::number(tracknumber));
      progressobject *newobject=new progressobject(text,processcounter,tracknumber);
      connect (newobject,SIGNAL(cancelsignal(const int)),this,SLOT(cancelprocess(const int)));
      activeripperslist.append(newobject);
      layout->insertWidget(0,newobject);
      processcounter++;
    }
}

void progresswidget::ripready(const QString filename,const int tracknumber)
{
  bool found=FALSE;
  QList<progressobject *>::Iterator progresswalkingpointer=activeripperslist.begin();
  while (!found&&progresswalkingpointer!=activeripperslist.end())
    {
      if ((*progresswalkingpointer)->gettracknumber()==tracknumber)
        {
          found=TRUE;
          (*progresswalkingpointer)->hide();
	  deletemelist.append(*progresswalkingpointer);
          activeripperslist.erase(progresswalkingpointer);
        }
      else
        progresswalkingpointer++;
    }
}

void progresswidget::encodeready(const QString filename,const int tracknumber)
{
  bool found=FALSE;
  QList<progressobject *>::Iterator progresswalkingpointer=activeencoderslist.begin();
  while (!found&&progresswalkingpointer!=activeencoderslist.end())
    {
      if ((*progresswalkingpointer)->gettracknumber()==tracknumber)
        {
          found=TRUE;
          (*progresswalkingpointer)->hide();
	  deletemelist.append(*progresswalkingpointer);
	  activeencoderslist.erase(progresswalkingpointer);
        }
      else
        progresswalkingpointer++;
    }
}

void progresswidget::cancelprocess(const int pnumber)
{
  qDebug("in function cancelprocess");
//Find out, what the type of this progress is
  bool found=FALSE;
  QList<progressobject *>::Iterator progresswalkingpointer=activeencoderslist.begin();
  while (!found&&progresswalkingpointer!=activeencoderslist.end())
    {
      if ((*progresswalkingpointer)->getprogressnumber()==pnumber)
        {
          found=TRUE;
          QStringList receivelist;
          receivelist<<"encode";
          messageobject *message=new messageobject("progresswidget", receivelist, QString("cancel"), QVariant ((*progresswalkingpointer)->gettracknumber()));
          adv_comm_if->sendmessage(message);
        }
      else
        progresswalkingpointer++;
    }
  if (!found)
    {
      progresswalkingpointer=activeripperslist.begin();
      while (!found&&progresswalkingpointer!=activeripperslist.end())
        {
          if ((*progresswalkingpointer)->getprogressnumber()==pnumber)
            {
              found=TRUE;
              QStringList receivelist;
              receivelist<<"discread";
              messageobject *message=new messageobject("progresswidget", receivelist, QString("cancel"), QVariant ((*progresswalkingpointer)->gettracknumber()));
              adv_comm_if->sendmessage(message);
            }
          else
            progresswalkingpointer++;
        }
    }
}

void progresswidget::cancelall()
{
  //Check what is running and send the message only to running interfaces
  QStringList receivelist;
  if (activeencoderslist.count())
    receivelist<<"encode";
  if (activeripperslist.count())
    receivelist<<"discread";
  messageobject *message=new messageobject("progresswidget", receivelist, QString("cancel all"), QVariant (0));
  adv_comm_if->sendmessage(message);
}

void progresswidget::requestslot(QStringList destination,messageobject *message)
{
  if (destination.contains("progressview"))
    {
      message->lock();
      if (!message->getsender().compare("discread"))
        {
          if (!message->gettype().compare("finished"))
            {
              comm_progress variantdata=message->getdata().value<comm_progress>();
              ripready(variantdata.filename,variantdata.track);
            }
          else if (!message->gettype().compare("progress"))
            {
              comm_progress variantdata=message->getdata().value<comm_progress>();
              ripprocess(variantdata.value,variantdata.track);
            }
        }
      else if (!message->getsender().compare("encoder"))
        {
		if (!message->gettype().compare("finished"))
            {
              comm_progress variantdata=message->getdata().value<comm_progress>();
              encodeready(variantdata.filename,variantdata.track);
            }
          else if (!message->gettype().compare("progress"))
            {
              comm_progress variantdata=message->getdata().value<comm_progress>();
              encodeprocess(variantdata.value,variantdata.track);
            }
        }
      if (!message->confirm("progressview"))
        {
          message->unlock();
          adv_comm_if->deletemessage(message->getmessageid());
        }
      else
        message->unlock();

    }
}

//The progressobject-declarations follow here

progressobject::progressobject(QString text,int pnumber,int tnumber):
    layout(new QHBoxLayout(this)),
    cancelbutton(new QPushButton(QIcon(":/icons/cancel.png"),tr("Cancel"),this)),
    textlabel(new QLabel(text,this)),
    progress(new QProgressBar(this)),
    progressnumber(pnumber),
    tracknumber(tnumber)
{
  connect (cancelbutton,SIGNAL(clicked()),this,SLOT(cancelslot()));
  layout->addWidget(textlabel,5);
  layout->addWidget(progress,10);
  layout->addWidget(cancelbutton,5);
  this->setLayout(layout);
  this->show();
}

progressobject::~progressobject()
{
}

const int progressobject::getprogressnumber()const
  {
    return progressnumber;
  }

const int progressobject::gettracknumber()const
  {
    return tracknumber;
  }

void progressobject::setprogress(const int value)
{
  if (progress->value()<value)
    progress->setValue(value);
}

void progressobject::cancelslot()
{
  emit cancelsignal(progressnumber);
}
