/***************************************************************************
 *   Copyright (C) 2007 by Roland Weigert   *
 *   roweigert@t-online.de             *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
/**
 *@file progresswidget.h
 *
 *This file holds the class forshowing of all progresses. It catches signals of all connected<br>
 *objects, that can generate a progress. New Progresses are generated, when a new object is<br>
 *sending a first progress. Progresses are removed, when they send there readysignal
 *@author Roland Weigert
 *@date 23-02-2009
 */

#ifndef _PROGRESSWIDGET_H_
#define _PROGRESSWIDGET_H_

#include <QGroupBox>
#include <QPushButton>
#include <QString>
#include <QLabel>
#include <QList>
#include <QProgressBar>
#include <QLayout>
#include <QWidget>
#include <QVariant>

#include "global-stuff/communicationinterface.h"
#include "global-stuff/view.h"

//Prototypedeclarations
class progressobject;

/**
*@brief The class that shows all progresses, and handles theyr abortbuttons
*/
class progresswidget : public view
  {
    Q_OBJECT
  public:
    /**
    *@brief Constructor
    *@param parent The Pointer for the mainwidget, so the Signals of it can be catched
    */
    progresswidget(QWidget *parent);
    /**
    *@brief Destructor
    */
    ~progresswidget();
  signals:
    void encodecancel(const int);
    void ripcancel(const int);
    void alldone();
  private slots:
    void encodeprocess(const int value,const int tracknumber);
    void ripprocess(const int value,const int tracknumber);
    void ripready(const QString filename,const int tracknumber);
    void encodeready(const QString filename,const int tracknumber);
    void cancelprocess(const int pnumber);
    void cancelall();
    void requestslot(QStringList,messageobject *);
  private:
    QList<progressobject *> activeripperslist;
    QList<progressobject *> activeencoderslist;
    QList<progressobject *> deletemelist;
    QWidget *containerwidget;
    QVBoxLayout *layout;
    QPushButton *cancelallbutton;
    int processcounter;
  };

/**
*@brief This class holds one progress, and handles its Cancelbutton
*/
class progressobject : public QGroupBox
  {
    Q_OBJECT
  public:
    /**
    *@brief Constructor
    *@param text The text that is shown for this progress
    *@param pnumber The Internal Processnumber, that is given by the progresswidget
    *@param tnumber The Tracknumber of this progress
    */
    progressobject(QString text,int pnumber,int tnumber);
    ~progressobject();
    const int getprogressnumber()const;
    const int gettracknumber()const;
  private:
    QHBoxLayout *layout;
    QPushButton *cancelbutton;
    QLabel *textlabel;
    QProgressBar *progress;
    int progressnumber;
    int tracknumber;
  signals:
    void cancelsignal(const int progressnumber);
  public slots:
    void setprogress(const int value);
  private slots:
    void cancelslot();
  };

#endif
