/***************************************************************************
 *   Copyright (C) 2007 by Roland Weigert   *
 *   roweigert@t-online.de             *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include "cdparanoiainterface.h"

extern fileoperator *fileoperationhandle;
extern communicationinterface *comm_if;
extern advancedcommunicationinterface *adv_comm_if;

cdparanoiainterface::cdparanoiainterface():
    taskrunning(false),
    abortrunning(false)
{
  connect(comm_if,SIGNAL(requestsignal(QString, QString, QVariant)),this,SLOT(requestslot(QString, QString, QVariant)));
  connect(comm_if,SIGNAL(requestsignal(QString, QString, QList<QVariant>)),this,SLOT(requestslot(QString, QString, QList<QVariant>)));
  connect(adv_comm_if,SIGNAL(requestsignal(QStringList,messageobject *)),this,SLOT(requestslot(QStringList,messageobject *)));
}

cdparanoiainterface::~cdparanoiainterface()
{
  QList<cdparanoiaprocess>::Iterator processwalkingpointer=processlist.begin();
  while (processwalkingpointer!=processlist.end())
    {
      delete (*processwalkingpointer).readprocess;
      processwalkingpointer++;
    }
}

const bool cdparanoiainterface::paranoiaok()
{
  QFileInfo paranoiacheck("/usr/bin/cdparanoia");
  return paranoiacheck.exists();
}

void cdparanoiainterface::readtrack(const int tracknumber,const QString path)
{
  if (paranoiaok())
    {
//Put this task, on the todolist
      struct cdparanoiaprocess newprocess=
        {
          0,0,
          FALSE,FALSE,
          tracknumber,
          QString("%1/Track%2.wav").arg(path,QString::number(tracknumber)),
          new QProcess
        };
      if (!taskrunning)
        {
//Ok, nothing is running atm, so we can start this task, and connect the readysignal, so it will start the next read when ready
          QStringList parameters;
          parameters<<"-e"<<QString::number(newprocess.tracknumber)<<newprocess.filename;
          newprocess.readprocess->start("/usr/bin/cdparanoia",parameters);
          newprocess.running=TRUE;
          taskrunning=TRUE;
          connect (newprocess.readprocess,SIGNAL(readyReadStandardError()),this,SLOT(processtracking()));
          connect (newprocess.readprocess,SIGNAL(finished(int,QProcess::ExitStatus)),this,SLOT(processfinished(int)));
        }
      processlist.append(newprocess);
    }
  else
    emit paranoiaerror(tr("Can't find cdparanoia! Is it installed?"));
}

void cdparanoiainterface::processtracking()
{
//Find the running task in the list, and get its data
  bool found=FALSE;
  QList<cdparanoiaprocess>::Iterator processwalkingpointer=processlist.begin();
  while (!found&&processwalkingpointer!=processlist.end())
    {
      if ((*processwalkingpointer).running==TRUE)
        found=TRUE;
      else
        processwalkingpointer++;
    }
  if (found)
    {
      QByteArray receiveddata=(*processwalkingpointer).readprocess->readAllStandardError();
      QString data=QString::fromLocal8Bit(receiveddata.constData());
      if (data.contains("Ripping from sector"))
        {
//Throw everything away, till the startsector
          data=data.right(data.length()-(data.indexOf("Ripping from sector")+19));
          QString buffer=data.left(data.indexOf("("));
          (*processwalkingpointer).start=buffer.toLong();
        }
      if (data.contains("to sector"))
        {
//Throw everything away, till the startsector
          data=data.right(data.length()-(data.indexOf("to sector")+9));
          QString buffer=data.left(data.indexOf("("));
          (*processwalkingpointer).end=buffer.toLong();
        }
      if (data.contains("[read]"))
        {
          //Take the first occurence of [read] and take everything from @ to \n as number
          QString buffer=data.right(data.length()-data.indexOf("[read]"));
          buffer=buffer.right(buffer.length()-(buffer.indexOf("@")+1));
          buffer=buffer.left(buffer.indexOf("\n"));
          long readingpos=(buffer.toLong()/1176)-(*processwalkingpointer).start;
          int progressvalue=readingpos*100/((*processwalkingpointer).end-(*processwalkingpointer).start);
          comm_progress transfer={(*processwalkingpointer).tracknumber,readingpos*100/((*processwalkingpointer).end-(*processwalkingpointer).start)};
          QVariant variant;
          variant.setValue(transfer);
          //comm_if->requestslot(QString("discread"),QString("progress"),variant);
          QStringList receivelist;
          receivelist<<"progressview";
          messageobject *message=new messageobject(QString("discread"), receivelist, QString("progress"), QVariant (variant));
          adv_comm_if->sendmessage(message);
        }
    }
}

void cdparanoiainterface::processfinished(int exitcode)
{
  bool found=FALSE;
  QList<cdparanoiaprocess>::Iterator processwalkingpointer=processlist.begin();
  while (!found&&processwalkingpointer!=processlist.end())
    {
      if ((*processwalkingpointer).running==TRUE)
        {
          (*processwalkingpointer).running=FALSE;
          (*processwalkingpointer).ready=TRUE;
          found=TRUE;
          //This is needed in the maenwhile to get the code running, until the mainwindow is converted to comm_if
          //emit trackreadready((*processwalkingpointer).filename, (*processwalkingpointer).tracknumber);
          QStringList receivelist;
          receivelist<<"progressview"<<"ripper";
          comm_progress transfer={(*processwalkingpointer).tracknumber,0,(*processwalkingpointer).filename};
          QVariant variant;
          variant.setValue(transfer);
          messageobject *message=new messageobject(QString("discread"), receivelist, QString("finished"), QVariant (variant));
          adv_comm_if->sendmessage(message);
        }
      else
        processwalkingpointer++;
    }
  if (found)
    {
      processlist.erase(processwalkingpointer);
      //Check if we still have some stuff to do, and launch the next task if there is one
      if (processlist.count()&&!abortrunning)
        {
          processwalkingpointer=processlist.begin();
          QStringList parameters;
          parameters<<"-e"<<QString::number((*processwalkingpointer).tracknumber)<<(*processwalkingpointer).filename;
          (*processwalkingpointer).readprocess->start("/usr/bin/cdparanoia",parameters);
          (*processwalkingpointer).running=TRUE;
          taskrunning=TRUE;
          connect ((*processwalkingpointer).readprocess,SIGNAL(readyReadStandardError()),this,SLOT(processtracking()));
          connect ((*processwalkingpointer).readprocess,SIGNAL(finished(int,QProcess::ExitStatus)),this,SLOT(processfinished(int)));
        }
      else
        taskrunning=FALSE;
    }
}

void cdparanoiainterface::cancelread(int track)
{
  bool found=FALSE;
  QList<cdparanoiaprocess>::Iterator processwalkingpointer=processlist.begin();
  while (!found&&processwalkingpointer!=processlist.end())
    {
      if ((*processwalkingpointer).tracknumber==track)
        {
          disconnect ((*processwalkingpointer).readprocess,SIGNAL(readyReadStandardError()),this,SLOT(processtracking()));
          disconnect ((*processwalkingpointer).readprocess,SIGNAL(finished(int,QProcess::ExitStatus)),this,SLOT(processfinished(int)));
          (*processwalkingpointer).readprocess->terminate();
          //Check if we need to delete the new file
          fileoperationhandle->deletefile((*processwalkingpointer).filename);
          QStringList receivelist;
          receivelist<<"progressview"<<"ripper";
          comm_progress transfer={(*processwalkingpointer).tracknumber,0,(*processwalkingpointer).filename};
          QVariant variant;
          variant.setValue(transfer);
          messageobject *message=new messageobject(QString("discread"), receivelist, QString("finished"), QVariant (variant));
          adv_comm_if->sendmessage(message);
          found=TRUE;
	  processlist.erase(processwalkingpointer);
        }
      else
        processwalkingpointer++;
    }
  if (found)
    {
      //Check if we still have some stuff to do, and launch the next task if there is one
      if (processlist.count())
        {
          processwalkingpointer=processlist.begin();
          QStringList parameters;
          parameters<<"-e"<<QString::number((*processwalkingpointer).tracknumber)<<(*processwalkingpointer).filename;
          (*processwalkingpointer).readprocess->start("/usr/bin/cdparanoia",parameters);
          (*processwalkingpointer).running=TRUE;
          taskrunning=TRUE;
          connect ((*processwalkingpointer).readprocess,SIGNAL(readyReadStandardError()),this,SLOT(processtracking()));
          connect ((*processwalkingpointer).readprocess,SIGNAL(finished(int,QProcess::ExitStatus)),this,SLOT(processfinished(int)));
        }
      else
        taskrunning=FALSE;
    }
}

void cdparanoiainterface::cancelall()
{
  abortrunning=true;
  QList<cdparanoiaprocess>::Iterator processwalkingpointer=processlist.begin();
  while (processwalkingpointer!=processlist.end())
    {
      //We need to send a ready signal for the one running process, to get rid of the progress displayed
      if ((*processwalkingpointer).running==TRUE)
        {
          //We have to call this for our own, because processfinished has an empty list after this
          QStringList receivelist;
          receivelist<<"progressview"<<"ripper";
          comm_progress transfer={(*processwalkingpointer).tracknumber,0,(*processwalkingpointer).filename};
          QVariant variant;
          variant.setValue(transfer);
          messageobject *message=new messageobject(QString("discread"), receivelist, QString("finished"), QVariant (variant));
          adv_comm_if->sendmessage(message);
          (*processwalkingpointer).readprocess->terminate();
        }
      //Check if we need to delete the new file
      fileoperationhandle->deletefile((*processwalkingpointer).filename);
      processlist.erase(processwalkingpointer);
      processwalkingpointer++;
    }
  taskrunning=FALSE;
}

const bool cdparanoiainterface::isrunning() const
  {
    return taskrunning;
  }

void cdparanoiainterface::requestslot(QStringList destination,messageobject *message)
{
  if (destination.contains("discread"))
    {
      message->lock();
      if (!message->getsender().compare("progresswidget"))
        {
          if (!message->gettype().compare("cancel all"))
            cancelall();
          else if (!message->gettype().compare("cancel"))
            cancelread(message->getdata().value<int>());
        }
      message->unlock();
      if (!message->confirm("discread"))
        adv_comm_if->deletemessage(message->getmessageid());
    }
}

void cdparanoiainterface::requestslot(QString sender,QString type, QVariant data)
{
  /*    if (!sender.compare("progresswidget"))
        {
          if (data.canConvert<int>())
            {
              int track=data.value<int>();
              if (!type.compare("cancel rip"))
                cancelread(track);
              else if (!type.compare("cancel all"))
                cancelall();
            }
        }*/
}


void cdparanoiainterface::requestslot(QString sender,QString type, QList<QVariant> data)
{
// This is not needed in this class. (Myabe this will change later)
}
