/***************************************************************************
 *   Copyright (C) 2007 by Roland Weigert   *
 *   roweigert@t-online.de             *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include "discselector.h"

discselector::discselector(const QList<freedbalbum> list):
    albumlist(list),
    layout(new QVBoxLayout),
    buttonlayout(new QHBoxLayout),
    infotext(new QLabel(tr("Select the right disc and click ok. If you are unsure press apply first."))),
    choicestree(new QTreeWidget),
    okbutton(new QPushButton(QIcon(":/icons/ok.png"),tr("&Ok"))),
    trybutton(new QPushButton(QIcon(":/icons/apply.png"),tr("&Apply"))),
    cancelbutton(new QPushButton(QIcon(":/icons/cancel.png"),tr("&Cancel")))
{
  choicestree->setColumnCount(4);
  QStringList headers;
  headers<<tr("Discid")<<tr("Genre")<<tr("Artist")<<tr("Album");
  choicestree->setHeaderLabels(headers);
  QList<freedbalbum>::ConstIterator albumwalkingpointer=list.begin();
  while (albumwalkingpointer!=list.end())
    {
      QStringList newentry;
      newentry<<(*albumwalkingpointer).fdb_discid<<(*albumwalkingpointer).fdb_genre<<(*albumwalkingpointer).fdb_artist<<(*albumwalkingpointer).fdb_album;
      new QTreeWidgetItem(choicestree,newentry);
      albumwalkingpointer++;
    }
  layout->addWidget(infotext);
  layout->addWidget(choicestree);
  okbutton->setEnabled(FALSE);
  trybutton->setEnabled(FALSE);
  buttonlayout->addWidget(okbutton);
  buttonlayout->addWidget(trybutton);
  buttonlayout->addWidget(cancelbutton);
  layout->addLayout(buttonlayout);
  connect (okbutton,SIGNAL(clicked()),this,SLOT(okclicked()));
  connect (cancelbutton,SIGNAL(clicked()),this,SIGNAL(cancelsignal()));
  connect (trybutton,SIGNAL(clicked()),this,SLOT(tryclicked()));
  connect (choicestree,SIGNAL(itemClicked(QTreeWidgetItem*,int)),this,SLOT(itemchanged(QTreeWidgetItem *,int)));
  this->setModal(TRUE);
  this->setLayout(layout);
}

discselector::~discselector()
{
  delete infotext;
  delete choicestree;
  delete okbutton;
  delete trybutton;
  delete cancelbutton;
  delete buttonlayout;
  delete layout;
}

void discselector::closeEvent(QCloseEvent *event)
{
  event->ignore();
  emit cancelsignal();
}

void discselector::tryclicked()
{
  emit trysignal(selectedalbum);
}

void discselector::okclicked()
{
  emit oksignal(selectedalbum);
}

void discselector::itemchanged(QTreeWidgetItem *item,int col)
{
  QString selected=item->text(0);
  okbutton->setEnabled(TRUE);
  trybutton->setEnabled(TRUE);
  //Find the right item and set it as selected
  QList<freedbalbum>::ConstIterator albumwalkingpointer=albumlist.begin();
  while (albumwalkingpointer!=albumlist.end())
    {
      if (!(*albumwalkingpointer).fdb_discid.compare(item->text(0))&&!(*albumwalkingpointer).fdb_genre.compare(item->text(1))&&!(*albumwalkingpointer).fdb_artist.compare(item->text(2))&&!(*albumwalkingpointer).fdb_album.compare(item->text(3)))
        {
          selectedalbum=(*albumwalkingpointer);
          break;
        }
      albumwalkingpointer++;
    }
}
