/***************************************************************************
 *   Copyright (C) 2007 by Roland Weigert   *
 *   roweigert@t-online.de             *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include "lameinterface.h"

extern fileoperator *fileoperationhandle;
extern communicationinterface *comm_if;
extern advancedcommunicationinterface *adv_comm_if;

lameinterface::lameinterface(int encodercount):
    runningtasks(0),
    numcoders(encodercount)
{
  connect(comm_if,SIGNAL(requestsignal(QString, QString, QVariant)),this,SLOT(requestslot(QString, QString, QVariant)));
  connect(comm_if,SIGNAL(requestsignal(QString, QString, QList<QVariant>)),this,SLOT(requestslot(QString, QString, QList<QVariant>)));
  connect(adv_comm_if,SIGNAL(requestsignal(QStringList,messageobject *)),this,SLOT(requestslot(QStringList,messageobject *)));
}

lameinterface::~lameinterface()
{
  QList<lameprocess *>::Iterator processwalkingpointer=encodelist.begin();
  while (processwalkingpointer!=encodelist.end())
    {
      delete (*processwalkingpointer);
      processwalkingpointer++;
    }
}

bool lameinterface::lameok()
{
  QFileInfo checklame("/usr/bin/lame");
  return checklame.exists();
}

void lameinterface::encodetrack(const int track,const QString path,const int bitrate,const int stereomode,bool high, bool fast)
{
	qDebug("track: %d  path: %s  bitrate: %d  stereomode: %d",track,qPrintable(path),bitrate, stereomode);
	if (lameok())
    {
      lameprocess *newprocess=new lameprocess(track,path,bitrate,stereomode,high,fast);
      if (runningtasks<numcoders)
        {
		newprocess->start();
          connect(newprocess,SIGNAL(lameerror(const QString)),this,SIGNAL(lameerror(const QString)));
          connect(newprocess,SIGNAL(trackencodeready(const QString,const int)),this,SLOT(gottrackencodeready(const QString,const int)));
          runningtasks++;
        }
      encodelist.append(newprocess);
    }
  else
    emit lameerror(tr("You don't have lame installed!"));
}

void lameinterface::gottrackencodeready(const QString savename, const int tracknumber)
{
  runningtasks--;
  //emit trackencodeready(savename,tracknumber);
  bool found=FALSE;
  QList<lameprocess *>::Iterator processwalkingpointer=encodelist.begin();
  while (processwalkingpointer!=encodelist.end()&&!found)
    {
      if (!(*processwalkingpointer)->isrunning())
        {
          found=TRUE;
          (*processwalkingpointer)->start();
          connect((*processwalkingpointer),SIGNAL(lameerror(const QString)),this,SIGNAL(lameerror(const QString)));
          connect((*processwalkingpointer),SIGNAL(trackencodeready(const QString,const int)),this,SLOT(gottrackencodeready(const QString,const int)));
          runningtasks++;
        }
      processwalkingpointer++;
    }
  if (!runningtasks)
    emit allencoded();
}

void lameinterface::cancelencode(int track)
{
  bool found=FALSE;
  QList<lameprocess *>::Iterator processwalkingpointer=encodelist.begin();
  while (processwalkingpointer!=encodelist.end()&&!found)
    {
      if ((*processwalkingpointer)->gettracknumber()==track)
        {
	  found=TRUE;
          (*processwalkingpointer)->cancelencode();
        }
      else
        processwalkingpointer++;
    }
}

void lameinterface::cancelall()
{
//Cancel all encoders
  QList<lameprocess *>::Iterator processwalkingpointer=encodelist.begin();
  while (processwalkingpointer!=encodelist.end())
    {
      (*processwalkingpointer)->cancelencode();
      processwalkingpointer++;
    }
}

const bool lameinterface::isrunning()
{
  if (runningtasks)
    return TRUE;
  else
    return FALSE;
}

void lameinterface::requestslot(QStringList destination,messageobject *message)
{
  if (destination.contains("encode"))
    {
      message->lock();
      if (!message->getsender().compare("progresswidget"))
        {
          if (!message->gettype().compare("cancel all"))
            cancelall();
          else if (!message->gettype().compare("cancel"))
	  {
		  cancelencode(message->getdata().value<int>());
	  }
        }
      message->unlock();
      if (!message->confirm("encode"))
        adv_comm_if->deletemessage(message->getmessageid());
    }
}

void lameinterface::requestslot(QString sender,QString type, QVariant data)
{
  if (!sender.compare("progresswidget"))
    {
      if (data.canConvert<int>())
        {
          if (!type.compare("cancel encode"))
            {
          int track=data.value<int>();
              cancelencode(track);
            }
          if (!type.compare("cancel all"))
            cancelall();
        }
    }
}

void lameinterface::requestslot(QString sender,QString type, QList<QVariant> data)
{
// This is not needed in this class. (Maybe this will change later)
}

lameprocess::lameprocess(const int track,const QString path,const int bitrate,const int stereomode,bool high, bool fast):
    tracknumber(track),
    source(QString("%1/Track%2.wav").arg(path,QString::number(tracknumber))),
    destination(QString("%1/Track%2.mp3").arg(path,QString::number(tracknumber))),
    process(new QProcess),
    executable(QString("/usr/bin/lame")),
    taskrunning(FALSE)
{
  qDebug(qPrintable(source));
  if (bitrate)
    parameters<<"-b"<<(QString::number(bitrate));
  else
    parameters<<"--vbr-new";
  if (high)
    parameters<<"-h";
  if (fast)
    parameters<<"-f";
  if (stereomode==audiomode::Stereo)
    parameters<<"-m"<<"s";
  else if (stereomode==audiomode::JointStereo)
    parameters<<"-m"<<"j";
  else if (stereomode==audiomode::SingleChannel)
    parameters<<"-m"<<"m";
  else if (stereomode==audiomode::DualChannel)
    parameters<<"-m"<<"d";
  parameters<<source<<destination;
}

lameprocess::~lameprocess()
{
  delete process;
}

void lameprocess::start()
{
  process->start(executable,parameters);
  taskrunning=1;
  connect (process,SIGNAL(readyReadStandardError()),this,SLOT(processtracking()));
  connect (process,SIGNAL(finished(int,QProcess::ExitStatus)),this,SLOT(processfinished(int)));
}

const int lameprocess::isrunning() const
  {
    return taskrunning;
  }

void lameprocess::cancelencode()
{
  if (taskrunning)
    process->terminate();
  fileoperationhandle->deletefile(destination);

}

void lameprocess::processtracking()
{
  QByteArray receiveddata=process->readAllStandardError();
  QString data=QString::fromLocal8Bit(receiveddata.constData());
  if (data.contains("%"))
    {
      data=data.right(data.length()-(data.indexOf("%)")-3));
      if (data.startsWith("("))
        data.remove("(");
      data=data.left(data.indexOf("%"));
      comm_progress transfer={tracknumber,data.toInt()};
      QVariant variant;
      variant.setValue(transfer);
      //comm_if->requestslot(QString("encode"),QString("progress"),variant);
      QStringList receivelist;
      receivelist<<"progressview";
      messageobject *message=new messageobject(QString("encoder"), receivelist, QString("progress"), QVariant (variant));
      adv_comm_if->sendmessage(message);
    }
}

void lameprocess::processfinished(int)
{
  comm_progress transfer={tracknumber,0,destination};
  QStringList receivelist;
  receivelist<<"progressview"<<"ripper";
  QVariant variant;
  variant.setValue(transfer);
  messageobject *message=new messageobject(QString("encoder"), receivelist, QString("finished"), QVariant (variant));
  adv_comm_if->sendmessage(message);
  fileoperationhandle->deletefile(source);
  emit trackencodeready(destination,tracknumber);
}

const int lameprocess::gettracknumber() const
  {
    return tracknumber;
  }

const QString lameprocess::getpath() const
  {
    return destination;
  }
