/***************************************************************************
 *   Copyright (C) 2007 by Roland Weigert   *
 *   roweigert@t-online.de             *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include "ripper.h"

extern fileoperator *fileoperationhandle;
extern rcfile *settingsfile;
extern communicationinterface *comm_if;
extern advancedcommunicationinterface *adv_comm_if;

ripper::ripper(QWidget *parent):
    view(parent),
    box(new QWidget),
    dialoglayout(new QVBoxLayout),
    selectallbutton(0),
    cdrominfo(0),
    freedbconnect(0),
    discchoice(0),
    tracksdisplay(0),
    selector(TRUE),
    infolabel(0),
    lameconnect(0),
    paranoiaconnect(0)
{
  setactivebuttons(validactions::rip|validactions::upload|validactions::cdinfo);
  generatetagbox();
  generatesplitbox();
  dialoglayout->addWidget(this->tagbox,1);
  dialoglayout->addWidget(this->splitter,5);
  box->setLayout(dialoglayout);
  this->setWidget(box );
  this->setWidgetResizable( TRUE );
  connect(comm_if,SIGNAL(requestsignal(QString, QString, QVariant)),this,SLOT(requestslot(QString, QString, QVariant)));
  connect(comm_if,SIGNAL(requestsignal(QString, QString, QList<QVariant>)),this,SLOT(requestslot(QString, QString, QList<QVariant>)));
  connect(adv_comm_if,SIGNAL(requestsignal(QStringList,messageobject *)),this,SLOT(requestslot(QStringList,messageobject *)));
}

ripper::~ripper()
{
  if (cdrominfo)
    delete cdrominfo;
  if (selectallbutton)
    delete selectallbutton;
  if (freedbconnect)
    delete freedbconnect;
  if (discchoice)
    delete discchoice;
  if (tracksdisplay)
    delete tracksdisplay;
  if (infolabel)
    delete infolabel;
  if (paranoiaconnect)
    delete paranoiaconnect;
  if (lameconnect)
    delete lameconnect;
//Tagbox
  delete artistlabel;
  delete albumlabel;
  delete genrelabel;
  delete releaselabel;
  delete artistcombo;
  delete albumcombo;
  delete genrecombo;
  delete releasespinner;
  delete taglayout;
  delete tagbox;
//Splitbox
  delete covercontrols;
  delete tracklayout;
  delete trackbox;
  delete controllayout;
  delete controlbox;
  delete splitter;
  delete dialoglayout;
  delete box;
}

void ripper::generatetagbox()
{
  artistlabel=new QLabel (tr("Artist"));
  albumlabel=new QLabel (tr("Album"));
  genrelabel=new QLabel(tr("Genre"));
  releaselabel=new QLabel(tr("Release"));
  artistcombo=new QComboBox;
  artistcombo->setEditable(TRUE);
  albumcombo=new QComboBox;
  albumcombo->setEditable(TRUE);
  genrecombo=new QComboBox;
  genrecombo->setEditable(TRUE);
  releasespinner=new QSpinBox;
  releasespinner->setRange(0,9999);
  tagbox=new QGroupBox(tr("Albumtags"));
  taglayout=new QGridLayout;
  taglayout->addWidget(artistlabel,0,0);
  taglayout->addWidget(artistcombo,0,1,1,2);
  taglayout->addWidget(albumlabel,0,3);
  taglayout->addWidget(albumcombo,0,4,1,2);
  taglayout->addWidget(genrelabel,1,0);
  taglayout->addWidget(genrecombo,1,1,1,2);
  taglayout->addWidget(releaselabel,1,3);
  taglayout->addWidget(releasespinner,1,5);
  tagbox->setLayout(taglayout);
  connect (albumcombo,SIGNAL( editTextChanged(const QString)),this,SLOT(updatesearchstring(const QString)));
  connect (artistcombo,SIGNAL( editTextChanged(const QString)),this,SLOT(updatesearchstring(const QString)));
}

void ripper::generatesplitbox()
{
  splitter=new QSplitter();
  trackbox=new QGroupBox(tr("Tracklist"));
  infolabel=new QLabel(tr("<center><b>No Discinformation.</b></center>"));
  tracklayout=new QVBoxLayout;
  tracklayout->addWidget(infolabel);
  trackbox->setMinimumSize(350,300);
  trackbox->setLayout(tracklayout);
  splitter->addWidget(trackbox);
  controlbox=new QGroupBox();
  covercontrols=new coverbox(this);
  covercontrols->setEnabled(FALSE);
  controllayout=new QVBoxLayout;
  controllayout->addWidget(covercontrols);
  controlbox->setLayout(controllayout);
  splitter->addWidget(controlbox);
  splitter->setStretchFactor(0,3);
  splitter->setStretchFactor(1,1);
}

void ripper::getdiscinformation()
{
  qDebug("Fetching discinfo");
  //Clear out all combos
  selector=TRUE;
  artistcombo->setEditText(QString::null);
  albumcombo->setEditText(QString::null);
  genrecombo->setEditText(QString::null);
  releasespinner->setValue(0);
  covercontrols->clearimages(TRUE);
  covercontrols->setEnabled(FALSE);
  if (tracksdisplay)
    {
      delete tracksdisplay;
      tracksdisplay=NULL;
    }
  if (!infolabel)
    {
      infolabel=new QLabel(tr("<center><b>No Discinformation.</b></center>"));
      tracklayout->addWidget(infolabel);
    }
  if (selectallbutton)
    {
      delete selectallbutton;
      selectallbutton=NULL;
    }
  trackbox->update();
  if (cdrominfo)
    {
      delete cdrominfo;
      cdrominfo=NULL;
    }
  cdrominfo=new cdinterface;
  connect (cdrominfo,SIGNAL(cantread(QString)),this,SLOT(handleerror(QString)));
  connect (cdrominfo,SIGNAL(gottoc(const toc*)),this,SLOT(gottoc(const toc *)));
  cdrominfo->readtoc();
}

void ripper::handleerror(QString text)
{
  qDebug(qPrintable(text));
  QMessageBox::information(this,"K-YAMO-Information",text);
}

void ripper::gottoc(const toc *cdinfo)
{
  if (cdrominfo->gettracks())
    {
      //First fetch the discid
      freedb_discid=cdrominfo->calcdiscid();
      if (infolabel)
        {
          delete infolabel;
          infolabel=NULL;
        }
      covercontrols->setEnabled(TRUE);
      covercontrols->setsearchstring(QString::null);
      tracksdisplay=new tracklister(cdrominfo->gettracks(),cdinfo);
      tracklayout->addWidget(tracksdisplay);
      selectallbutton=new QPushButton(tr("Select all"));
      tracklayout->addWidget(selectallbutton);
      trackbox->update();
      freedbconnect=new freedbinterface("freedb.freedb.org");
      freedbconnect->getdisc(*cdrominfo,freedb_discid);
      connect (freedbconnect,SIGNAL(gotdisc(const QList<freedbalbum>)),this,SLOT(gotdisc(const QList<freedbalbum>))) ;
      connect (freedbconnect,SIGNAL(queryerror(const QString)),this,SLOT(handleerror(const QString))) ;
      connect (artistcombo,SIGNAL(editTextChanged(const QString)),tracksdisplay,SIGNAL(artistsignal(const QString)));
      connect (tracksdisplay,SIGNAL(checkedtracks(const int)),this,SLOT(checkselectall(const int)));
      connect (selectallbutton,SIGNAL(clicked()),this,SLOT(selectorslot()));
    }
}

void ripper::gotdisc(const QList<freedbalbum> albumresult)
{
  if (albumresult.count()==1)
    {
      disconnect (albumcombo,SIGNAL( editTextChanged(const QString)),this,SLOT(updatesearchstring(const QString)));
      disconnect (artistcombo,SIGNAL( editTextChanged(const QString)),this,SLOT(updatesearchstring(const QString)));
      albumcombo->setEditText(albumresult.first().fdb_album);
      genrecombo->setEditText(albumresult.first().fdb_genre);
      artistcombo->setEditText(albumresult.first().fdb_artist);
      releasespinner->setValue(albumresult.first().fdb_release);
      QList<freedbtrack>::ConstIterator trackswalkingpointer=albumresult.first().fdb_tracklist.begin();
      covercontrols->setsearchstring(albumresult.first().fdb_artist+"+"+albumresult.first().fdb_album);
      connect (albumcombo,SIGNAL( editTextChanged(const QString)),this,SLOT(updatesearchstring(const QString)));
      connect (artistcombo,SIGNAL( editTextChanged(const QString)),this,SLOT(updatesearchstring(const QString)));
      while (trackswalkingpointer!=albumresult.first().fdb_tracklist.end())
        {
          if (tracksdisplay)
            tracksdisplay->changetrack((*trackswalkingpointer).fdb_number,(*trackswalkingpointer).fdb_title);
          trackswalkingpointer++;
        }
    }
  else if (albumresult.count()>1)
    {
      discchoice=new discselector(albumresult);
      discchoice->show();
      connect (discchoice,SIGNAL(trysignal(const freedbalbum)),this,SLOT(trydisc(const freedbalbum)));
      connect (discchoice,SIGNAL(oksignal(const freedbalbum)),this,SLOT(setdisc(const freedbalbum)));
      connect (discchoice,SIGNAL(cancelsignal()),this,SLOT(closediscselector()));
    }
  else
    if (freedbconnect)
      {
        delete freedbconnect;
        freedbconnect=0;
      }
}

void ripper::checkselectall(const int counter)
{
  QVariant data(0);
  if (counter)
    comm_if->requestslot(QString("ripper"),QString("startenable"), data);
  else
    comm_if->requestslot(QString("ripper"),QString("startdisable"), data);
  if (cdrominfo)
    {
      if (counter==cdrominfo->getvalidtrackcount())
        {
          selectallbutton->setText(tr("Deselect all"));
          selector=FALSE;
        }
      else if (counter==0)
        {
          selectallbutton->setText(tr("Select all"));
          selector=TRUE;
        }
    }
}

void ripper::selectorslot()
{
//we must disconnect this signal in the meantime,otherwise the dialog gets confused
  disconnect (tracksdisplay,SIGNAL(checkedtracks(const int)),this,SLOT(checkselectall(const int)));
  if (tracksdisplay)
    {
      for (int i=1;i<=cdrominfo->getvalidtrackcount();i++)
        tracksdisplay->changechecked(i,selector);
    }
  if (selector)
    {
      selectallbutton->setText(tr("Deselect all"));
      selector=FALSE;
    }
  else
    {
      selectallbutton->setText(tr("Select all"));
      selector=TRUE;
    }
  QVariant data(0);
  if (!selector)
    comm_if->requestslot(QString("ripper"),QString("startenable"), data);
  else
    comm_if->requestslot(QString("ripper"),QString("startdisable"), data);
  connect (tracksdisplay,SIGNAL(checkedtracks(const int)),this,SLOT(checkselectall(const int)));
}

void ripper::startripping()
{
  if (cdrominfo)
    {
      //Check if we have still the same disc in
      cdinterface checkdisc;
      checkdisc.readtoc();
      if (!(checkdisc.calcdiscid().compare(freedb_discid)))
        {
          emit needprogress();
          tempdir=settingsfile->gettempdir();
          if (!(artistcombo->currentText().isEmpty()))
            {
              tempdir+="/"+artistcombo->currentText();
            }
          else
            tempdir+="/UnknownArtist";
          if (!(albumcombo->currentText().isEmpty()))
            {
              tempdir+="-"+albumcombo->currentText();
            }
          else
            tempdir+="-UnknownAbum";
          QDir dirobject;
          if (dirobject.mkpath(tempdir))
            {
              paranoiaconnect=new cdparanoiainterface;
              QList<selectionobject> ripselection;
              if (tracksdisplay)
                {
                  ripselection=tracksdisplay->getselection();
                  QList<selectionobject>::ConstIterator selectionwalkingpointer=ripselection.begin();
                  while (selectionwalkingpointer!=ripselection.end())
                    {
                      paranoiaconnect->readtrack((*selectionwalkingpointer).number,tempdir);
                      selectionwalkingpointer++;
                    }
                }
            }
        }
      else
        {
          //User changed the disc, or even removed it,start a real rescan
          QMessageBox::information (this,tr("Discchange"),tr("The chosen disc is not in your CDRom!"));
          getdiscinformation();
        }
    }
}

void ripper::trydisc(const freedbalbum album)
{
  albumcombo->setEditText(album.fdb_album);
  genrecombo->setEditText(album.fdb_genre);
  artistcombo->setEditText(album.fdb_artist);
  releasespinner->setValue(album.fdb_release);
  QList<freedbtrack>::ConstIterator trackswalkingpointer=album.fdb_tracklist.begin();
  while (trackswalkingpointer!=album.fdb_tracklist.end())
    {
      if (tracksdisplay)
        tracksdisplay->changetrack((*trackswalkingpointer).fdb_number,(*trackswalkingpointer).fdb_title);
      trackswalkingpointer++;
    }
}

void ripper::setdisc(const freedbalbum album)
{
  disconnect (albumcombo,SIGNAL( editTextChanged(const QString)),this,SLOT(updatesearchstring(const QString)));
  disconnect (artistcombo,SIGNAL( editTextChanged(const QString)),this,SLOT(updatesearchstring(const QString)));
  albumcombo->setEditText(album.fdb_album);
  genrecombo->setEditText(album.fdb_genre);
  artistcombo->setEditText(album.fdb_artist);
  releasespinner->setValue(album.fdb_release);
  covercontrols->setsearchstring(album.fdb_artist+"+"+album.fdb_album);
  connect (albumcombo,SIGNAL( editTextChanged(const QString)),this,SLOT(updatesearchstring(const QString)));
  connect (artistcombo,SIGNAL( editTextChanged(const QString)),this,SLOT(updatesearchstring(const QString)));
  QList<freedbtrack>::ConstIterator trackswalkingpointer=album.fdb_tracklist.begin();
  while (trackswalkingpointer!=album.fdb_tracklist.end())
    {
      if (tracksdisplay)
        tracksdisplay->changetrack((*trackswalkingpointer).fdb_number,(*trackswalkingpointer).fdb_title);
      trackswalkingpointer++;
    }
  if (discchoice)
    {
      delete discchoice;
      discchoice=0;
    }
  if (freedbconnect)
    {
      delete freedbconnect;
      freedbconnect=0;
    }
}

void ripper::closediscselector()
{
  albumcombo->setEditText(QString::null);
  genrecombo->setEditText(QString::null);
  artistcombo->setEditText(QString::null);
  releasespinner->setValue(0);
  for (int i=1;i<=cdrominfo->getvalidtrackcount();i++)
    {
      if (tracksdisplay)
        tracksdisplay->changetrack(i,QString::null);
    }
  if (discchoice)
    {
      delete discchoice;
      discchoice=0;
    }
  if (freedbconnect)
    {
      delete freedbconnect;
      freedbconnect=0;
    }
}

void ripper::encodetrack(const QString savename, const int tracknumber)
{
  if (!lameconnect)
    {
      lameconnect=new lameinterface(settingsfile->getrip_number_encoders());
      connect(lameconnect,SIGNAL(allencoded()),this,SLOT(checkdone()));
    }
  lameconnect->encodetrack(tracknumber,tempdir,settingsfile->getrip_bitrate(),settingsfile->getrip_stereotype(),TRUE,FALSE);
}

void ripper::encodeready(const QString filename,const int tracknumber)
{
  //First set the tags, than save them to filename
  mediafile *readyfile=new mediafile(filename);
  if (readyfile->issupportedfile())//This call is needed, to set classinternals right
    {
      QList<selectionobject> ripselection;
      ripselection=tracksdisplay->getselection();
      QList<selectionobject>::ConstIterator selectionwalkingpointer=ripselection.begin();
      bool found=FALSE;
      while (selectionwalkingpointer!=ripselection.end()&&!found)
        {
          if ((*selectionwalkingpointer).number==tracknumber)
            {
              found=TRUE;
              readyfile->setartistname((*selectionwalkingpointer).artist);
              readyfile->setalbumname(albumcombo->currentText());
              readyfile->setsongname ((*selectionwalkingpointer).title);
              readyfile->setgenrename(genrecombo->currentText());
              readyfile->settracknumber(tracknumber);
              readyfile->setreleasedate(releasespinner->value());
            }
          else
            selectionwalkingpointer++;
        }
      const QImage front(covercontrols->getfrontimage());
      const QImage back(covercontrols->getbackimage());
      const QImage media(covercontrols->getmediaimage());
      if (!front.isNull())
        readyfile->setpicture(front,Coverart::FrontCover);
      if (!back.isNull())
        readyfile->setpicture(back,Coverart::BackCover);
      if (!media.isNull())
        readyfile->setpicture(media,Coverart::Media);
      readyfile->savetags();
      delete readyfile;
      fileoperationhandle->movefile(filename);
    }
}

void ripper::checkdone()
{
//Check if ripper and encoder are still active, and delete them if not
  if (lameconnect&&paranoiaconnect)
    {
      if (!lameconnect->isrunning()&&!paranoiaconnect->isrunning())
        {
          delete paranoiaconnect;
          delete lameconnect;
          paranoiaconnect=0;
          lameconnect=0;
          //Check if the tempdir still exists, and delete it if not deleted by fileoperator
          //This depends on the deletemode set in rcfile
          if (settingsfile->getsav_deletemode()==fileoperations::fileonly)
            {
              QFileInfo checkstillhere(tempdir);
              if (checkstillhere.exists()&&checkstillhere.isDir()&&checkstillhere.isWritable())
                {
                  QDir deletedir(tempdir);
                  deletedir.rmdir(tempdir);
                }
            }
          emit alldone();
        }
    }
}

void ripper::updatesearchstring(const QString &)
{
  covercontrols->setsearchstring(albumcombo->currentText()+"+"+artistcombo->currentText());
}

void ripper::requestslot(QStringList destination,messageobject *message)
{
  if (destination.contains("ripper"))
    {
      message->lock();
      if (!message->getsender().compare("discread"))
        {
          if (!message->gettype().compare("finished"))
            {
	      comm_progress variantdata=message->getdata().value<comm_progress>();
              //On abort there would be no file, so check if the file is here before starting encodertry
              QFileInfo checkforfile(variantdata.filename);
              if (checkforfile.exists())
                encodetrack(variantdata.filename,variantdata.track);
            }
        }
      else if (!message->getsender().compare("encoder"))
        {
          if (!message->gettype().compare("finished"))
            {
              comm_progress variantdata=message->getdata().value<comm_progress>();
              encodeready(variantdata.filename,variantdata.track);
            }
        }
	message->unlock();
	if (!message->confirm("ripper"))
        adv_comm_if->deletemessage(message->getmessageid());
    }
}

void ripper::requestslot(QString sender,QString type, QVariant data)
{
/*  if (!sender.compare("discread"))
    {
      if (data.canConvert<comm_progress>())
        {
          comm_progress variantdata=data.value<comm_progress>();
          if (!type.compare("finished"))
            {
              encodetrack(variantdata.filename,variantdata.track);
            }
        }
    }
  else if (!sender.compare("encode"))
    {
      if (data.canConvert<comm_progress>())
        {
          comm_progress variantdata=data.value<comm_progress>();
          if (!type.compare("finished"))
            {
              encodeready(variantdata.filename,variantdata.track);
            }
        }
    }*/
  if (!sender.compare("main"))
    {
      if (data.canConvert<int>())
        {
          if (!type.compare("discinfo"))
            {
              getdiscinformation();
            }
          else if (!type.compare("rip"))
            startripping();
        }
    }
}

void ripper::requestslot(QString sender,QString type, QList<QVariant>)
{
// This is not needed in this class. (Myabe this will change later)
}
