/***************************************************************************
 *   Copyright (C) 2007 by Roland Weigert   *
 *   roweigert@t-online.de             *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
/**
 *@file ripper.h
 *
 *This file holds the class for the rippersubwindow.<br>
 *It handles all ripping itself, and gets only signals from the mainwindow.
 *@author Roland Weigert
 *@date 29-01-2009
 *@todo Make everything configurable
  */

#ifndef _RIPPER_H_
#define _RIPPER_H_

#include <QScrollArea>
#include <QGroupBox>
#include <QLayout>
#include <QWidget>
#include <QLabel>
#include <QComboBox>
#include <QSpinBox>
#include <QSplitter>
#include <QPushButton>
#include <QList>
#include <QMessageBox>
#include <QDir>
#include <QImage>

#include "global-stuff/view.h"
#include "freedb-interface/freedbinterface.h"
#include "ripper/cdinterface.h"
#include "coverbox/coverbox.h"
#include "ripper/discselector.h"
#include "ripper/tracklister.h"
#include "ripper/lameinterface.h"
#include "tag-interfaces/mediafile.h"
#include "ripper/cdparanoiainterface.h"
#include "settings/rcfile.h"
#include "global-stuff/communicationinterface.h"

class ripper : public view
  {
    Q_OBJECT
  public:
    ripper(QWidget *parent);
    ~ripper();
  private:
    void generatetagbox();
    void generatesplitbox();
  private slots:
    void requestslot(QString sender,QString type, QVariant data);
    void requestslot(QString sender,QString type, QList<QVariant> data);
  private slots:
    void gotdisc(const QList<freedbalbum> albumresult);
    void getdiscinformation();
    void handleerror(QString);
    void gottoc(const toc *);
    void checkselectall(const int);
    void selectorslot();
    void startripping();
    void trydisc(const freedbalbum album);
    void setdisc(const freedbalbum album);
    void closediscselector();
    void encodetrack(const QString savename, const int tracknumber);
    void encodeready(const QString filename,const int tracknumber);
    void checkdone();
    void updatesearchstring(const QString &);
    void requestslot(QStringList,messageobject *);
	  signals:
    void trackreadready(const QString savename,const int track);
    void trackencodeready(const QString savename,const int track);
    void trackreadprogress(const int value,const int track);
    void trackencodeprogress(const int value,const int track);
    void cancelencode(const int);
    void cancelread(const int);
    void cancelall();
    void alldone();
    void needprogress();
  private:
    QWidget *box;
    QVBoxLayout *dialoglayout;
//Tagboxpart
    QGroupBox *tagbox;
    QGridLayout *taglayout;
    QLabel *artistlabel;
    QLabel *albumlabel;
    QLabel *genrelabel;
    QLabel *releaselabel;
    QComboBox *artistcombo;
    QComboBox *albumcombo;
    QComboBox *genrecombo;
    QSpinBox *releasespinner;
//Splitboxpart
    QSplitter *splitter;
    QGroupBox *trackbox;
    QVBoxLayout *tracklayout;
    QPushButton *selectallbutton;
    QGroupBox *controlbox;
    QVBoxLayout *controllayout;
    coverbox *covercontrols;
//Needed Objects
    cdinterface *cdrominfo;
    QString freedb_discid;
    freedbinterface *freedbconnect;
    discselector *discchoice;
    tracklister *tracksdisplay;
    bool selector;
    QLabel *infolabel;
    lameinterface *lameconnect;
    QString tempdir;
    cdparanoiainterface *paranoiaconnect;
  };


#endif
