/***************************************************************************
 *   Copyright (C) 2007 by Roland Weigert   *
 *   roweigert@t-online.de             *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

/**
 *@file trackitem.h
 *The class that handles every information stored by a track<br>
 *
 *@author Roland Weigert
 *@date 29-07-2007
 */

#ifndef _TRACKITEM_H_
#define _TRACKITEM_H_

#include <QGroupBox>
#include <QString>
#include <QCheckBox>
#include <QLineEdit>
#include <QLabel>
#include <QHBoxLayout>

/**
*@brief The trackitem-class
*
*This class manages the Trackinformation of one track in the ripperdialog.<br>
*/
class trackitem : public QGroupBox
  {
    Q_OBJECT
  public:
    /**
    *@brief Constructor
    *
    *Creates a new trackitem out of the information it gets.
    *@param number The Tracknumber that is displayed
    *@param mode The mode of the Track (0=Audio, all others disable the trackitem, and set standard-Text)
    *@param time The timestring that is to display.
    */
    trackitem(int number,int mode,QString time);
    /**
    *@brief Destructor
    */
    ~trackitem();
    /**
    *@brief The changetitle-function
    *
    *Use this to set the title for the trackitem.
    *@param newname The new Tracktitle
    */
    void changetitle(const QString newname);
    /**
    *@brief the Title getterfunction
    *@return A QString with the tracktitle of trackitem.
    */
    const QString gettitle() const;
    /**
    *@brief the Artist getterfunction
    *@return QString with the tracks artist
    */
    const QString getartist() const;
    /**
    *@brief the checkbox getterfucction
    *@return int with with value if checked or not
    */
    const bool getchecked() const;
    /**
    *@brief The setterfunction for checkbox
    *@param check The new state of the checkbox
    */
    void setchecked(const bool check);
  private:
    QLabel *tracknumber;
    QCheckBox *readbox;
    QLineEdit *artistledit;
    QLineEdit *titleledit;
    QLabel *songtime;
    QHBoxLayout *layout;
  public slots:
    /**
    *@brief Call this slot to set the artist of the trackitem.
    *
    *Use this Slot to set the artist for all trackitems in the tracklister at once.
    */
    void changeartist(const QString newname);
    signals:
    /**
    *@brief This signal is emitted, when the state of the checkbox changes
    */
    void checkboxchanged();
  };

#endif
