/***************************************************************************
 *   Copyright (C) 2007 by Roland Weigert   *
 *   roweigert@t-online.de             *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include "tracklister.h"

tracklister::tracklister(int trackcount,const toc *cdinfo):
    box(new QWidget),
    layout(new QVBoxLayout),
    header1(new  QLabel(tr("Rip"))),
    header2(new  QLabel(tr("Nr."))),
    header3(new  QLabel(tr("Artist"))),
    header4(new  QLabel(tr("Title"))),
    header5(new  QLabel(tr("Time"))),
    headerlayout(new QHBoxLayout)

{
  layout->setMargin(3);
  layout->setSpacing(3);
  headerlayout->addWidget(header1,1);
  headerlayout->addWidget(header2,1);
  headerlayout->addWidget(header3,3);
  headerlayout->addWidget(header4,5);
  headerlayout->addWidget(header5,1);
  layout->addLayout(headerlayout);
  //Generate the needed Trackitems
  for (int i=1;i<=trackcount;i++)
    {
      long duration=cdinfo[i].frameoffset-cdinfo[i-1].frameoffset;
      QString timeinfo;
      int minutes=duration/4500;
      int seconds=(duration-(minutes*4500))/75;
      if (minutes<10)
        timeinfo=QString("0%1").arg(QString::number(minutes));
      else
        timeinfo=QString("%1").arg(QString::number(minutes));
      if (seconds<10)
        timeinfo=QString("%1:0%2").arg(timeinfo,QString::number(seconds));
      else
        timeinfo=QString("%1:%2").arg(timeinfo,QString::number(seconds));
      int mode=cdinfo[i-1].mode;//mode 0=audio,mode 4=data
      trackitem *newitem=new trackitem(i,mode,timeinfo);
      layout->addWidget(newitem);
      tracklist.append(newitem);
      connect (this,SIGNAL(artistsignal(const QString)),newitem,SLOT(changeartist(const QString)));
      connect (newitem,SIGNAL(checkboxchanged()),this,SLOT(checkallboxes()));
    }
  layout->addStretch(5);
  box->setLayout(layout);
  this->setWidget( box );
  this->setWidgetResizable( TRUE );
}

tracklister::~tracklister()
{
//First clear out the trackitem-widgets
  if (tracklist.count())
    {
      QList<trackitem *>::Iterator tracksindex=tracklist.begin();
      while (tracksindex!=tracklist.end())
        {
          delete (*tracksindex);
          tracksindex++;
        }
    }
  delete header1;
  delete header2;
  delete header3;
  delete header4;
  delete header5;
  delete headerlayout;
  delete layout;
  delete box;
}

void tracklister::changetrack (int number,const QString newtitle)
{
//For safety validate that we have enough tracks
  if (tracklist.count()>=(number))
    {
      tracklist.at(number-1)->changetitle(newtitle);
    }
}

void tracklister::changechecked(const int number,const int checked)
{
//For safety validate that we have enough tracks
  if (tracklist.count()>=(number))
    {
      tracklist.at(number-1)->setchecked(checked);
    }
}

void tracklister::checkallboxes()
{
//Check through all trackitems, wether they are check or not, and emit checkedtracks-signal with the count of them
  int counter=0;
  QList<trackitem *>::Iterator trackswalkingpointer=tracklist.begin();
  while (trackswalkingpointer!=tracklist.end())
    {
      counter+=(*trackswalkingpointer)->getchecked();
      trackswalkingpointer++;
    }
  emit checkedtracks(counter);
}

const QList<selectionobject> tracklister::getselection()
{
//Check through all trackitems, and get the numbers of selected ones
  QList<selectionobject> selectionlist;
  QList<trackitem *>::Iterator trackswalkingpointer=tracklist.begin();
  int counter=1;
  while (trackswalkingpointer!=tracklist.end())
    {
      if ((*trackswalkingpointer)->getchecked())
        {
          selectionobject newselection;
          newselection.number=counter;
          newselection.artist=(*trackswalkingpointer)->getartist();
          newselection.title=(*trackswalkingpointer)->gettitle();
          selectionlist.append(newselection);
        }
      trackswalkingpointer++;
      counter++;
    }
  return selectionlist;
}

