/***************************************************************************
 *   Copyright (C) 2007 by Roland Weigert   *
 *   roweigert@t-online.de             *
 *                                                                         *
 *   This program is free software you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include "settings/rcfile.h"

extern fileoperator *fileoperationhandle;

rcfile::rcfile():
    tempdir(QDir::homePath()+"/KYamo/temp"),
    workingdir(QDir::homePath()),
    destinationdir(QDir::homePath()+"/KYamo/organised"),
    filehandle(new QFile(QString("%1/.kyamorc.xml").arg(QDir::homePath()))),
    verbose(false),
    db_type(database::none),
    db_autocompleter(false),
    sav_tagengine(tagengine::taglib),
    sav_tagtype(tagversion::v1tag|tagversion::v2tag),
    sav_filter("#GENRE#/#BAND#/#ALBUM#/#TRACK#-#SONG#"),
    sav_picturename(QString("cover")),
    sav_picture(true),
    sav_picturetype(picturetype::bmp),
    sav_picturesize(150),
    sav_ovmode(fileoperations::checkbitrate),
    sav_deletemode(fileoperations::directoryrecursive),
    sav_move(false),
    rip_bitrate(192),
    rip_stereotype(audiomode::Stereo),
    rip_encoder(encoder::lame),
    rip_lame_options("-h"),
    rip_gogo_options("-h"),
    rip_blade_options("-nogap -prio=HIGHEST"),
    rip_oggenc_options(QString::null),
    rip_number_encoders(3),
    conv_bitrate(128),
    conv_stereotype(audiomode::Stereo),
    conv_encoder(encoder::lame),
    net_usefreedb(false),
    net_freedbserver("freedb.freedb.org"),
    net_freedbuser("kyamouser"),
    net_usemusicbrainz(false)
{
  searchpathes<<"/usr/bin"<<"/usr/sbin";
  sav_ignorelist<<"cover.bmp"<<"folder.png";
}

rcfile::~rcfile()
{
  if (filehandle->isOpen())
    filehandle->close();
  delete filehandle;
}

bool rcfile::load()
{
  bool ok=false;
  if (filehandle->open(QFile::ReadOnly|QFile::Text))
    {
      QString errorstring;
      int errorline=0;
      int errorcolumn=0;
      QDomDocument file;
      if (!file.setContent(filehandle,true,&errorstring,&errorline,&errorcolumn))
        {
          filewarning(QString(tr("Your settingsfile is corrupt.<br>Opening failed because %1 at line %2 column %3<br>Using defaultsettings instead!")).arg(errorstring,QString::number(errorline),QString::number(errorcolumn)));
          ok=false;
        }
      else
        {
//Ok we can try to load load the settings
          ok=true;
          QDomElement rootelem=file.documentElement();
          if (rootelem.tagName()=="KYamo")
            {
              QDomElement baselem=rootelem.firstChildElement();
              if (baselem.tagName()=="Settings")
                {
                  QDomElement settingselem=baselem.firstChildElement();
                  while (!settingselem.isNull())
                    {
                      if (settingselem.tagName()=="Directories")
                        {
                          QDomElement direlem=settingselem.firstChildElement();
                          while (!direlem.isNull())
                            {
                              if (direlem.tagName()=="Temdir")
                                settempdir(direlem.attribute("name"));
                              if (direlem.tagName()=="Workdir")
                                setworkingdir(direlem.attribute("name"));
                              if (direlem.tagName()=="Destdir")
                                setdestinationdir(direlem.attribute("name"));
                              if (direlem.tagName()=="Searchpath")
                                {
                                  QStringList searchlist;
                                  QDomElement pathelem=direlem.firstChildElement();
                                  while (!pathelem.isNull())
                                    {
                                      if (pathelem.tagName()=="Path")
                                        searchlist<<(pathelem.attribute("name"));
                                      pathelem=pathelem.nextSiblingElement();
                                    }
                                  setsearchpathes(searchlist);
                                }
                              direlem=direlem.nextSiblingElement();
                            }
                        }
                      else if (settingselem.tagName()=="Global")
                        {
                          QDomElement globalelem=settingselem.firstChildElement();
                          while (!globalelem.isNull())
                            {
                              if (globalelem.tagName()=="Databse")
                                {

                                  if (globalelem.attribute("type")=="none")
                                    setdb_type(database::none);
                                  else if (globalelem.attribute("type")=="mysql")
                                    setdb_type(database::mysql);
                                  else if (globalelem.attribute("type")=="postgresql")
                                    setdb_type(database::postgresql);
                                  else if (globalelem.attribute("type")=="sqllite")
                                    setdb_type(database::sqllite);
                                  if (globalelem.attribute("completer")=="true")
                                    setdb_autocompleter(true);
                                  else
                                    setdb_autocompleter(false);
                                }
                              globalelem=globalelem.nextSiblingElement();
                            }
                        }
                      else if (settingselem.tagName()=="Saving")
                        {
                          QDomElement savingelem=settingselem.firstChildElement();
                          while (!savingelem.isNull())
                            {
                              if (savingelem.tagName()=="Tagengine")
                                {
                                  if (savingelem.attribute("name")=="taglib")
                                    setsav_tagengine(tagengine::taglib);
                                  else if (savingelem.attribute("name")=="id3lib")
                                    setsav_tagengine(tagengine::id3lib);
                                }
                              else if (savingelem.tagName()=="Tagversion")
                                {
                                  int tversion=savingelem.attribute("value").toInt();
                                  setsav_tagtype(tversion);
                                }
                              else if (savingelem.tagName()=="Filter")
                                setsav_filter(savingelem.attribute("name"));
                              else if (savingelem.tagName()=="Picture")
                                {
                                  if (savingelem.attribute("save")=="true")
                                    setsav_picture(true);
                                  else if (savingelem.attribute("save")=="false")
                                    setsav_picture(false);
                                  if (savingelem.attribute("type")=="bmp")
                                    setsav_picturetype(picturetype::bmp);
                                  else if (savingelem.attribute("type")=="jpg")
                                    setsav_picturetype(picturetype::jpg);
                                  else if (savingelem.attribute("type")=="png")
                                    setsav_picturetype(picturetype::png);
                                  else if (savingelem.attribute("type")=="gif")
                                    setsav_picturetype(picturetype::gif);
                                  setsav_picturename(savingelem.attribute("name"));
                                  setsav_picturesize(savingelem.attribute("size").toInt());
                                }
                              else if (savingelem.tagName()=="Overwrite")
                                {
                                  if (savingelem.attribute("mode")=="Checkbitrate")
                                    setsav_ovmode(fileoperations::checkbitrate);
                                  else if (savingelem.attribute("mode")=="Keepold")
                                    setsav_ovmode(fileoperations::keepold);
                                  else if (savingelem.attribute("mode")=="Keepnew")
                                    setsav_ovmode(fileoperations::keepnew);
                                  else if (savingelem.attribute("mode")=="Ask")
                                    setsav_ovmode(fileoperations::ask);
                                }
                              else if (savingelem.tagName()=="Delete")
                                {
                                  if (savingelem.attribute("mode")=="Fileonly")
                                    setsav_deletemode(fileoperations::fileonly);
                                  else if (savingelem.attribute("mode")=="Directory")
                                    setsav_deletemode(fileoperations::directory);
                                  else if (savingelem.attribute("mode")=="Directoryrecursive")
                                    setsav_deletemode(fileoperations::directoryrecursive);
                                }
                              else if (savingelem.tagName()=="Move")
                                {
                                  if (savingelem.attribute("value")=="true")
                                    setsav_move(true);
                                  else if (savingelem.attribute("value")=="false")
                                    setsav_move(false);
                                }
                              else if (savingelem.tagName()=="Ignore")
                                {
                                  QStringList ignorelist;
                                  QDomElement pathelem=savingelem.firstChildElement();
                                  while (!pathelem.isNull())
                                    {
                                      if (pathelem.tagName()=="File")
                                        ignorelist<<(pathelem.attribute("name"));
                                      pathelem=pathelem.nextSiblingElement();
                                    }
                                  setsav_ignorelist(ignorelist);
                                }
                              savingelem=savingelem.nextSiblingElement();
                            }
                        }
                      else if (settingselem.tagName()=="Ripper")
                        {
                          QDomElement ripperelem=settingselem.firstChildElement();
                          while (!ripperelem.isNull())
                            {
                              if (ripperelem.tagName()=="Bitrate")
                                setrip_bitrate(ripperelem.attribute("value").toInt());
                              if (ripperelem.tagName()=="Stereo")
                                {
                                  if (ripperelem.attribute("type")=="Stereo")
                                    setrip_stereotype(audiomode::Stereo);
                                  else if (ripperelem.attribute("type")=="JointStereo")
                                    setrip_stereotype(audiomode::JointStereo);
                                  else if (ripperelem.attribute("type")=="SingleChannel")
                                    setrip_stereotype(audiomode::SingleChannel);
                                }
                              else if (ripperelem.tagName()=="Encoder")
                                {
                                  if (ripperelem.attribute("name")=="lame")
                                    setrip_encoder(encoder::lame);
                                  else if (ripperelem.attribute("name")=="gogo")
                                    setrip_encoder(encoder::gogo);
                                  else if (ripperelem.attribute("name")=="oggenc")
                                    setrip_encoder(encoder::oggenc);
                                  else if (ripperelem.attribute("name")=="bladeenc")
                                    setrip_encoder(encoder::bladeenc);
                                }
                              else if (ripperelem.tagName()=="Lame")
                                setrip_lame_options(ripperelem.attribute("options"));
                              else if (ripperelem.tagName()=="Gogo")
                                setrip_gogo_options(ripperelem.attribute("options"));
                              else if (ripperelem.tagName()=="Ogg")
                                setrip_oggenc_options(ripperelem.attribute("options"));
                              else if (ripperelem.tagName()=="Blade")
                                setrip_blade_options(ripperelem.attribute("options"));
                              else if (ripperelem.tagName()=="Parallel")
                                setrip_number_encoders(ripperelem.attribute("count").toInt());
                              ripperelem=ripperelem.nextSiblingElement();
                            }
                        }
                      settingselem=settingselem.nextSiblingElement();
                    }
                }
              else
                {
                  filewarning(QString("Your settingsfile has the wrong syntax.<br>The baseelemt is not <Settings>!"));
                  ok=false;
                }
            }
          else
            {
              filewarning(QString("Your settingsfile has the wrong syntax.<br>The rootelemt is not <KYamo>!"));
              ok=false;
            }
        }
      filehandle->close();
    }
  else
    {
      QMessageBox::information(0,tr("K-Yamo-Information"),QString(tr("You dont have a settingsfile yet.<br>Generating defaultconfiguration and saving it.")));
//The standard-values are all setup in Constructor
      ok=save();
    }
  return ok;
}

bool rcfile::save()
{
  if (filehandle->open(QFile::WriteOnly|QFile::Text))
    {
      filehandle->write(QString("<?xml version='1.0' encoding='UTF-8' standalone='no'?>\n"
                                "<!-- Settingsfile for K-Yamo -->\n"
                                "<KYamo>\n  <Settings>\n").toAscii());
      //Directories
      filehandle->write((QString("    <Directories>\n"
                                 "      <Tempdir name='%1' />\n"
                                 "      <Workdir name='%2' />\n"
                                 "      <Destdir name='%3' />\n"
                                 "      <Searchpath>\n").arg(gettempdir(),getworkingdir(),getdestinationdir())).toAscii());
      for (int i=0;i<searchpathes.count();i++)
        {
          filehandle->write((QString("        <Path name='%1' />\n").arg(searchpathes.at(i))).toAscii());
        }
      filehandle->write(QString("      </Searchpath>\n"
                                "    </Directories>\n").toAscii());
      //Global
      QString dbstring;
      if (db_type==database::none)
        dbstring="none";
      if (db_type==database::mysql)
        dbstring="mysql";
      if (db_type==database::postgresql)
        dbstring="postgresql";
      if (db_type==database::sqllite)
        dbstring="sqllite";
      QString completerstring;
      if (db_autocompleter)
        completerstring=QString("true");
      else
        completerstring=QString("false");

      filehandle->write((QString("    <Global>\n"
                                 "      <Database type='%1' completer='%2'/>\n"
                                 "    </Global>\n").arg(dbstring,completerstring)).toAscii());
      //Saving
      QString tagenginestring;
      if (sav_tagengine==tagengine::taglib)
        tagenginestring=QString("taglib");
      else
        tagenginestring=QString("id3lib");
      filehandle->write((QString("    <Saving>\n"
                                 "      <Tagengine name='%1' />\n"
                                 "      <Tagversion  value='%2' />\n"
                                 "      <Filter name='%3' />\n").arg(tagenginestring,QString::number(sav_tagtype),sav_filter)).toAscii());

      QString picsave;
      if (sav_picture)
        picsave=QString("true");
      else
        picsave=QString("false");
      QString pictype;
      if (sav_picturetype==picturetype::bmp)
        pictype=QString("bmp");
      else 	if (sav_picturetype==picturetype::png)
        pictype=QString("png");
      else 	if (sav_picturetype==picturetype::jpg)
        pictype=QString("jpg");
      else 	if (sav_picturetype==picturetype::gif)
        pictype=QString("gif");
      filehandle->write((QString("      <Picture save='%1' type='%2' name='%3' size='%4' />\n").arg(picsave,pictype,sav_picturename,QString::number(sav_picturesize))).toAscii());
      QString ovstring;
      if (sav_ovmode==fileoperations::keepold)
        ovstring=QString("Keepold");
      else if (sav_ovmode==fileoperations::keepnew)
        ovstring=QString("Keepnew");
      else if (sav_ovmode==fileoperations::checkbitrate)
        ovstring=QString("Checkbitrate");
      else if (sav_ovmode==fileoperations::ask)
        ovstring=QString("Ask");
      QString delstring;
      if (sav_deletemode==fileoperations::fileonly)
        delstring=QString("Fileonly");
      else if (sav_deletemode==fileoperations::directory)
        delstring=QString("Directory");
      else if (sav_deletemode==fileoperations::directoryrecursive)
        delstring=QString("Directoryrecursive");
      QString movestring;
      if (sav_move)
        movestring=QString("true");
      else
        movestring=QString("false");
      filehandle->write((QString("      <Overwrite mode='%1' />\n"
                                 "      <Delete mode='%2' />\n"
                                 "      <Move value='%3' />\n"
                                 "      <Ignore>\n").arg(ovstring,delstring,movestring)).toAscii());
      for (int i=0;i<sav_ignorelist.count();i++)
        filehandle->write((QString("        <File name='%1' />\n").arg(sav_ignorelist.at(i))).toAscii());
      filehandle->write(QString("      </Ignore>\n"
                                "    </Saving>\n").toAscii());
      //Ripper
      QString stereostring;
      if (rip_stereotype==audiomode::Stereo)
        stereostring=QString("Stereo");
      else if (rip_stereotype==audiomode::JointStereo)
        stereostring=QString("JointStereo");
      else if (rip_stereotype==audiomode::SingleChannel)
        stereostring=QString("SingleChannel");
      QString encoderstring;
      if (rip_encoder==encoder::lame)
        encoderstring=QString("lame");
      if (rip_encoder==encoder::gogo)
        encoderstring=QString("gogo");
      if (rip_encoder==encoder::oggenc)
        encoderstring=QString("oggenc");
      if (rip_encoder==encoder::bladeenc)
        encoderstring=QString("bladeenc");
      filehandle->write((QString("    <Ripper>\n"
                                 "      <Bitrate value='%1' />\n"
                                 "      <Stereo type='%2' />\n"
                                 "      <Encoder name='%3' />\n"
                                 "      <Parallel count='%4' />\n").arg(QString::number(rip_bitrate),stereostring,encoderstring,QString::number(rip_number_encoders))).toAscii());
      filehandle->write((QString("      <Lame options='%1' />\n"
                                 "      <Gogo options='%2' />\n"
                                 "      <Ogg options='%3' />\n"
                                 "      <Blade options='%4' />\n"
                                 "    </Ripper>\n").arg(rip_lame_options,rip_gogo_options,rip_oggenc_options,rip_blade_options)).toAscii());
      filehandle->write(QString("  </Settings>\n</KYamo>\n").toAscii());
      filehandle->close();
    }
}

const QString rcfile::gettempdir() const
  {
    return tempdir;
  }

const QString rcfile::getworkingdir() const
  {
    return workingdir;
  }

const QString rcfile::getdestinationdir() const
  {
    return destinationdir;
  }

const QStringList rcfile::getsearchpathes() const
  {
    return searchpathes;
  }

const bool rcfile::getverbose()const
  {
    return verbose;
  }

const int rcfile::getdb_type() const
  {
    return db_type;
  }

const bool rcfile::getdb_autocompleter() const
  {
    return db_autocompleter;
  }

const int rcfile::getsav_tagtype() const
  {
    return sav_tagtype;
  }

const QString rcfile::getsav_filter() const
  {
    return sav_filter;
  }

const QString rcfile::getsav_picturename() const
  {
    return sav_picturename;
  }

const bool rcfile::getsav_picture() const
  {
    return sav_picture;
  }

const int rcfile::getsav_picturetype() const
  {
    return sav_picturetype;
  }

const int rcfile::getsav_picturesize() const
  {
    return sav_picturesize;
  }

const int rcfile::getsav_ovmode() const
  {
    return sav_ovmode;
  }

const int rcfile::getsav_deletemode() const
  {
    return sav_deletemode;
  }

const bool rcfile::getsav_move() const
  {
    return sav_move;
  }

const QStringList rcfile::getsav_ignorelist() const
  {
    return sav_ignorelist;
  }

const int rcfile::getsav_tagengine() const
  {
    return sav_tagengine;
  }

const int rcfile::getrip_bitrate() const
  {
    return rip_bitrate;
  }

const int rcfile::getrip_stereotype() const
  {
    return rip_stereotype;
  }

const int rcfile::getrip_encoder() const
  {
    return rip_encoder;
  }

const QString rcfile::getrip_lame_options() const
  {
    return rip_lame_options;
  }

const QString rcfile::getrip_gogo_options() const
  {
    return rip_gogo_options;
  }

const QString rcfile::getrip_blade_options() const
  {
    return rip_blade_options;
  }

const QString rcfile::getrip_oggenc_options() const
  {
    return rip_oggenc_options;
  }

const int  rcfile::getrip_number_encoders()const
  {
    return rip_number_encoders;
  }

const int rcfile::getconv_bitrate() const
  {
    return conv_bitrate;
  }

const int rcfile::getconv_stereotype() const
  {
    return conv_stereotype;
  }

const int rcfile::getconv_encoder() const
  {
    return conv_encoder;
  }

const bool rcfile::getnet_usefreedb() const
  {
    return net_usefreedb;
  }

const QString rcfile::getnet_freedbserver() const
  {
    return net_freedbserver;
  }

const QString rcfile::getnet_freedbuser() const
  {
    return net_freedbuser;
  }

const bool rcfile::getnet_usemusicbrainz() const
  {
    return net_usemusicbrainz;
  }

//All setterfunctions
void rcfile::settempdir(const QString value)
{
  tempdir=value;
}

void rcfile::setworkingdir(const QString value)
{
  workingdir=value;
}

void rcfile::setdestinationdir(const QString value)
{
  destinationdir=value;
}

void rcfile::setsearchpathes(const QStringList list)
{
  searchpathes.clear();
  searchpathes<<list;
}

void rcfile::setverbose(const bool value)
{
  verbose=value;
}

void rcfile::setdb_type(const int value)
{
  db_type=value;
}

void rcfile::setdb_autocompleter(const bool value)
{
  db_autocompleter=value;
}

void rcfile::setsav_tagtype(const int value)
{
  sav_tagtype=value;
}

void rcfile::setsav_filter(const QString value)
{
  sav_filter=value;
  if (fileoperationhandle)
    fileoperationhandle->setfilter(value);
}

void rcfile::setsav_picturename(const QString value)
{
  sav_picturename=value;
}

void rcfile::setsav_picture(const bool value)
{
  sav_picture=value;
}

void rcfile::setsav_picturetype(const int value)
{
  sav_picturetype=value;
}

void rcfile::setsav_picturesize(const int value)
{
  sav_picturesize=value;
}

void rcfile::setsav_ovmode(const int value)
{
  sav_ovmode=value;
}

void rcfile::setsav_deletemode(const int value)
{
  sav_deletemode=value;
}

void rcfile::setsav_move(const bool value)
{
  sav_move=value;
}

void rcfile::setsav_ignorelist(const QStringList list)
{
  sav_ignorelist.clear();
  sav_ignorelist<<list;
  if (fileoperationhandle)
    fileoperationhandle->setignorelist(sav_ignorelist);
}

void rcfile::setsav_tagengine(const int tagengine)
{
  sav_tagengine=tagengine;
}

void rcfile::setrip_bitrate(const int value)
{
  rip_bitrate=value;
}

void rcfile::setrip_stereotype(const int value)
{
  rip_stereotype=value;
}

void rcfile::setrip_encoder(const int value)
{
  rip_encoder=value;
}

void rcfile::setrip_lame_options(const QString value)
{
  rip_lame_options=value;
}

void rcfile::setrip_gogo_options(const QString value)
{
  rip_gogo_options=value;
}

void rcfile::setrip_blade_options(const QString value)
{
  rip_blade_options=value;
}

void rcfile::setrip_oggenc_options(const QString value)
{
  rip_oggenc_options=value;
}

void rcfile::setrip_number_encoders(const int value)
{
  rip_number_encoders=value;
}

void rcfile::setconv_bitrate(const int value)
{
  conv_bitrate=value;
}

void rcfile::setconv_stereotype(const int value)
{
  conv_stereotype=value;
}

void rcfile::setconv_encoder(const int value)
{
  conv_encoder=value;
}

void rcfile::setnet_usefreedb(const bool value)
{
  net_usefreedb=value;
}

void rcfile::setnet_freedbserver(const QString value)
{
  net_freedbserver=value;
}

void rcfile::setnet_freedbuser(const QString value)
{
  net_freedbuser=value;
}

void rcfile::setnet_usemusicbrainz(const bool value)
{
  net_usemusicbrainz=value;
}

void rcfile::filewarning(const QString text)
{
  QMessageBox::warning(0,tr("K-Yamo-Warning"),QString(tr("%1<br>Check the file <b>.kyamorc.xml</b> in your homedirectory")).arg(text));
}
