/***************************************************************************
 *   Copyright (C) 2007 by Roland Weigert   *
 *   roweigert@t-online.de             *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

/**
 *@file rcfile.h
 *
 *This file holds the class that stores the settings.
 *There is one instance only in the whole programm.
 *So it is designed as pure singleton.
 *The one and only instance is initiated in main, and used by all classes.
 *
 *It also checks, if needed aplications are installed, so other classes can ask this object<br>
 *If some featrues have to be hidden.
 *@author Roland Weigert
 *@date 26-02-2009
 *@todo Write all functions
 *@todo Write the documentation (Much functions are a lot of work)
 */

#ifndef _RCFILE_H_
#define _RCFILE_H_

#include <QObject>
#include <QString>
#include <QStringList>
#include <QDir>
#include <QDomDocument>
#include <QMessageBox>
#include <QTextStream>

#include "global-stuff/globals.h"
#include "global-stuff/fileoperator.h"

class rcfile : public QObject
  {
    Q_OBJECT
  public:
    rcfile();
    ~rcfile();
    bool load();
    bool save();
    //All getterfunctions
    const QString gettempdir() const;
    const QString getworkingdir() const;
    const QString getdestinationdir() const;
    const QStringList getsearchpathes() const;
    const bool getverbose()const;
    const int getdb_type() const;
    const bool getdb_autocompleter() const;
    const int getsav_tagengine() const;
    const int getsav_tagtype() const;
    const QString getsav_filter() const;
    const QString getsav_picturename() const;
    const bool getsav_picture() const;
    const int getsav_picturetype() const;
    const int getsav_picturesize() const;
    const int getsav_ovmode() const;
    const int getsav_deletemode() const;
    const bool getsav_move() const;
    const QStringList getsav_ignorelist() const;
    const int getrip_bitrate() const;
    const int getrip_stereotype() const;
    const int getrip_encoder() const;
    const QString getrip_lame_options() const;
    const QString getrip_gogo_options() const;
    const QString getrip_blade_options() const;
    const QString getrip_oggenc_options() const;
    const int getrip_number_encoders() const;
    const int getconv_bitrate() const;
    const int getconv_stereotype() const;
    const int getconv_encoder() const;
    const bool getnet_usefreedb() const;
    const QString getnet_freedbserver() const;
    const QString getnet_freedbuser() const;
    const bool getnet_usemusicbrainz() const;
    //All setterfunctions
    void settempdir(const QString value);
    void setworkingdir(const QString value);
    void setdestinationdir(const QString value);
    void setsearchpathes(const QStringList list);
    void setverbose(const bool value);
    void setdb_type(const int value);
    void setdb_autocompleter(const bool value);
    void setsav_tagengine(const int tagengine);
    void setsav_tagtype(const int value);
    void setsav_filter(const QString value);
    void setsav_picturename(const QString value);
    void setsav_picture(const bool value);
    void setsav_picturetype(const int value);
    void setsav_picturesize(const int value);
    void setsav_ovmode(const int value);
    void setsav_deletemode(const int value);
    void setsav_move(const bool value);
    void setsav_ignorelist(const QStringList list);
    void setrip_bitrate(const int value);
    void setrip_stereotype(const int value);
    void setrip_encoder(const int value);
    void setrip_lame_options(const QString value);
    void setrip_gogo_options(const QString value);
    void setrip_blade_options(const QString value);
    void setrip_oggenc_options(const QString value);
    void setrip_number_encoders(const int);
    void setconv_bitrate(const int value);
    void setconv_stereotype(const int value);
    void setconv_encoder(const int value);
    void setnet_usefreedb(const bool value);
    void setnet_freedbserver(const QString value);
    void setnet_freedbuser(const QString value);
    void setnet_usemusicbrainz(const bool value);
  private:
    void filewarning(const QString);
//Directories
    QString tempdir;
    QString workingdir;
    QString destinationdir;
    QStringList searchpathes;
    QFile *filehandle;
    bool verbose;
//Database
    int db_type;
    bool db_autocompleter;
//Saving
    int sav_tagengine;
    int sav_tagtype;
    QString sav_filter;
    QString sav_picturename;
    bool sav_picture;
    int sav_picturetype;
    int sav_picturesize;
    int sav_ovmode;
    int sav_deletemode;
    bool sav_move;
    QStringList sav_ignorelist;
//Ripper
    int rip_bitrate;
    int rip_stereotype;
    int rip_encoder;
    QString rip_lame_options;
    QString rip_gogo_options;
    QString rip_blade_options;
    QString rip_oggenc_options;
    int rip_number_encoders;
//Converter
    int conv_bitrate;
    int conv_stereotype;
    int conv_encoder;
//Internet
    bool net_usefreedb;
    QString net_freedbserver;
    QString net_freedbuser;
    bool net_usemusicbrainz;
  };

#endif
