/***************************************************************************
 *   Copyright (C) 2007 by Roland Weigert   *
 *   roweigert@t-online.de   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include "settings/settingsdialog.h"

extern rcfile *settingsfile;
extern fileoperator *fileoperationhandle;

settings::settings(QWidget *parent):
    view(parent),
    dialoglayout(new QVBoxLayout)
{
//Selectiontree
  splitter=new QSplitter(this);
  splitter->setOrientation(Qt::Horizontal);
  leftarea=new QScrollArea(splitter);
  leftarea->setWidgetResizable(TRUE);
  selector=new QListWidget(splitter);
  selector->setIconSize(QSize(32,32));
  leftarea->setWidget(selector);
  new QListWidgetItem ( QIcon(":/icons/filesave.png"), tr("Saving Options"),selector);
  new QListWidgetItem ( QIcon(":icons/fileopen.png"), tr("Directory Options"),selector);
  new QListWidgetItem ( QIcon(":/icons/cdaudio.png"), tr("Ripper Options"),selector);
  new QListWidgetItem ( QIcon(":/icons/internet.png"), tr("Internet Options"),selector);
  new QListWidgetItem ( QIcon(":/icons/player_play.png"), tr("Player Options"),selector);
  new QListWidgetItem ( QIcon(":/icons/recode.png"), tr("Recoder Options"),selector);
  new QListWidgetItem ( QIcon(":/icons/db.png"), tr("Database Options"),selector);
  splitter->addWidget(leftarea);
//Right container
  containerwidget=new QStackedWidget(splitter);
  splitter->addWidget(containerwidget);
  generatesavingsbox();
  generatedirectorybox();
  generateripperbox();
  filterchanged();
  dialoglayout->addWidget(splitter);
  this->setLayout(dialoglayout);
  connect (selector,SIGNAL(currentRowChanged ( int )),this,SLOT(changeview(int)));
}

settings::~settings()
{

}

void settings::generatesavingsbox()
{
  savingscroller=new QScrollArea(containerwidget);
  savingarea=new QWidget(containerwidget);
  savinglayout=new QGridLayout(savingarea);
  QStringList choices;
//Tagenginepart####################
  taglabel=new QLabel(tr("ID3-Tag-Version"),containerwidget);
  tagenginecombo=new QComboBox(containerwidget);
  tagenginecombo->addItem(QIcon(":/icons/taglib.png"),tr("Taglib V1 only"));
  tagenginecombo->addItem(QIcon(":/icons/taglib.png"),tr("Taglib V2.4 only"));
  tagenginecombo->addItem(QIcon(":/icons/taglib.png"),tr("Taglib V1 and V2.4"));
  tagenginecombo->addItem(QIcon(":/icons/id3lib.png"),tr("Id3lib V1 only"));
  tagenginecombo->addItem(QIcon(":/icons/id3lib.png"),tr("Id3lib V2.4 only"));
  tagenginecombo->addItem(QIcon(":/icons/id3lib.png"),tr("Id3lib V1 and V2.4"));
  tagenginecombo->setEditable(FALSE);
  savinglayout->addWidget(taglabel,0,0);
  savinglayout->addWidget(tagenginecombo,0,1);
  if (settingsfile->getsav_tagengine()==tagengine::taglib)
    tagenginecombo->setCurrentIndex(settingsfile->getsav_tagtype()-1);
  else
    tagenginecombo->setCurrentIndex(settingsfile->getsav_tagtype()+2);
//Filterpart#########################
  savefilterbox=new QGroupBox(tr("Filenamingfilter"),containerwidget);
  savefilterbox->setAutoFillBackground(TRUE);
  savefilterlayout=new QGridLayout(savefilterbox);
  savefiltercombo=new QComboBox(savefilterbox);
  choices.clear();
  choices<<settingsfile->getsav_filter();
  if (settingsfile->getsav_filter().compare("#GENRE#/#BAND#/#ALBUM#/#TRACK#-#SONG#"))
    choices<<"#GENRE#/#BAND#/#ALBUM#/#TRACK#-#SONG#";
  if (settingsfile->getsav_filter().compare("#GENRE#/#BAND#/[#YEAR#]#ALBUM#/#TRACK#-#SONG#"))
    choices<<"#GENRE#/#BAND#/[#YEAR#]#ALBUM#/#TRACK#-#SONG#";
  if (settingsfile->getsav_filter().compare("#BAND#/#ALBUM#/#TRACK#-#SONG#"))
    choices<<"#BAND#/#ALBUM#/#TRACK#-#SONG#";
  if (settingsfile->getsav_filter().compare("#BAND#/#TRACK#-#SONG#"))
    choices<<"#BAND#/#TRACK#-#SONG#";
  choices<<"";
  savefiltercombo->insertItems(0,choices);
  savefiltercombo->setEditText(settingsfile->getsav_filter());
  savefiltercombo->setEditable(TRUE);
  savefiltercombo->setCompleter(0);
  genrebutton=new QPushButton("#GENRE#",savefilterbox);
  albumbutton=new QPushButton("#ALBUM#",savefilterbox);
  bandbutton=new QPushButton("#BAND#",savefilterbox);
  songbutton=new QPushButton("#SONG#",savefilterbox);
  trackbutton=new QPushButton("#TRACK#",savefilterbox);
  yearbutton=new QPushButton("#YEAR#",savefilterbox);
  directorybutton=new QPushButton("/",savefilterbox);
  clearbutton=new QPushButton(tr("Clear"));
  savefilterlayout->addWidget(savefiltercombo,0,0,1,4);
  savefilterlayout->addWidget(genrebutton,1,0);
  savefilterlayout->addWidget(albumbutton,1,1);
  savefilterlayout->addWidget(bandbutton,1,2);
  savefilterlayout->addWidget(songbutton,1,3);
  savefilterlayout->addWidget(trackbutton,2,0);
  savefilterlayout->addWidget(yearbutton,2,1);
  savefilterlayout->addWidget(directorybutton,2,2);
  savefilterlayout->addWidget(clearbutton,2,3);
  savefilterbox->setLayout(savefilterlayout);
  savinglayout->addWidget(savefilterbox,1,0,1,2);
  QString filter=settingsfile->getsav_filter();
  connect (savefiltercombo,SIGNAL(editTextChanged(const QString &)),this,SLOT(filterchanged()));
  connect (clearbutton,SIGNAL(clicked()),this,SLOT(clearslot()));
  connect (genrebutton,SIGNAL(clicked()),this,SLOT(genreslot()));
  connect (albumbutton,SIGNAL(clicked()),this,SLOT(albumslot()));
  connect (bandbutton,SIGNAL(clicked()),this,SLOT(bandslot()));
  connect (songbutton,SIGNAL(clicked()),this,SLOT(songslot()));
  connect (trackbutton,SIGNAL(clicked()),this,SLOT(trackslot()));
  connect (yearbutton,SIGNAL(clicked()),this,SLOT(yearslot()));
  connect (directorybutton,SIGNAL(clicked()),this,SLOT(directoryslot()));
  //Picturepart########################
  picturebox=new QGroupBox(tr("Save picture in folder"),containerwidget);
  picturebox->setCheckable(TRUE);
  picturelayout=new QGridLayout(picturebox);
  picturelabel=new QLabel(tr("Filename"),picturebox);
  picturename=new QComboBox(picturebox);
  choices.clear();
  choices<<"albumart"<<"cover"<<"folder";
  picturename->insertItems(0,choices);
  picturename->setEditable(TRUE);
  choices.clear();
  choices<<"bmp"<<"jpg"<<"gif"<<"png";
  pictureformat=new QComboBox(picturebox);
  pictureformat->insertItems(0,choices);
  pictureformat->setEditable(FALSE);
  picturesizelabel=new QLabel(tr("Picturesize"),picturebox);
  picturesizespinner=new QSpinBox(picturebox);
  picturesizespinner->setRange(20,300);
  picturelayout->addWidget(picturelabel,0,0);
  picturelayout->addWidget(picturename,0,1);
  picturelayout->addWidget(pictureformat,0,2);
  picturelayout->addWidget(picturesizelabel,1,0);
  picturelayout->addWidget(picturesizespinner,1,2);
  picturebox->setLayout(picturelayout);
  savinglayout->addWidget(picturebox,2,0,1,2);
  pictureformat->setCurrentIndex(settingsfile->getsav_picturetype());
  picturesizespinner->setValue(settingsfile->getsav_picturesize());
  picturename->setEditText(settingsfile->getsav_picturename());
  picturebox->setChecked(settingsfile->getsav_picture());
//Savemodepart##########################
  savemodesbox=new QGroupBox(tr("Saving properties"),containerwidget);
  savemodelayout=new QGridLayout(savemodesbox);
  overwritelabel=new QLabel(tr("Overwritemode"),savemodesbox);
  overwritecombo=new QComboBox(savemodesbox);
  choices.clear();
  choices<<tr("Allways keep old file")
  <<tr("Always keep new file")
  <<tr("Keep file with higher bitrate")
  <<tr("Ask for overwriting");
  overwritecombo->insertItems(0,choices);
  overwritecombo->setEditable(FALSE);
  movecheck=new QCheckBox(tr("Move files"),savemodesbox);
  deletemodecombo=new QComboBox(savemodesbox);
  choices.clear();
  choices<<tr("Delete only file")
  <<tr("Delete dirctory if empty")
  <<tr("Delete directory recursive");
  deletemodecombo->insertItems(0,choices);
  savemodelayout->addWidget(overwritelabel,0,0);
  savemodelayout->addWidget(overwritecombo,0,1);
  savemodelayout->addWidget(movecheck,1,0);
  savemodelayout->addWidget(deletemodecombo,1,1);
  ignorelistbox=new QGroupBox(tr("Treat this files as not existing on delete"),savemodesbox);
  ignorelistlayout=new QGridLayout(ignorelistbox);
  ignoreledit=new QLineEdit(ignorelistbox);
  newignorebutton=new QPushButton(QIcon(":/icons/edit_add.png"),QString::null,ignorelistbox);
  removeignorebutton=new QPushButton(QIcon(":/icons/eraser.png"),QString::null,ignorelistbox);
  ignorelister=new QListWidget(ignorelistbox);
  ignorelister->setSelectionMode(QAbstractItemView::ExtendedSelection);
  ignorelistlayout->addWidget(ignoreledit,0,0,1,3);
  ignorelistlayout->addWidget(newignorebutton,0,3,1,1);
  ignorelistlayout->addWidget(removeignorebutton,0,4,1,1);
  ignorelistlayout->addWidget(ignorelister,1,0,3,5);
  ignorelistbox->setLayout(ignorelistlayout);
  savemodelayout->addWidget(ignorelistbox,2,0,1,2);
  savemodesbox->setLayout(savemodelayout);
  savinglayout->addWidget(savemodesbox,3,0,1,2);
  overwritecombo->setCurrentIndex(settingsfile->getsav_ovmode());
  deletemodecombo->setEnabled(settingsfile->getsav_move());
  deletemodecombo->setCurrentIndex(settingsfile->getsav_deletemode());
  movecheck->setChecked(settingsfile->getsav_move());
  const QStringList ignorelist=settingsfile->getsav_ignorelist();
  QStringList::ConstIterator ignorewp=ignorelist.begin();
  while (ignorewp!=ignorelist.end())
    {
      new QListWidgetItem ((*ignorewp),ignorelister);
      ignorewp++;
    }
  QSpacerItem *savingspacer=new QSpacerItem(0,0,QSizePolicy::Expanding,QSizePolicy::Expanding);
  savinglayout->addItem(savingspacer,3,0);
  savingarea->setLayout(savinglayout);
  savingscroller->setWidget(savingarea);
  savingscroller->setWidgetResizable( TRUE );
  containerwidget->addWidget(savingscroller);
  newignorebutton->setEnabled(FALSE);
  removeignorebutton->setEnabled(FALSE);
  connect (tagenginecombo,SIGNAL(activated(int)),this,SLOT(selecttagversion(int)));
  connect (picturebox,SIGNAL(toggled(bool)),this,SLOT(togglepicture(bool)));
  connect (picturename,SIGNAL(editTextChanged(const QString &)),this,SLOT(changepicturename(const QString &)));
  connect (pictureformat,SIGNAL(activated(int)),this,SLOT(changepictureformat(int)));
  connect (picturesizespinner,SIGNAL(valueChanged(int)),this,SLOT(changepicturesize(int)));
  connect (movecheck,SIGNAL(toggled(bool)),this,SLOT(togglemove(bool)));
  connect (deletemodecombo,SIGNAL(activated(int)),this,SLOT(sav_deleteslot(int)));
  connect (overwritecombo,SIGNAL(activated(int)),this,SLOT(sav_ovslot(int)));
  connect (ignoreledit,SIGNAL(textChanged(const QString &)),this,SLOT(activateaddfile(const QString &)));
  connect (ignorelister,SIGNAL(itemSelectionChanged()),this,SLOT(activateremovefile()));
  connect (newignorebutton,SIGNAL(clicked(bool)),this,SLOT(ignorelistadd()));
  connect (removeignorebutton,SIGNAL(clicked(bool)),this,SLOT(ignorelistremove()));
}

void settings::generatedirectorybox()
{
  directoryscroller=new QScrollArea(containerwidget);
  directoryarea=new QWidget(containerwidget);
  directorylayout=new QGridLayout(directoryarea);
  tempdirledit=new QLineEdit(directoryarea);
  tempdirledit->setText(settingsfile->gettempdir());
  tempdirledit->setReadOnly(true);
  tempdirbutton=new QPushButton(tr("Select tempdir"),directoryarea);
  workingdirledit=new QLineEdit(directoryarea);
  workingdirledit->setText(settingsfile->getworkingdir());
  workingdirledit->setReadOnly(true);
  workingdirbutton=new QPushButton(tr("Select sourcedir"),directoryarea);
  destinationdirledit=new QLineEdit(directoryarea);
  destinationdirledit->setText(settingsfile->getdestinationdir());
  destinationdirledit->setReadOnly(true);
  destinationdirbutton=new QPushButton(tr("Select destinationdir"),directoryarea);
  containerwidget->addWidget(directoryscroller);
  directorylayout->addWidget(tempdirledit,0,0,1,3);
  directorylayout->addWidget(tempdirbutton,0,3,1,2);
  directorylayout->addWidget(workingdirledit,1,0,1,3);
  directorylayout->addWidget(workingdirbutton,1,3,1,2);
  directorylayout->addWidget(destinationdirledit,2,0,1,3);
  directorylayout->addWidget(destinationdirbutton,2,3,1,2);
  searchpathesbox=new QGroupBox(tr("Executable-searchfolders"),directoryarea);
  searchpatheslayout=new QGridLayout(searchpathesbox);
  searchpatheslister=new QListWidget(searchpathesbox);
  searchpathesledit=new QLineEdit(searchpathesbox);
  newsearchpathbutton=new QPushButton(QIcon(":/icons/edit_add.png"),QString::null,searchpathesbox);
  removesearchpathbutton=new QPushButton(QIcon(":/icons/eraser.png"),QString::null,searchpathesbox);
  selectsearchpathbutton=new QPushButton(tr("Select Path"),searchpathesbox) ;
  removesearchpathbutton->setEnabled(FALSE);
  newsearchpathbutton->setEnabled(FALSE);
  const QStringList searchlist=settingsfile->getsearchpathes();
  for (int i=0;i<searchlist.count();i++)
    new QListWidgetItem (searchlist.at(i),searchpatheslister);
  searchpatheslayout->addWidget(searchpathesledit,0,0,1,3);
  searchpatheslayout->addWidget(selectsearchpathbutton,0,3,1,1);
  searchpatheslayout->addWidget(newsearchpathbutton,0,4,1,1);
  searchpatheslayout->addWidget(removesearchpathbutton,0,5,1,1);
  searchpatheslayout->addWidget(searchpatheslister,1,0,1,6);
  searchpathesbox->setLayout(searchpatheslayout);
  directorylayout->addWidget(searchpathesbox,3,0,2,5);
  directoryspacer=new QSpacerItem(0,0,QSizePolicy::Expanding,QSizePolicy::Expanding);
  directorylayout->addItem(directoryspacer,4,0);
  directoryarea->setLayout(directorylayout);
  directoryscroller->setWidget(directoryarea);
  directoryscroller->setWidgetResizable( TRUE );
  containerwidget->addWidget(directoryscroller);
  connect(tempdirbutton,SIGNAL(clicked()),this,SLOT(settempdirslot()));
  connect(workingdirbutton,SIGNAL(clicked()),this,SLOT(setworkingdirslot()));
  connect(destinationdirbutton,SIGNAL(clicked()),this,SLOT(setdestinationdirslot()));
  connect(searchpathesledit,SIGNAL(textChanged(const QString &)),this,SLOT(activatesearchpathaddslot(const QString &)));
  connect (searchpatheslister,SIGNAL(itemSelectionChanged()),this,SLOT(activatesearchpathremoveslot()));
  connect(selectsearchpathbutton,SIGNAL(clicked()),this,SLOT(setnewsearchpathslot()));
  connect (newsearchpathbutton,SIGNAL(clicked(bool)),this,SLOT(searchpathlistaddslot()));
  connect (removesearchpathbutton,SIGNAL(clicked(bool)),this,SLOT(searchpathlistremoveslot()));
}

void settings::generateripperbox()
{
  ripperscroller=new QScrollArea(containerwidget);
  ripperarea=new QWidget(containerwidget);
  ripperlayout=new QGridLayout(directoryarea);
  bitratelabel=new QLabel(tr("Bitrate"),containerwidget);
  bitratecombo=new QComboBox(containerwidget);
  QStringList choices;
  choices<<"320 kb/s"<<"256 kb/s"<<"192 kb/s"<<"128 kb/s";
  bitratecombo->insertItems(0,choices);
  bitratecombo->setEditable(FALSE);
  if (settingsfile->getrip_bitrate()==128)
    bitratecombo->setCurrentIndex(3);
  if (settingsfile->getrip_bitrate()==192)
    bitratecombo->setCurrentIndex(2);
  if (settingsfile->getrip_bitrate()==256)
    bitratecombo->setCurrentIndex(1);
  if (settingsfile->getrip_bitrate()==320)
    bitratecombo->setCurrentIndex(0);
  stereomodelabel=new QLabel(tr("Stereomode"),containerwidget);
  stereomodecombo=new QComboBox(containerwidget);
  choices.clear();
  choices<<"Stereo"<<"Joint Stereo"<<"Mono";
  stereomodecombo->insertItems(0,choices);
  stereomodecombo->setEditable(FALSE);
  if (settingsfile->getrip_stereotype()==0)
    stereomodecombo->setCurrentIndex(0);
  if (settingsfile->getrip_stereotype()==1)
    stereomodecombo->setCurrentIndex(1);
  if (settingsfile->getrip_stereotype()==3)
    stereomodecombo->setCurrentIndex(2);
  encodertypelabel=new QLabel(tr("Encoder"),containerwidget);
  encodertypecombo=new QComboBox(containerwidget);
  choices.clear();
  choices<<"Lame"<<"Gogo"<<"OggEnc"<<"BladeEnc";
  encodertypecombo->insertItems(0,choices);
  encodertypecombo->setEditable(FALSE);
  encodeoptionslabel=new QLabel(tr("Encoderoptions"),containerwidget);
  encodeoptionsledit=new QLineEdit(containerwidget);
  if (settingsfile->getrip_encoder()==encoder::lame)
    {
      encodertypecombo->setCurrentIndex(0);
      encodeoptionsledit->setText(settingsfile->getrip_lame_options());
    }
  else   if (settingsfile->getrip_encoder()==encoder::gogo)
    {
      encodertypecombo->setCurrentIndex(1);
      encodeoptionsledit->setText(settingsfile->getrip_gogo_options());
    }
  else   if (settingsfile->getrip_encoder()==encoder::oggenc)
    {
      encodertypecombo->setCurrentIndex(2);
      encodeoptionsledit->setText(settingsfile->getrip_oggenc_options());
    }
  else   if (settingsfile->getrip_encoder()==encoder::bladeenc)
    {
      encodertypecombo->setCurrentIndex(3);
      encodeoptionsledit->setText(settingsfile->getrip_blade_options());
    }
  numencoderslabel=new QLabel(tr("Simultan running encoders"),containerwidget);
  numencodersspinbox=new QSpinBox(containerwidget);
  numencodersspinbox->setRange(1,5);
  numencodersspinbox->setValue(settingsfile->getrip_number_encoders());
  ripperlayout->addWidget(bitratelabel,0,0);
  ripperlayout->addWidget(bitratecombo,0,1);
  ripperlayout->addWidget(stereomodelabel,1,0);
  ripperlayout->addWidget(stereomodecombo,1,1);
  ripperlayout->addWidget(encodertypelabel,2,0);
  ripperlayout->addWidget(encodertypecombo,2,1);
  ripperlayout->addWidget(encodeoptionslabel,3,0);
  ripperlayout->addWidget(encodeoptionsledit,3,1);
  ripperlayout->addWidget(numencoderslabel,4,0);
  ripperlayout->addWidget(numencodersspinbox,4,1);
  ripperspacer=new QSpacerItem(0,0,QSizePolicy::Expanding,QSizePolicy::Expanding);
  ripperlayout->addItem(ripperspacer,5,0);
  ripperarea->setLayout(ripperlayout);
  ripperscroller->setWidget(ripperarea);
  ripperscroller->setWidgetResizable( TRUE );
  containerwidget->addWidget(ripperscroller);
  connect(bitratecombo,SIGNAL(currentIndexChanged(int)),this,SLOT(bitratechangedslot(int)));
  connect(stereomodecombo,SIGNAL(currentIndexChanged(int)),this,SLOT(stereomodecombochangedslot(int)));
  connect(encodertypecombo,SIGNAL(currentIndexChanged(int)),this,SLOT(encoderchangedslot(int)));
  connect(encodeoptionsledit,SIGNAL(textChanged(const QString &)),this,SLOT(encodeoptionschangedslot(const QString &)));
  connect(numencodersspinbox,SIGNAL(valueChanged(int)),this,SLOT(numencoderschangedslot(int)));
}

//Savingpart-slots################################################################################################
void settings::selecttagversion(int value)
{
  if (value>2)
    {
      QMessageBox::information(this,"K-YAMO-Information","ID3Lib-Interface not yet implemented");
      tagenginecombo->setCurrentIndex(settingsfile->getsav_tagtype()-1);
      /*
      settingsfile->setsav_tagengine(tagengine::id3lib);
      settingsfile->setsav_tagtype(value-2);
      */
    }
  else
    {
      settingsfile->setsav_tagengine(tagengine::taglib);
      settingsfile->setsav_tagtype(value+1);
    }
}

void settings::filterchanged()
{
  bool notallowed=FALSE;
  QString text=savefiltercombo->currentText();
  QStringList checkmnemnomics;
  checkmnemnomics<<"#GENRE#"<<"#BAND#"<<"#ALBUM#"<<"#YEAR#"<<"#TRACK#"<<"#TITLE#"<<"#SONG#";
  QStringList::Iterator checkwp=checkmnemnomics.begin();
  while (checkwp!=checkmnemnomics.end())
    {
      if (text.count(*checkwp)>1)
        {
          QMessageBox::warning(this,tr("K-Yamo-Warning"),QString(tr("%1 already used. Removing last entry!")).arg(*checkwp));
          //Check for the Cursorposition, and remove the entry next to cursor
          int position=savefiltercombo->lineEdit()->cursorPosition();
          int position2=text.indexOf(*checkwp);
          if (position>=position2&&position<=(position2+(*checkwp).length()))
            text=text.remove(position2,(*checkwp).length());
          else
            text=text.remove(text.lastIndexOf(*checkwp),(*checkwp).length());
          while (text.startsWith("/"))
            text.remove(0,1);
          notallowed=TRUE;
        }
      checkwp++;
    }
  if (text.contains("//"))
    {
      text=text.replace("//","/");
      QMessageBox::warning(this,tr("K-Yamo-Warning"),tr("You can't create directories without a name!"));
      notallowed=TRUE;
    }
  QStringList removesigns;
  removesigns<<"?"<<":"<<"|"<<"!"<<"*"<<".";
  QStringList::Iterator removewp=removesigns.begin();
  while (removewp!=removesigns.end())
    {
      if (text.contains(*removewp))
        {
          QMessageBox::warning(this,tr("K-Yamo-Warning"),QString(tr("Removin problematic %1-sign!")).arg(*removewp));
          text=text.remove(*removewp);
          notallowed=TRUE;
        }
      removewp++;
    }
  if (notallowed)
    {
      savefiltercombo->setEditText(text);
    }
  if (fileoperationhandle->setfilter(text))
    {
      QPalette palette;
      palette.setColor(QPalette::Base, QColor(160,255,160));
      savefiltercombo->setPalette(palette);
    }
  else
    {
      QPalette palette;
      palette.setColor(QPalette::Base, QColor(255,160,160));
      savefiltercombo->setPalette(palette);
    }
  settingsfile->setsav_filter(text);
}

void settings::genreslot()
{
  QString text=savefiltercombo->currentText();
  if (!text.contains("#GENRE#"))
    {
      text+=QString("#GENRE#");
      savefiltercombo->setEditText(text);
    }
  else
    QMessageBox::warning(this,tr("K-Yamo-Warning"),tr("You can't use the same atribute twice!"));
}

void settings::albumslot()
{
  QString text=savefiltercombo->currentText();
  if (!text.contains("#ALBUM#"))
    {
      text+=QString("#ALBUM#");
      savefiltercombo->setEditText(text);
    }
  else
    QMessageBox::warning(this,tr("K-Yamo-Warning"),tr("You can't use the same atribute twice!"));
}

void settings::bandslot()
{
  QString text=savefiltercombo->currentText();
  if (!text.contains("#BAND#"))
    {
      text+=QString("#BAND#");
      savefiltercombo->setEditText(text);
    }
  else
    QMessageBox::warning(this,tr("K-Yamo-Warning"),tr("You can't use the same atribute twice!"));
}

void settings::songslot()
{
  QString text=savefiltercombo->currentText();
  if (!text.contains("#SONG#"))
    {
      text+=QString("#SONG#");
      savefiltercombo->setEditText(text);
    }
  else
    QMessageBox::warning(this,tr("K-Yamo-Warning"),tr("You can't use the same atribute twice!"));
}

void settings::trackslot()
{
  QString text=savefiltercombo->currentText();
  if (!text.contains("#TRACK#"))
    {
      text+=QString("#TRACK#");
      savefiltercombo->setEditText(text);
    }
  else
    QMessageBox::warning(this,tr("K-Yamo-Warning"),tr("You can't use the same atribute twice!"));
}

void settings::yearslot()
{
  QString text=savefiltercombo->currentText();
  if (!text.contains("#YEAR#"))
    {
      text+=QString("#YEAR#");
      savefiltercombo->setEditText(text);
    }
  else
    QMessageBox::warning(this,tr("K-Yamo-Warning"),tr("You can't use the same atribute twice!"));
}

void settings::directoryslot()
{
  QString text=savefiltercombo->currentText();
  if (!text.endsWith("/")&&text.length())
    {
      text+=QString("/");
      savefiltercombo->setEditText(text);
    }
  else
    QMessageBox::warning(this,tr("K-Yamo-Warning"),tr("You can't create directories without a name!"));
}

void settings::clearslot()
{
  savefiltercombo->setEditText(QString::null);
}

void settings::togglepicture(bool value)
{
  settingsfile->setsav_picture(value);
}

void settings::changepicturename(const QString &text)
{
  settingsfile->setsav_picturename(text);
}

void settings::changepictureformat(int value)
{
  settingsfile->setsav_picturetype(value);
}

void settings::changepicturesize(int value)
{
  settingsfile->setsav_picturesize(value);
}

void settings::togglemove(bool value)
{
  settingsfile->setsav_move(value);
  deletemodecombo->setEnabled(value);
}

void settings::sav_ovslot(int value)
{
  settingsfile->setsav_ovmode(value);
  fileoperationhandle->setovmode(value);
}

void settings::sav_deleteslot(int value)
{
  settingsfile->setsav_deletemode(value);
  fileoperationhandle->setdeletemode(value);
}

void settings::activateaddfile(const QString &text)
{
  if (text.length())
    {
      bool found=FALSE;
      for (int i=0;i<ignorelister->count();i++)
        {
          QString actualstring=QString(ignorelister->item(i)->text());
          if (!text.compare(actualstring))
            {
              found=TRUE;
              break;
            }
        }
      if (!found)
        newignorebutton->setEnabled(TRUE);
      else
        newignorebutton->setEnabled(FALSE);
    }
  else
    newignorebutton->setEnabled(FALSE);
}

void settings::activateremovefile()
{
  QList<QListWidgetItem *> selectionlist=ignorelister->selectedItems();
  if (selectionlist.count())
    removeignorebutton->setEnabled(TRUE);
  else
    removeignorebutton->setEnabled(FALSE);
}

void settings::ignorelistadd()
{
  QString newentry=ignoreledit->displayText();
  new QListWidgetItem (newentry,ignorelister);
  QStringList files;
  for (int i=0;i<ignorelister->count();i++)
    {
      QString actualstring=QString(ignorelister->item(i)->text());
      files<<actualstring;
    }
  settingsfile->setsav_ignorelist(files);
  fileoperationhandle->setignorelist(files);
  ignoreledit->setText(QString::null);
}

void settings::ignorelistremove()
{
  QList<QListWidgetItem *> selectionlist=ignorelister->selectedItems();
  QList<QListWidgetItem *>::Iterator selectionwp=selectionlist.begin();
  while (selectionwp!=selectionlist.end())
    {
      delete (*selectionwp);
      selectionwp++;
    }
  QStringList files;
  for (int i=0;i<ignorelister->count();i++)
    {
      QString actualstring=QString(ignorelister->item(i)->text());
      files<<actualstring;
    }
  settingsfile->setsav_ignorelist(files);
  fileoperationhandle->setignorelist(files);
}


//Directory-Part-Slots############################################################################################

void settings::settempdirslot()
{
  QString newpath = QFileDialog::getExistingDirectory(this, tr("Open Directory"),settingsfile->gettempdir(),QFileDialog::ShowDirsOnly| QFileDialog::DontResolveSymlinks);
  if (!newpath.isNull())
    {
      settingsfile->settempdir(newpath);
      tempdirledit->setText(newpath);
    }
}

void settings::setworkingdirslot()
{
  QString newpath = QFileDialog::getExistingDirectory(this, tr("Open Directory"),settingsfile->getworkingdir(),QFileDialog::ShowDirsOnly| QFileDialog::DontResolveSymlinks);
  if (!newpath.isNull())
    {
      settingsfile->setworkingdir(newpath);
      workingdirledit->setText(newpath);
    }
}

void settings::setdestinationdirslot()
{
  QString newpath = QFileDialog::getExistingDirectory(this, tr("Open Directory"),settingsfile->getdestinationdir(),QFileDialog::ShowDirsOnly| QFileDialog::DontResolveSymlinks);
  if (!newpath.isNull())
    {
      settingsfile->setdestinationdir(newpath);
      destinationdirledit->setText(newpath);
    }
}

void settings::activatesearchpathaddslot(const QString &path)
{
  QFileInfo checkdir(path);
  if (checkdir.exists()&&checkdir.isDir())
    newsearchpathbutton->setEnabled(true);
  else
    newsearchpathbutton->setEnabled(false);
}

void settings::activatesearchpathremoveslot()
{
  QList<QListWidgetItem *> selectionlist=searchpatheslister->selectedItems();
  if (selectionlist.count())
    removesearchpathbutton->setEnabled(TRUE);
  else
    removesearchpathbutton->setEnabled(FALSE);
}

void settings::setnewsearchpathslot()
{
  QString newpath = QFileDialog::getExistingDirectory(this, tr("Open Directory"),settingsfile->getdestinationdir(),QFileDialog::ShowDirsOnly| QFileDialog::DontResolveSymlinks);
  if (!newpath.isNull())
    searchpathesledit->setText(newpath);
}

void settings::searchpathlistaddslot()
{
  bool found=false;
  QString newentry=searchpathesledit->displayText();
  for (int i=0;i<searchpatheslister->count();i++)
    {
      if (!searchpatheslister->item(i)->text().compare(newentry))
        {
          found=true;
          break;
        }
    }
  if (!found)
    {
      new QListWidgetItem (newentry,searchpatheslister);
      QStringList dirs;
      for (int i=0;i<searchpatheslister->count();i++)
        {
          QString actualstring=QString(searchpatheslister->item(i)->text());
          dirs<<actualstring;
        }
      settingsfile->setsearchpathes(dirs);
    }
  searchpathesledit->setText(QString::null);
}

void settings::searchpathlistremoveslot()
{
  QList<QListWidgetItem *> selectionlist=searchpatheslister->selectedItems();
  QList<QListWidgetItem *>::Iterator selectionwp=selectionlist.begin();
  while (selectionwp!=selectionlist.end())
    {
      delete (*selectionwp);
      selectionwp++;
    }
  QStringList dirs;
  for (int i=0;i<searchpatheslister->count();i++)
    {
      QString actualstring=QString(searchpatheslister->item(i)->text());
      dirs<<actualstring;
    }
  settingsfile->setsearchpathes(dirs);
}

//Ripper-Part-Slos################################################################################################
void settings::bitratechangedslot(int i)
{
  if (i==3)
    settingsfile->setrip_bitrate(128);
  else if (i==2)
    settingsfile->setrip_bitrate(192);
  else if (i==1)
    settingsfile->setrip_bitrate(256);
  else if (i==0)
    settingsfile->setrip_bitrate(320);
}

void settings::stereomodecombochangedslot(int i)
{
  if (i==0)
    settingsfile->setrip_stereotype(audiomode::Stereo);
  else if (i==1)
    settingsfile->setrip_stereotype(audiomode::JointStereo);
  else if (i==2)
    settingsfile->setrip_stereotype(audiomode::SingleChannel);
}

void settings::encoderchangedslot(int i)
{
  if (i==0)
    {
      settingsfile->setrip_encoder(encoder::lame);
      encodeoptionsledit->setText(settingsfile->getrip_lame_options());
    }
  else if (i==1)
    {
      settingsfile->setrip_encoder(encoder::gogo);
      encodeoptionsledit->setText(settingsfile->getrip_gogo_options());
    }
  else if (i==2)
    {
      settingsfile->setrip_encoder(encoder::oggenc);
      encodeoptionsledit->setText(settingsfile->getrip_oggenc_options());
    }
  else if (i==3)
    {
      settingsfile->setrip_encoder(encoder::bladeenc);
      encodeoptionsledit->setText(settingsfile->getrip_blade_options());
    }
if (i>0)
{
QMessageBox::information(this,tr("K-Yamo information"),tr("Suport for other encoders than lame is not yet implemented."));
	encodertypecombo->setCurrentIndex(0);
}
}

void settings::encodeoptionschangedslot(const QString &text)
{
  int actualencoder=settingsfile->getrip_encoder();
  if (actualencoder==encoder::lame)
    settingsfile->setrip_lame_options(text);
  else if (actualencoder==encoder::gogo)
    settingsfile->setrip_gogo_options(text);
  else if (actualencoder==encoder::oggenc)
    settingsfile->setrip_oggenc_options(text);
  else if (actualencoder==encoder::bladeenc)
    settingsfile->setrip_blade_options(text);
}

void settings::numencoderschangedslot(int value)
{
  settingsfile->setrip_number_encoders(value);
}

//Tabchange#######################################################################################################
void settings::changeview(int value)
{
  if (value<=2)
    containerwidget->setCurrentIndex(value);
  else
    QMessageBox::information(this,tr("K-Yamo-Information"),tr("This is not yet written!Option locked!"));
}

