/***************************************************************************
 *   Copyright (C) 2007 by Roland Weigert   *
 *   roweigert@t-online.de   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

/**
 *@file settingsdialog.h
 *
 *This file holds the settingsdialog-class.
 *It contains only the gui to the settings.
 *The settings themself are in rcfile.h.
 *
 *@author Roland Weigert
 *@date 29-01-2009
 *@todo Include picturesupport into fileoperator
 */

#ifndef _SETTINGSDIALOG_H_
#define _SETTINGSDIALOG_H_

#include <QScrollArea>
#include <QListWidget>
#include <QListWidgetItem>
#include <QWidget>
#include <QLayout>
#include <QSize>
#include <QSplitter>
#include <QStackedWidget>
#include <QLabel>
#include <QComboBox>
#include <QSpinBox>
#include <QGroupBox>
#include <QPushButton>
#include <QCheckBox>
#include <QLineEdit>
#include <QString>
#include <QStringList>
#include <QPalette>
#include <QFileDialog>
#include <QMessageBox>

#include "global-stuff/view.h"
#include "settings/rcfile.h"
#include "global-stuff/globals.h"
#include "global-stuff/fileoperator.h"


class settings : public  view
  {
    Q_OBJECT
  public:
    settings(QWidget *parent);
    ~settings();
  private:
    QVBoxLayout *dialoglayout;
    QSplitter *splitter;
    QScrollArea *leftarea;
    QListWidget *selector;
    QStackedWidget *containerwidget;
//#########Savingpart##################
    QScrollArea *savingscroller;
    QWidget *savingarea;
    QGridLayout *savinglayout;
    QLabel *taglabel;
    QComboBox *tagenginecombo;
    QGroupBox *picturebox;
    QGridLayout *picturelayout;
    QLabel *picturelabel;
    QComboBox *picturename;
    QComboBox *pictureformat;
    QLabel *picturesizelabel;
    QSpinBox *picturesizespinner;
    QGroupBox *savefilterbox;
    QGridLayout *savefilterlayout;
    QComboBox *savefiltercombo;
    QPushButton *genrebutton;
    QPushButton *albumbutton;
    QPushButton *bandbutton;
    QPushButton *songbutton;
    QPushButton *trackbutton;
    QPushButton *yearbutton;
    QPushButton *directorybutton;
    QPushButton *clearbutton;
    QGroupBox *savemodesbox;
    QGridLayout *savemodelayout;
    QLabel *overwritelabel;
    QComboBox *overwritecombo;
    QCheckBox *movecheck;
    QComboBox *deletemodecombo;
    QGroupBox *ignorelistbox;
    QGridLayout *ignorelistlayout;
    QListWidget *ignorelister;
    QLineEdit *ignoreledit;
    QPushButton *newignorebutton;
    QPushButton *removeignorebutton;
    void generatesavingsbox();
//######################################
//#########Directoriespart##############
    QScrollArea *directoryscroller;
    QWidget *directoryarea;
    QGridLayout *directorylayout;
    QLineEdit *tempdirledit;
    QPushButton *tempdirbutton;
    QLineEdit *workingdirledit;
    QPushButton *workingdirbutton;
    QLineEdit *destinationdirledit;
    QPushButton *destinationdirbutton;
    QGroupBox *searchpathesbox;
    QGridLayout *searchpatheslayout;
    QListWidget *searchpatheslister;
    QLineEdit *searchpathesledit;
    QPushButton *selectsearchpathbutton;
    QPushButton *newsearchpathbutton;
    QPushButton *removesearchpathbutton;
    QSpacerItem *directoryspacer;
    void generatedirectorybox();
//######################################
//############Ripperpart################
    QScrollArea *ripperscroller;
    QWidget *ripperarea;
    QGridLayout *ripperlayout;
    QLabel *bitratelabel;
    QComboBox *bitratecombo;
    QLabel *stereomodelabel;
    QComboBox *stereomodecombo;
    QLabel *encodertypelabel;
    QComboBox *encodertypecombo;
    QLabel *encodeoptionslabel;
    QLineEdit *encodeoptionsledit;
    QLabel *numencoderslabel;
    QSpinBox *numencodersspinbox;
    QSpacerItem *ripperspacer;
    void generateripperbox();
//######################################
  private slots:
//##########Savingpart##################
    void selecttagversion(int);
    void filterchanged();
    void togglepicture(bool);
    void changepicturename(const QString &);
    void changepictureformat(int);
    void changepicturesize(int);
    void togglemove(bool);
    void ignorelistadd();
    void ignorelistremove();
    void activateaddfile(const QString &);
    void activateremovefile();
    void genreslot();
    void albumslot();
    void bandslot();
    void songslot();
    void trackslot();
    void yearslot();
    void directoryslot();
    void clearslot();
    void sav_ovslot(int);
    void sav_deleteslot(int);
//######################################
//#########Directoriespart##############
    void settempdirslot();
    void setworkingdirslot();
    void setdestinationdirslot();
    void activatesearchpathaddslot(const QString &);
    void activatesearchpathremoveslot();
    void setnewsearchpathslot();
    void searchpathlistaddslot();
    void searchpathlistremoveslot();
//######################################
//########Ripperpart####################
    void bitratechangedslot(int);
    void stereomodecombochangedslot(int);
    void encoderchangedslot(int);
    void encodeoptionschangedslot(const QString &);
    void numencoderschangedslot(int);
//######################################
    void changeview(int);

  };

#endif
