/***************************************************************************
 *   Copyright (C) 2007 by Roland Weigert   *
 *   roweigert@t-online.de   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include "filetree.h"

extern advancedcommunicationinterface *adv_comm_if;
extern fileoperator *fileoperationhandle;

filetree::filetree(QWidget *parent,const QString directory):QTreeWidget(parent),
    rootitem(NULL),
    currentdir(directory),
    watcher(new QFileSystemWatcher(this)),
    refreshcount(0),
    contextmenu(new QMenu(this)),
    editoropen(false)
{
  this->setColumnCount(2);
  setColumnWidth(0,220);
  setColumnWidth(1,32);
  QStringList headerlist;
  headerlist<<tr("file")<<tr("modified");
  this->setHeaderLabels(headerlist);
  this->setSelectionMode(QAbstractItemView::ExtendedSelection);
  updatedirectory();
  this->setAcceptDrops(TRUE);
  connect(watcher,SIGNAL(directoryChanged( const QString& )),this,SLOT(filesystemchange(const QString &)));
  connect(this,SIGNAL(itemSelectionChanged ()),this,SLOT(recurseselection()));
}

filetree::~filetree()
{
}

const QString filetree::getcurrentdir() const
  {
    return currentdir;
  }

void filetree::updatedirectory(const QString newdirectory)
{
  if (rootitem)
    delete rootitem;
  if (!newdirectory.isNull())
    currentdir=newdirectory;
  QStringList watchlist=watcher->directories();
  if (watchlist.count())
    watcher->removePaths(watchlist);
  rootitem=new filetreeitem(this);
  rootitem->setText(0,currentdir.toLocal8Bit().constData());
  rootitem->setdir(true);
  rootitem->setfilename(currentdir);
  this->expandItem(rootitem);
  refreshcount=0;
  getsubdirectory(currentdir,rootitem);
}

void filetree::getsubdirectory(QString subdir,filetreeitem *parent)
{
//To let the user see, that we are working, let the gui refresh
  //Recursion is a pretty thing!!
//This part can also call itself to go further down the dirtree
  watcher->addPath(subdir);
  QDir readindir(subdir);
  QStringList dirresult=readindir.entryList(QDir::Dirs|QDir::NoDotAndDotDot);
  for (int i=0;i< dirresult.count();i++)
    {
      if (++refreshcount%50==0)
        qApp->processEvents();
      filetreeitem *newitem=new filetreeitem(parent);
      newitem->setText(0,dirresult.at(i).toLocal8Bit().constData());
      newitem->setdir(true);
      if (subdir.endsWith("/"))
        {
          newitem->setfilename(QString("%1%2").arg(subdir,dirresult.at(i)));
          getsubdirectory(QString("%1%2").arg(subdir,dirresult.at(i)),newitem);
        }
      else
        {
          newitem->setfilename(QString("%1/%2").arg(subdir,dirresult.at(i)));
          getsubdirectory(QString("%1/%2").arg(subdir,dirresult.at(i)),newitem);
        }
      parent->addChild(newitem);
    }
  QStringList extensionfilters;
  extensionfilters<<"*.mp3"<<"*.ogg";//<<"*.flac"<<"*.ape";
  readindir.setNameFilters(extensionfilters);
  dirresult=readindir.entryList(QDir::Files|QDir::NoDotAndDotDot);
  for (int i=0;i< dirresult.count();i++)
    {
      refreshcount++;
      if (refreshcount==50)
        {
          qApp->processEvents();
          refreshcount=0;
        }
      filetreeitem *newitem=new filetreeitem(parent);
      newitem->setText(0,dirresult.at(i).toLocal8Bit().constData());
      if (subdir.endsWith("/"))
        {
          newitem->setfilename(QString("%1%2").arg(subdir,dirresult.at(i)));
        }
      else
        newitem->setfilename(QString("%1/%2").arg(subdir,dirresult.at(i)));
      parent->addChild(newitem);
    }
}

void filetree::recurseselection()
{
  if (editoropen)
    {
      QTreeWidgetItem *contextitem=itemAt(storedpoint);
      closePersistentEditor(contextitem);
      //Get the text out of the treeitem, and manipulate the filename in it, than change it in the fileopenlist of tagger also
      QString newname;
      if (contextitem->parent())
        newname=static_cast<filetreeitem *>(contextitem->parent())->getfilename()+"/"+contextitem->text(0);
      else
        newname=currentdir+"/"+contextitem->text(0);
      fileoperationhandle->movefile(static_cast<filetreeitem *>(contextitem)->getfilename(),newname);
      editoropen=false;
    }
  //Rearrange the selection, so everything in directories gets selected
  disconnect(this,SIGNAL(itemSelectionChanged ()),this,SLOT(recurseselection()));
  QList<QTreeWidgetItem *> selection=this->selectedItems () ;
  for (int i=0;i<selection.count();i++)
    {
      filetreeitem *actualitem=static_cast<filetreeitem *>(selection.at(i));
      if (actualitem->isdir()&&actualitem->childCount())
        {
          dirselect(actualitem);
          actualitem->setSelected(true);
        }
    }
  emit fileselectionchanged();
  connect(this,SIGNAL(itemSelectionChanged ()),this,SLOT(recurseselection()));
}

void filetree::dirselect(filetreeitem *diritem)
{
  for (int i=0;i<diritem->childCount();i++)
    {
      if (static_cast<filetreeitem *>(diritem->child(i))->isdir()&&diritem->childCount())
        dirselect(static_cast<filetreeitem *>(diritem->child(i)));
      diritem->child(i)->setSelected(true);
    }
}

void filetree::filesystemchange(const QString &text)
{
  //Check if the rootitem is the right item
  if (!static_cast<filetreeitem *>(rootitem)->getfilename().compare(text))
    {
      rescandirectory(static_cast<filetreeitem *>(rootitem));
    }
  else
    {
      //Iterate through to find the right directory;
      bool found=false;
      bool out=false;
      bool iterate=false;
      filetreeitem *item=rootitem;
      while (!found&&!out)
        {
          iterate=false;
          for (int i=0;i<item->childCount();i++)
            {
              if (text.startsWith(static_cast<filetreeitem *>(item->child(i))->getfilename()))
                {
                  //if the name fully matches, we have found the right directory finally
                  if (!text.compare(static_cast<filetreeitem *>(item->child(i))->getfilename()))
                    {
                      item=static_cast<filetreeitem *>(item->child(i));
                      found=true;
                      break;
                    }
                  else
                    {
                      item=static_cast<filetreeitem *>(item->child(i));
                      iterate=true;
                    }
                }
            }
          //If we got here, without finding the right dir, we ran all through, and the dir was already deleteted
          if (!found&&!iterate)
            out=true;
        }
      if (found)
        {
          rescandirectory(item);
        }
    }
  sortItems(0,Qt::AscendingOrder);
}


void filetree::rescandirectory(filetreeitem *item)
{
  disconnect(this,SIGNAL(itemSelectionChanged ()),this,SLOT(recurseselection()));
  bool selectionchange=false;
  //First check if all children, of our filetreeitem still exist
  for (int i=0;i<item->childCount();i++)
    {
      QFileInfo checkfile(static_cast<filetreeitem *>(item->child(i))->getfilename());
      if (!checkfile.exists())
        {
          //First deselect the item, if it is selected
          if (item->child(i)->isSelected())
            {
              selectionchange=true;
              item->child(i)->setSelected(false);
            }
          removeitem(static_cast<filetreeitem *>(item->child(i)));
          qApp->processEvents();
        }
    }
//Now check if there is some new stuff in the directory. Use recursion if there is a new directory
  QDir readindir(item->getfilename());
  QStringList dirresult=readindir.entryList(QDir::Dirs|QDir::NoDotAndDotDot);
  for (int i=0;i< dirresult.count();i++)
    {
      bool found=false;
      //try to find the directory in the tree
      for (int j=0;j<item->childCount();j++)
        {
          if (!static_cast<filetreeitem *>(item->child(j))->getfilename().compare(QString("%1/%2").arg(item->getfilename(),dirresult.at(i))))
            {
              found=true;
              break;
            }
        }
      if (!found)
        {
          //We have discovered a new item, snap it in the tree with getsubdirectory
          filetreeitem *newitem=new filetreeitem(item);
          newitem->setText(0,dirresult.at(i).toLocal8Bit().constData());
          newitem->setdir(true);
          if (item->getfilename().endsWith("/"))
            {
              newitem->setfilename(QString("%1%2").arg(item->getfilename(),dirresult.at(i)));
              getsubdirectory(QString("%1%2").arg(item->getfilename(),dirresult.at(i)),newitem);
            }
          else
            {
              newitem->setfilename(QString("%1/%2").arg(item->getfilename(),dirresult.at(i)));
              getsubdirectory(QString("%1/%2").arg(item->getfilename(),dirresult.at(i)),newitem);
            }
          item->addChild(newitem);
          qApp->processEvents();
        }
    }
  QStringList extensionfilters;
  extensionfilters<<"*.mp3"<<"*.ogg";//<<"*.flac"<<"*.ape";
  readindir.setNameFilters(extensionfilters);
  dirresult=readindir.entryList(QDir::Files|QDir::NoDotAndDotDot);
  for (int i=0;i< dirresult.count();i++)
    {
      bool found=false;
      //try to find the file in the tree
      for (int j=0;j<item->childCount();j++)
        {
          if (!static_cast<filetreeitem *>(item->child(j))->getfilename().compare(QString("%1/%2").arg(item->getfilename(),dirresult.at(i))))
            {
              found=true;
              break;
            }
        }
      if (!found)
        {
          //We have discovered a new item, snap it in the tree
          filetreeitem *newitem=new filetreeitem(item);
          newitem->setText(0,dirresult.at(i).toLocal8Bit().constData());
          if (item->getfilename().endsWith("/"))
            {
              newitem->setfilename(QString("%1%2").arg(item->getfilename(),dirresult.at(i)));
            }
          else
            {
              newitem->setfilename(QString("%1/%2").arg(item->getfilename(),dirresult.at(i)));
            }
          item->addChild(newitem);
          qApp->processEvents();
        }
    }
  if (selectionchange)
    emit fileselectionchanged();
  connect(this,SIGNAL(itemSelectionChanged ()),this,SLOT(recurseselection()));
}

void filetree::removeitem(filetreeitem *item)
{
//Remove item rekursively from tree
  if (!item->childCount())
    {
      QStringList receivelist;
      receivelist<<"tagger";
      messageobject *message=new messageobject(QString("filetree"), receivelist, QString("remove"), QVariant (item->getfilename()));
      adv_comm_if->sendmessage(message);
    }
  else
    {
      for (int i=0;i<item->childCount();i++)
        {
          if (static_cast<filetreeitem *>(item->child(i))->isdir())
            {
              watcher->removePath(item->getfilename());
              removeitem(static_cast<filetreeitem *>(item->child(i)));
            }
          else
            {
              QStringList receivelist;
              receivelist<<"tagger";
              messageobject *message=new messageobject(QString("filetree"), receivelist, QString("remove"), QVariant (static_cast<filetreeitem *>(item->child(i))->getfilename()));
              adv_comm_if->sendmessage(message);
              item->removeChild(item->child(i));
            }
        }
    }
  item->parent()->removeChild(item);
}

void filetree::dragEnterEvent(QDragEnterEvent *event)
{
  if (event->mimeData()->hasUrls())
    event->acceptProposedAction();
}

void filetree::dragMoveEvent(QDragMoveEvent *event)
{
//This function is only needed, to get the drag and drop working in the QTreeWidget
}


void filetree::dropEvent(QDropEvent *event)
{
  QString newlink;
  if (event->mimeData()->hasFormat("text/plain"));
  {
    QString filename=event->mimeData()->text();
    filename=filename.remove("file://");
    QFileInfo checkfile(filename);
    if (checkfile.isDir())
      newlink=filename;
    else if (filename.toLower().endsWith("mp3")||filename.toLower().endsWith("ogg")||filename.toLower().endsWith("ape")||filename.toLower().endsWith("flac"))
      newlink=filename.left(filename.lastIndexOf("/"));
    if (!newlink.isNull())
      {
        //Safetyquestion first, if there are changed tags
        bool found=false;
        //Simply check the root-item, if there is something, that is changed, but not saved.
        if (rootitem->ismodified()==1)
          found=true;
        if (found)
          {
            if (QMessageBox::question(this,"K-YAMO-Information",tr("You have changed tags in one or more files.<br>Do you want to loose your changes, and load a new directory?"),QMessageBox::Yes,QMessageBox::No)==QMessageBox::Yes)
              found=false;
          }
        //if found is reset  this will work also
        if (!found)
          {
            QStringList receivelist;
            receivelist<<"tagger";
            messageobject *message=new messageobject(QString("filetree"), receivelist, QString("reread"), QVariant (0));
            adv_comm_if->sendmessage(message);
            updatedirectory(newlink);
          }
      }
  }
}

void filetree::contextMenuEvent ( QContextMenuEvent * event )
{
  this->clearSelection();
  contextmenu->clear();
  storedpoint=event->pos();
  QTreeWidgetItem *contextitem=itemAt(storedpoint);
  if (contextitem)
    {
      contextitem->setSelected(true);
      //Show contextmenu depending on the selection (single file,directory)
      if (static_cast<filetreeitem *>(contextitem)->isdir())
        {
          QAction *renameaction=contextmenu->addAction ( QIcon(":/icons/edit.png"),QString(tr("Rename")));
          QAction *deleteaction=contextmenu->addAction ( QIcon(":/icons/eraser.png"),QString(tr("Delete")));
          QAction *queueaction=contextmenu->addAction ( QIcon(":/icons/edit_add.png"),QString(tr("Append songs on playlist")));
          QAction *playaction=contextmenu->addAction ( QIcon(":/icons/player_play.png"),QString(tr("Play songs")));
          QAction *cdaction=contextmenu->addAction ( QIcon(":/icons/goto.png"),QString(tr("CD in here")));
          connect(deleteaction,SIGNAL(triggered( bool )),this,SLOT(deleteslot()));
          connect(renameaction,SIGNAL(triggered( bool )),this,SLOT(renameslot()));
          connect(cdaction,SIGNAL(triggered( bool )),this,SLOT(cdslot()));
          connect(queueaction,SIGNAL(triggered( bool )),this,SLOT(enqeueslot()));
          connect(playaction,SIGNAL(triggered( bool )),this,SLOT(playslot()));
          contextmenu->popup(mapToGlobal(storedpoint));
        }
      else
        {
          QAction *renameaction=contextmenu->addAction ( QIcon(":/icons/edit.png"),QString(tr("Rename")));
          QAction *deleteaction=contextmenu->addAction ( QIcon(":/icons/eraser.png"),QString(tr("Delete")));
          QAction *queueaction=contextmenu->addAction ( QIcon(":/icons/edit_add.png"),QString(tr("Append songs on playlist")));
          QAction *playaction=contextmenu->addAction ( QIcon(":/icons/player_play.png"),QString(tr("Play songs")));
          connect(deleteaction,SIGNAL(triggered( bool )),this,SLOT(deleteslot()));
          connect(renameaction,SIGNAL(triggered( bool )),this,SLOT(renameslot()));
          connect(queueaction,SIGNAL(triggered( bool )),this,SLOT(enqeueslot()));
          connect(playaction,SIGNAL(triggered( bool )),this,SLOT(playslot()));
          contextmenu->popup(mapToGlobal(storedpoint));
        }
    }
  else
    {
      //Show contextmenu for empty selection
      qDebug("No selection");

    }
}

void filetree::cdslot()
{
  QTreeWidgetItem *contextitem=itemAt(storedpoint);
  bool found=false;
  //Simply check the root-item, if there is something, that is changed, but not saved.
  if (rootitem->ismodified()==1)
    found=true;
  if (found)
    {
      if (QMessageBox::question(this,"K-YAMO-Information",tr("You have changed tags in one or more files.<br>Do you want to loose your changes, and load a new directory?"),QMessageBox::Yes,QMessageBox::No)==QMessageBox::Yes)
        found=false;
    }
  //if found is reset  this will work also
  if (!found)
    {
      QStringList receivelist;
      receivelist<<"tagger";
      messageobject *message=new messageobject(QString("filetree"), receivelist, QString("reread"), QVariant (0));
      adv_comm_if->sendmessage(message);
      updatedirectory(static_cast<filetreeitem *>(contextitem)->getfilename());
    }
}

void filetree::renameslot()
{
  QTreeWidgetItem *contextitem=itemAt(storedpoint);
  openPersistentEditor(contextitem);
  editoropen=true;
}

void filetree::checkrename()
{
  QTreeWidgetItem *contextitem=itemAt(storedpoint);
  closePersistentEditor(contextitem);
}

void filetree::deleteslot()
{
  QTreeWidgetItem *contextitem=itemAt(storedpoint);
  filetreeitem *item=static_cast<filetreeitem *>(contextitem);
  if (QMessageBox::warning(this,"K-YAMO-Information",tr("Do you really want to delete the selected files?<br>All files will be unrecovareable lost!"),QMessageBox::Yes,QMessageBox::No)==QMessageBox::Yes)
    fileoperationhandle->deletefile(item->getfilename());
}

void filetree::enqeueslot()
{
  QStringList playlistadd;
  QList<QTreeWidgetItem *> selection=selectedItems () ;
  int numberfiles=0;
  for (int i=0;i<selection.count();i++)
    {
      filetreeitem *actualitem=static_cast<filetreeitem *>(selection.at(i));
      if (!actualitem->isdir())
        {
          numberfiles++;
          playlistadd.append(actualitem->getfilename());
        }
    }
    if(numberfiles)
    {
	    	    QStringList receivelist;
    receivelist<<"player";
    messageobject *message=new messageobject(QString("filetree"), receivelist, QString("append"), QVariant (playlistadd));
    adv_comm_if->sendmessage(message);
    }
}

void filetree::playslot()
{
	QStringList playlistadd;
	QList<QTreeWidgetItem *> selection=selectedItems () ;
	int numberfiles=0;
	for (int i=0;i<selection.count();i++)
	{
		filetreeitem *actualitem=static_cast<filetreeitem *>(selection.at(i));
		if (!actualitem->isdir())
		{
			numberfiles++;
			playlistadd.append(actualitem->getfilename());
		}
	}
	if(numberfiles)
	{
		QStringList receivelist;
		receivelist<<"player";
		messageobject *message=new messageobject(QString("filetree"), receivelist, QString("play"), QVariant (playlistadd));
		adv_comm_if->sendmessage(message);
	}
}

/*######################################################################################################
 *#                      Filetreeitem-specific stuff following here                                    #
 *######################################################################################################*/

filetreeitem::filetreeitem(filetreeitem *parent): QTreeWidgetItem(static_cast<QTreeWidgetItem*>(parent)),
    modified(0),
    isdirectory(false),
    filename(QString::null)
{
  //folderchangedpic=QIcon(":icons/folderchanged.png");
  setIcon(0,QIcon(":/icons/audiopic.png"));
  setIcon(1,QIcon(":/icons/greenflag.png"));
}

filetreeitem::filetreeitem(QTreeWidget *parent): QTreeWidgetItem(parent),
    modified(0),
    isdirectory(false),
    filename(QString::null)
{
  //folderchangedpic=QIcon(":icons/folderchanged.png");
  setIcon(0,QIcon(":/icons/audiopic.png"));
  setIcon(1,QIcon(":/icons/greenflag.png"));
}

void filetreeitem::setfilename(const QString name)
{
  filename=name;
}

const QString filetreeitem::getfilename()
{
  return filename;
}

void filetreeitem::setmodified(const int value)
{
  if (value>=0&&value<=2)
    {
      modified=value;
      if (modified==0)
        setIcon(1,QIcon(":/icons/greenflag.png"));
      else if (modified==1)
        setIcon(1,QIcon(":/icons/redflag.png"));
      else if (modified==2)
        setIcon(1,QIcon(":/icons/filesave.png"));
      if (modified==1&&isdir())
        setIcon(0,QIcon(":icons/folderchanged.png"));
      else if (modified!=1&&isdir())
        setIcon(0,QIcon(":icons/folder.png"));
    }
}

const int filetreeitem::ismodified()
{
  return modified;
}

void filetreeitem::setdir(const bool value)
{
  isdirectory=value;
  setIcon(0,QIcon(":/icons/folder.png"));
}
const bool filetreeitem::isdir()
{
  return isdirectory;
}
