/***************************************************************************
 *   Copyright (C) 2007 by Roland Weigert   *
 *   roweigert@t-online.de   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
/**
 *@file filetree.h
 *The class for the filetree that shows and handles files and their status.<br>
 *
 *@author Roland Weigert
 *@date 20-02-2009
 *@todo All from scratch
 */

/**
 * If the watcher brings memorywarnings, you have to start this command as root:<br>
 * echo 16384 > /proc/sys/fs/inotify/max_user_watches
 * This enables more watchable files in the kernel.
 */

#ifndef _FILETREE_H_
#define _FILETREE_H_

#include <QTreeWidget>
#include <QTreeWidgetItem>
#include <QDir>
#include <QFileSystemWatcher>
#include <QList>
#include <QStringList>
#include <QIcon>
#include <QDropEvent>
#include <QObject>
#include <QApplication>
#include <QMessageBox>
#include <QMenu>
#include <QContextMenuEvent>

#include "global-stuff/communicationinterface.h"
#include "global-stuff/fileoperator.h"

//Prototypedeclaration
class filetreeitem;

class filetree : public QTreeWidget
  {
    Q_OBJECT
  public:
    filetree(QWidget *parent,const QString directory=QDir::homePath() );
    ~filetree();
    void updatedirectory(const QString newdirectory=QString::null);
    const QString getcurrentdir() const;
  private:
    void dragEnterEvent(QDragEnterEvent *event);
    void dropEvent(QDropEvent *event);
    void dragMoveEvent(QDragMoveEvent *event);
    filetreeitem *rootitem;
    QString currentdir;
    QIcon folderpic;
    QFileSystemWatcher *watcher;
    void getsubdirectory(const QString subdir,filetreeitem *parent);
    void dirselect(filetreeitem *diritem);
    int refreshcount;
    QMenu *contextmenu;
    QPoint storedpoint;
    bool editoropen;
  private slots:
    void recurseselection();
    void filesystemchange(const QString &);
    void rescandirectory(filetreeitem *);
    void removeitem(filetreeitem *);
    void contextMenuEvent ( QContextMenuEvent * event );
    void cdslot();
    void renameslot();
    void deleteslot();
    void checkrename();
    void enqeueslot();
    void playslot();
  signals:
    void fileselectionchanged();
  };


class filetreeitem : public QTreeWidgetItem
  {
  public:
    filetreeitem(filetreeitem *parent);
    filetreeitem(QTreeWidget *parent);
    void setfilename(const QString name);
    const QString getfilename();
    void setmodified(const int modified);
    const int ismodified();
    void setdir(const bool isdir);
    const bool isdir();
  private:
    int modified;
    bool isdirectory;
    QString filename;
  };
#endif
