/***************************************************************************
 *   Copyright (C) 2007 by Roland Weigert   *
 *   roweigert@t-online.de   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include "tagger.h"

extern rcfile *settingsfile;
extern communicationinterface *comm_if;
extern advancedcommunicationinterface *adv_comm_if;
extern fileoperator *fileoperationhandle;

tagger::tagger(QWidget *parent):
    view (parent)
{
  setactivebuttons(validactions::open|validactions::save|validactions::guess|validactions::organise);
  tree=new filetree(this,settingsfile->getworkingdir());
  createtagcontrols();
  splitview=new QSplitter(Qt::Horizontal,this);
  splitview->addWidget(tree);
  splitview->addWidget(controlsbox);
  splitview->setStretchFactor(0,1);
  splitview->setStretchFactor(1,1);
  dialoglayout=new QVBoxLayout(this);
  dialoglayout->addWidget(splitview);
  box=new QWidget(this);
  box->setLayout(dialoglayout);
  this->setWidget(box );
  this->setWidgetResizable( TRUE );
//all ready, than deactivate the widgets, cause nothing is selected
  allenabled(false);
  connect(comm_if,SIGNAL(requestsignal(QString, QString, QVariant)),this,SLOT(requestslot(QString, QString, QVariant)));
  connect(comm_if,SIGNAL(requestsignal(QString, QString, QList<QVariant>)),this,SLOT(requestslot(QString, QString, QList<QVariant>)));
  connect(adv_comm_if,SIGNAL(requestsignal(QStringList,messageobject *)),this,SLOT(requestslot(QStringList,messageobject *)));
  connect (tree,SIGNAL(fileselectionchanged( )),this,SLOT(checkfiles( )));
  connect (albumcombo,SIGNAL( editTextChanged(const QString)),this,SLOT(updatesearchstring(const QString)));
  connect (artistcombo,SIGNAL( editTextChanged(const QString)),this,SLOT(updatesearchstring(const QString)));
  connect (albumcombo,SIGNAL( editTextChanged(const QString)),this,SLOT(albumchanged(const QString)));
  connect (artistcombo,SIGNAL( editTextChanged(const QString)),this,SLOT(artistchanged(const QString)));
  connect (trackcombo,SIGNAL( editTextChanged(const QString)),this,SLOT(titlechanged(const QString)));
  connect (trackspinner,SIGNAL( valueChanged(const int)),this,SLOT(trackchanged(const int)));
  connect (genrecombo,SIGNAL( editTextChanged(const QString)),this,SLOT(genrechanged(const QString)));
  connect (releasespinner,SIGNAL( valueChanged(const int)),this,SLOT(yearchanged(const int)));
  connect(covercontrols,SIGNAL(coverchange()),this,SLOT(coverchanged()));
}

tagger::~tagger()
{
//Clear out all the loaded tags.
  for (int i=0;i<activefileslist.count();i++)
    {
      delete activefileslist.at(i);
    }
}

void tagger::createtagcontrols()
{
  controlsbox=new QGroupBox();
  controlsbox->setMinimumSize(350,550);
  //Artistbox
  artistlabel=new QLabel(tr("Artist"));
  artistcombo=new QComboBox;
  artistcombo->setEditable(TRUE);
  QStringList fill;
  fill<<""<<"Hallo"<<"Ich"<<"Füll"<<"Dir"<<"Eine";
  artistcompleter=new QCompleter(fill,artistcombo);
  artistcompleter->setCaseSensitivity(Qt::CaseInsensitive);
  artistcombo->setCompleter(artistcompleter);
  artistcombo->addItems(fill);
  artistbox=new QGroupBox;
  artistboxlayout=new QHBoxLayout;
  artistboxlayout->setMargin(15);
  artistboxlayout->addWidget(artistlabel,10);
  artistboxlayout->addWidget(artistcombo,50);
  artistbox->setLayout(artistboxlayout);
  //Albumbox
  albumlabel=new QLabel(tr("Album"));
  albumcombo=new QComboBox;
  albumcombo->setEditable(TRUE);
  albumbox=new QGroupBox;
  albumboxlayout=new QHBoxLayout;
  albumboxlayout->setMargin(15);
  albumboxlayout->addWidget(albumlabel,10);
  albumboxlayout->addWidget(albumcombo,50);
  albumbox->setLayout(albumboxlayout);
  //Trackbox
  trackspinner=new QSpinBox;
  trackspinner->setRange(0,99);
  trackcombo=new QComboBox;
  trackcombo->setEditable(TRUE);
  trackbox=new QGroupBox;
  trackboxlayout=new QHBoxLayout;
  trackboxlayout->setMargin(15);
  trackboxlayout->addWidget(trackspinner,10);
  trackboxlayout->addWidget(trackcombo,50);
  trackbox->setLayout(trackboxlayout);
  //Genrebox
  genrelabel=new QLabel(tr("Genre"));
  genrecombo=new QComboBox;
  genrecombo->setEditable(TRUE);
  genrebox=new QGroupBox;
  genreboxlayout=new QHBoxLayout;
  genreboxlayout->setMargin(15);
  genreboxlayout->addWidget(genrelabel,10);
  genreboxlayout->addWidget(genrecombo,50);
  genrebox->setLayout(genreboxlayout);
  //Releasebox
  releaselabel=new QLabel(tr("Release"));
  releasespinner=new QSpinBox;
  releasespinner->setRange(0,9999);
  releasebox=new QGroupBox;
  releaseboxlayout=new QHBoxLayout;
  releaseboxlayout->setMargin(15);
  releaseboxlayout->addWidget(releaselabel,10);
  releaseboxlayout->addStretch();
  releaseboxlayout->addWidget(releasespinner,5);
  releasebox->setLayout(releaseboxlayout);
  //Coverbox
  covercontrols=new coverbox(this);
  //Taglayout
  taglayout=new QVBoxLayout;
  taglayout->addWidget(artistbox);
  taglayout->addWidget(albumbox);
  taglayout->addWidget(trackbox);
  taglayout->addWidget(genrebox);
  taglayout->addWidget(releasebox);
  taglayout->addWidget(covercontrols);
  controlsbox->setLayout(taglayout);
}

void tagger::changedirectorytree()
{
  //Check if there are some modified files
  bool found=false;
  for (int i=0;i<activefileslist.count();i++)
    {
      if (activefileslist.at(i)->item->ismodified()==1)
        {
          found=true;
          break;
        }
    }
  if (found)
    {
      if (QMessageBox::question(this,"K-YAMO-Information",tr("You have changed tags in one or more files.<br>Do you want to loose your changes, and load a new directory?"),QMessageBox::Yes,QMessageBox::No)==QMessageBox::Yes)
        found=false;
    }
  //if found is reset  this will work also
  if (!found)
    {
      QString newpath = QFileDialog::getExistingDirectory(this, tr("Open Directory"),tree->getcurrentdir(),QFileDialog::ShowDirsOnly| QFileDialog::DontResolveSymlinks);
      QVariant data(0);
      //Send this to mainwindow, so it knows, that no files are selected
      comm_if->requestslot(QString("tagger"),QString("nofilesselected"), data);
      //Clear out the pointers of the activefileslist
      for (int i=0;i<activefileslist.count();i++)
        delete activefileslist.at(i);
      activefileslist.clear();
      if (!newpath.isNull())
        {
          QStringList receivelist;
          receivelist<<"main";
          messageobject *message=new messageobject("tagger", receivelist, QString("openbutton"), QVariant (false));
          adv_comm_if->sendmessage(message);
          tree->updatedirectory(newpath);
          message=new messageobject("tagger", receivelist, QString("openbutton"), QVariant (true));
          adv_comm_if->sendmessage(message);
        }
    }
  checkfiles();
}

void tagger::checkfiles( )
{
  disconnect (albumcombo,SIGNAL( editTextChanged(const QString)),this,SLOT(albumchanged(const QString)));
  disconnect (artistcombo,SIGNAL( editTextChanged(const QString)),this,SLOT(artistchanged(const QString)));
  disconnect (trackcombo,SIGNAL( editTextChanged(const QString)),this,SLOT(titlechanged(const QString)));
  disconnect (trackspinner,SIGNAL( valueChanged(const int)),this,SLOT(trackchanged(const int)));
  disconnect (genrecombo,SIGNAL( editTextChanged(const QString)),this,SLOT(genrechanged(const QString)));
  disconnect (releasespinner,SIGNAL( valueChanged(const int)),this,SLOT(yearchanged(const int)));
  disconnect(covercontrols,SIGNAL(coverchange()),this,SLOT(coverchanged()));
//First clear out everything
  artistcombo->setEditText(QString::null);
  albumcombo->setEditText(QString::null);
  trackcombo->setEditText(QString::null);
  trackspinner->setValue(0);
  genrecombo->setEditText(QString::null);
  releasespinner->setValue(0);
  covercontrols->clearimages(TRUE);
  QList<QTreeWidgetItem *> selection=tree->selectedItems () ;
  int numberfiles=0;
  for (int i=0;i<selection.count();i++)
    {
      filetreeitem *actualitem=static_cast<filetreeitem *>(selection.at(i));
      if (!actualitem->isdir())
        numberfiles++;
    }
//##################################################################################################
  if (numberfiles>1)
    {
      allenabled(true);
      multiselect(true);
      //Scan all the files through for matching parts
      //We also need to read them all in, if not yet done, so , this is no big hassle
      QString artist,album,title,genre;
      int track=0;
      int year=0;
      QImage front,media,back;
      bool artistout=false;
      bool albumout=false;
      bool titleout=false;
      bool genreout=false;
      bool yearout=false;
      bool coverout=false;
      bool firstread=true;
      for (int i=0;i<selection.count();i++)
        {
          filetreeitem *actualitem=static_cast<filetreeitem *>(selection.at(i));
          if (!actualitem->isdir())
            {
              QList<activetagfile *>::Iterator activefileswp=activefileslist.begin();
              bool found=false;
              while (!found&& activefileswp!=activefileslist.end())
                {
                  if (!(*activefileswp)->item->getfilename().compare(actualitem->getfilename()))
                    found=true;
                  else
                    activefileswp++;
                }
              if (!found)
                {
                  mediafile *actualtagfile=new mediafile(actualitem->getfilename());
                  if (actualtagfile->issupportedfile())
                    {
                      actualtagfile->readtags(tagversion::v2tag);
                      if (firstread)
                        {
                          artist=actualtagfile->getartistname();
                          album=actualtagfile->getalbumname();
                          title=actualtagfile->getsongname();
                          track=actualtagfile->gettracknumber();
                          genre=actualtagfile->getgenrename();
                          year=actualtagfile->getreleasedate();
                          artistcombo->setEditText(artist);
                          albumcombo->setEditText(album);
                          trackcombo->setEditText(title);
                          trackspinner->setValue(track);
                          genrecombo->setEditText(genre);
                          releasespinner->setValue(year);
                          if (actualtagfile->canhandlepicture())
                            {
                              const QList<coverart> coverlist=actualtagfile->getpictures();
                              QList<coverart>::ConstIterator coverindex=coverlist.begin();
                              for (int i=0;i<coverlist.count();i++)
                                {
                                  if (coverlist.at(i).picposition==Coverart::FrontCover)
                                    front=coverlist.at(i).image;
                                  if (coverlist.at(i).picposition==Coverart::Media)
                                    media=coverlist.at(i).image;
                                  if (coverlist.at(i).picposition==Coverart::BackCover)
                                    back=coverlist.at(i).image;
                                }
                            }
                          else
                            covercontrols->clearimages(FALSE);
                          activetagfile *actualfile=new activetagfile;
                          actualfile->item=actualitem;
                          actualfile->tagfile=actualtagfile;
                          activefileslist.append(actualfile);
                          firstread=false;
                        }
                      else
                        {
                          if (!artistout && artist.compare(actualtagfile->getartistname()))
                            {
                              artistout=true;
                              artistbox->setChecked(false);
                              artistcombo->setEditText(QString::null);
                            }
                          if (!albumout && album.compare(actualtagfile->getalbumname()))
                            {
                              albumout=true;
                              albumbox->setChecked(false);
                              albumcombo->setEditText(QString::null);
                            }
                          if (!genreout && genre.compare(actualtagfile->getgenrename()))
                            {
                              genreout=true;
                              genrebox->setChecked(false);
                              genrecombo->setEditText(QString::null);
                            }
                          if (!titleout && title.compare(actualtagfile->getsongname())&& track!=actualtagfile->gettracknumber())
                            {
                              titleout=true;
                              trackbox->setChecked(false);
                              trackcombo->setEditText(QString::null);
                              trackspinner->setValue(0);
                            }
                          if (!yearout && year!=actualtagfile->getreleasedate())
                            {
                              yearout=true;
                              releasebox->setChecked(false);
                              releasespinner->setValue(0);
                            }
                          if (actualtagfile->canhandlepicture()&&!coverout)
                            {
                              const QList<coverart> coverlist=actualtagfile->getpictures();
                              QList<coverart>::ConstIterator coverindex=coverlist.begin();
                              while (coverindex!=coverlist.end())
                                {
                                  if ((*coverindex).picposition==Coverart::FrontCover)
                                    if (front!=(*coverindex).image)
                                      coverout=true;
                                  if ((*coverindex).picposition==Coverart::Media)
                                    if (media!=(*coverindex).image)
                                      coverout=true;
                                  if ((*coverindex).picposition==Coverart::BackCover)
                                    if (back!=(*coverindex).image)
                                      coverout=true;
                                  coverindex++;
                                }
                            }
                          else
                            coverout=true;
                          activetagfile *actualfile=new activetagfile;
                          actualfile->item=actualitem;
                          actualfile->tagfile=actualtagfile;
                          activefileslist.append(actualfile);
                        }
                    }
                  else
                    delete actualtagfile;
                }
              else
                {
                  if (firstread)
                    {
                      artist=(*activefileswp)->tagfile->getartistname();
                      album=(*activefileswp)->tagfile->getalbumname();
                      title=(*activefileswp)->tagfile->getsongname();
                      track=(*activefileswp)->tagfile->gettracknumber();
                      genre=(*activefileswp)->tagfile->getgenrename();
                      year=(*activefileswp)->tagfile->getreleasedate();
                      artistcombo->setEditText(artist);
                      albumcombo->setEditText(album);
                      trackcombo->setEditText(title);
                      trackspinner->setValue(track);
                      genrecombo->setEditText(genre);
                      releasespinner->setValue(year);
                      if ((*activefileswp)->tagfile->canhandlepicture())
                        {
                          const QList<coverart> coverlist=(*activefileswp)->tagfile->getpictures();
                          QList<coverart>::ConstIterator coverindex=coverlist.begin();
                          while (coverindex!=coverlist.end())
                            {
                              if ((*coverindex).picposition==Coverart::FrontCover)
                                front=(*coverindex).image;
                              if ((*coverindex).picposition==Coverart::Media)
                                media=(*coverindex).image;
                              if ((*coverindex).picposition==Coverart::BackCover)
                                back=(*coverindex).image;
                              coverindex++;
                            }
                        }
                      else
                        coverout=true;
                      firstread=false;
                    }
                  else
                    {
                      if (!artistout && artist.compare((*activefileswp)->tagfile->getartistname()))
                        {
                          artistout=true;
                          artistbox->setChecked(false);
                          artistcombo->setEditText(QString::null);
                        }
                      if (!albumout && album.compare((*activefileswp)->tagfile->getalbumname()))
                        {
                          albumout=true;
                          albumbox->setChecked(false);
                          albumcombo->setEditText(QString::null);
                        }
                      if (!genreout && genre.compare((*activefileswp)->tagfile->getgenrename()))
                        {
                          genreout=true;
                          genrebox->setChecked(false);
                          genrecombo->setEditText(QString::null);
                        }
                      if (!titleout && title.compare((*activefileswp)->tagfile->getsongname())&& track!=(*activefileswp)->tagfile->gettracknumber())
                        {
                          titleout=true;
                          trackbox->setChecked(false);
                          trackcombo->setEditText(QString::null);
                          trackspinner->setValue(0);
                        }
                      if (!yearout && year!=(*activefileswp)->tagfile->getreleasedate())
                        {
                          yearout=true;
                          releasebox->setChecked(false);
                          releasespinner->setValue(0);
                        }
                      if ((*activefileswp)->tagfile->canhandlepicture()&&!coverout)
                        {
                          const QList<coverart> coverlist=(*activefileswp)->tagfile->getpictures();
                          QList<coverart>::ConstIterator coverindex=coverlist.begin();
                          while (coverindex!=coverlist.end())
                            {
                              if ((*coverindex).picposition==Coverart::FrontCover)
                                if (front!=(*coverindex).image)
                                  coverout=true;
                              if ((*coverindex).picposition==Coverart::Media)
                                if (media!=(*coverindex).image)
                                  coverout=true;
                              if ((*coverindex).picposition==Coverart::BackCover)
                                if (back!=(*coverindex).image)
                                  coverout=true;
                              coverindex++;
                            }
                        }
                      else
                        coverout=true;
                    }
                }
            }
        }
      if (!coverout)
        {
          if (!front.isNull())
            covercontrols->setimage(front,Coverart::FrontCover);
          if (!back.isNull())
            covercontrols->setimage(back,Coverart::BackCover);
          if (!media.isNull())
            covercontrols->setimage(media,Coverart::Media);
          covercontrols->setChecked(true);
        }
      else
        covercontrols->setChecked(false);
    }
//###############################################################################
  else
    {
      multiselect(false);
      if (!numberfiles)
        {
          allenabled(false);
        }
      else
        {
          allenabled(true);
          //only one file, to process, but we have to check, wether we loaded this already.
          bool found=false;
          for (int i=0;i<selection.count();i++)
            {
              filetreeitem *actualitem=static_cast<filetreeitem *>(selection.at(i));
              if (!actualitem->isdir())
                {
                  QList<activetagfile *>::Iterator activefileswp=activefileslist.begin();
                  found=false;
                  while (!found&& activefileswp!=activefileslist.end())
                    {
                      if (!(*activefileswp)->item->getfilename().compare(actualitem->getfilename()))
                        found=true;
                      else
                        activefileswp++;
                    }
                  if (!found)
                    {
                      mediafile *actualtagfile=new mediafile(actualitem->getfilename());
                      if (actualtagfile->issupportedfile())
                        {
                          actualtagfile->readtags(tagversion::v2tag);
                          artistcombo->setEditText(actualtagfile->getartistname());
                          albumcombo->setEditText(actualtagfile->getalbumname());
                          trackcombo->setEditText(actualtagfile->getsongname());
                          trackspinner->setValue(actualtagfile->gettracknumber());
                          genrecombo->setEditText(actualtagfile->getgenrename());
                          releasespinner->setValue(actualtagfile->getreleasedate());
                          if (actualtagfile->canhandlepicture())
                            {
                              const QList<coverart> coverlist=actualtagfile->getpictures();
                              QList<coverart>::ConstIterator coverindex=coverlist.begin();
                              for (int i=0;i<coverlist.count();i++)
                                {
                                  if (!coverlist.at(i).image.isNull())
                                    covercontrols->setimage(coverlist.at(i).image,coverlist.at(i).picposition);
                                }
                            }
                          else
                            covercontrols->clearimages(FALSE);
                          activetagfile *actualfile=new activetagfile;
                          actualfile->item=actualitem;
                          actualfile->tagfile=actualtagfile;
                          activefileslist.append(actualfile);
                        }
                      else
                        delete actualtagfile;
                    }
                  else
                    {
                      artistcombo->setEditText((*activefileswp)->tagfile->getartistname());
                      albumcombo->setEditText((*activefileswp)->tagfile->getalbumname());
                      trackcombo->setEditText((*activefileswp)->tagfile->getsongname());
                      trackspinner->setValue((*activefileswp)->tagfile->gettracknumber());
                      genrecombo->setEditText((*activefileswp)->tagfile->getgenrename());
                      releasespinner->setValue((*activefileswp)->tagfile->getreleasedate());
                      if ((*activefileswp)->tagfile->canhandlepicture())
                        {
                          const QList<coverart> coverlist=(*activefileswp)->tagfile->getpictures();
                          QList<coverart>::ConstIterator coverindex=coverlist.begin();
                          {
                            if (!coverlist.at(i).image.isNull())
                              covercontrols->setimage(coverlist.at(i).image,coverlist.at(i).picposition);
                          }
                        }
                      else
                        covercontrols->clearimages(FALSE);
                    }
                }
            }
        }
    }
  QVariant data(0);
  if (numberfiles)
    comm_if->requestslot(QString("tagger"),QString("filesselected"), data);
  else
    comm_if->requestslot(QString("tagger"),QString("nofilesselected"), data);
  connect (albumcombo,SIGNAL( editTextChanged(const QString)),this,SLOT(albumchanged(const QString)));
  connect (artistcombo,SIGNAL( editTextChanged(const QString)),this,SLOT(artistchanged(const QString)));
  connect (trackcombo,SIGNAL( editTextChanged(const QString)),this,SLOT(titlechanged(const QString)));
  connect (trackspinner,SIGNAL( valueChanged(const int)),this,SLOT(trackchanged(const int)));
  connect (genrecombo,SIGNAL( editTextChanged(const QString)),this,SLOT(genrechanged(const QString)));
  connect (releasespinner,SIGNAL( valueChanged(const int)),this,SLOT(yearchanged(const int)));
  connect(covercontrols,SIGNAL(coverchange()),this,SLOT(coverchanged()));
}

void tagger::multiselect(bool value)
{
  artistbox->setCheckable(value);
  albumbox->setCheckable(value);
  trackbox->setCheckable(value);
  genrebox->setCheckable(value);
  releasebox->setCheckable(value);
  covercontrols->setCheckable(value);
}

void tagger::allenabled(bool value)
{
  artistbox->setEnabled(value);
  albumbox->setEnabled(value);
  trackbox->setEnabled(value);
  genrebox->setEnabled(value);
  releasebox->setEnabled(value);
  covercontrols->setEnabled(value);
}

void tagger::updatesearchstring(const QString)
{
  covercontrols->setsearchstring(albumcombo->currentText()+"+"+artistcombo->currentText());
}

void tagger::albumchanged(const QString text)
{
  QList<QTreeWidgetItem *> selection=tree->selectedItems () ;
  for (int i=0;i<selection.count();i++)
    {
      filetreeitem *actualitem=static_cast<filetreeitem *>(selection.at(i));
      if (!actualitem->isdir())
        {
          //Find the file in the loadedtagslist
          for (int j=0;j<activefileslist.count();j++)
            {
              if (!activefileslist.at(j)->item->getfilename().compare(actualitem->getfilename()))
                {
                  activefileslist.at(j)->tagfile->setalbumname(text);
                  actualitem->setmodified(1);
                  setparrentchanged(actualitem);
                  break;
                }
            }
        }
    }
}

void tagger::artistchanged(const QString text)
{
  QList<QTreeWidgetItem *> selection=tree->selectedItems () ;
  for (int i=0;i<selection.count();i++)
    {
      filetreeitem *actualitem=static_cast<filetreeitem *>(selection.at(i));
      if (!actualitem->isdir())
        {
          //Find the file in the loadedtagslist
          for (int j=0;j<activefileslist.count();j++)
            {
              if (!activefileslist.at(j)->item->getfilename().compare(actualitem->getfilename()))
                {
                  activefileslist.at(j)->tagfile->setartistname(text);
                  actualitem->setmodified(1);
                  setparrentchanged(actualitem);
                  break;
                }
            }
        }
    }
}

void tagger::genrechanged(const QString text)
{
  QList<QTreeWidgetItem *> selection=tree->selectedItems () ;
  for (int i=0;i<selection.count();i++)
    {
      filetreeitem *actualitem=static_cast<filetreeitem *>(selection.at(i));
      if (!actualitem->isdir())
        {
          //Find the file in the loadedtagslist
          for (int j=0;j<activefileslist.count();j++)
            {
              if (!activefileslist.at(j)->item->getfilename().compare(actualitem->getfilename()))
                {
                  activefileslist.at(j)->tagfile->setgenrename(text);
                  actualitem->setmodified(1);
                  setparrentchanged(actualitem);
                  break;
                }
            }
        }
    }
}

void tagger::titlechanged(const QString text)
{
  QList<QTreeWidgetItem *> selection=tree->selectedItems () ;
  for (int i=0;i<selection.count();i++)
    {
      filetreeitem *actualitem=static_cast<filetreeitem *>(selection.at(i));
      if (!actualitem->isdir())
        {
          //Find the file in the loadedtagslist
          for (int j=0;j<activefileslist.count();j++)
            {
              if (!activefileslist.at(j)->item->getfilename().compare(actualitem->getfilename()))
                {
                  activefileslist.at(j)->tagfile->setsongname(text);
                  actualitem->setmodified(1);
                  setparrentchanged(actualitem);
                  break;
                }
            }
        }
    }
}

void tagger::trackchanged(const int value)
{
  QList<QTreeWidgetItem *> selection=tree->selectedItems () ;
  for (int i=0;i<selection.count();i++)
    {
      filetreeitem *actualitem=static_cast<filetreeitem *>(selection.at(i));
      if (!actualitem->isdir())
        {
          //Find the file in the loadedtagslist
          for (int j=0;j<activefileslist.count();j++)
            {
              if (!activefileslist.at(j)->item->getfilename().compare(actualitem->getfilename()))
                {
                  activefileslist.at(j)->tagfile->settracknumber(value);
                  actualitem->setmodified(1);
                  setparrentchanged(actualitem);
                  break;
                }
            }
        }
    }
}

void tagger::yearchanged(const int value)
{
  QList<QTreeWidgetItem *> selection=tree->selectedItems () ;
  for (int i=0;i<selection.count();i++)
    {
      filetreeitem *actualitem=static_cast<filetreeitem *>(selection.at(i));
      if (!actualitem->isdir())
        {
          //Find the file in the loadedtagslist
          for (int j=0;j<activefileslist.count();j++)
            {
              if (!activefileslist.at(j)->item->getfilename().compare(actualitem->getfilename()))
                {
                  activefileslist.at(j)->tagfile->setreleasedate(value);
                  actualitem->setmodified(1);
                  setparrentchanged(actualitem);
                  break;
                }
            }
        }
    }
}

void tagger::coverchanged()
{
  const QImage front(covercontrols->getfrontimage());
  const QImage back(covercontrols->getbackimage());
  const QImage media(covercontrols->getmediaimage());
  QList<QTreeWidgetItem *> selection=tree->selectedItems () ;
  for (int i=0;i<selection.count();i++)
    {
      filetreeitem *actualitem=static_cast<filetreeitem *>(selection.at(i));
      if (!actualitem->isdir())
        {
          //Find the file in the loadedtagslist
          for (int j=0;j<activefileslist.count();j++)
            {
              if (!activefileslist.at(j)->item->getfilename().compare(actualitem->getfilename()))
                {

                  activefileslist.at(j)->tagfile->setpicture(front,Coverart::FrontCover);
                  activefileslist.at(j)->tagfile->setpicture(back,Coverart::BackCover);
                  activefileslist.at(j)->tagfile->setpicture(media,Coverart::Media);
                  actualitem->setmodified(1);
                  setparrentchanged(actualitem);
                  break;
                }
            }
        }
    }
}

void tagger::setparrentchanged(QTreeWidgetItem *file)
{
//Highlite the folders down to the root with recursion, when a file is highlited
  QTreeWidgetItem *parent=file->parent();
  if (parent)
    {
      static_cast<filetreeitem *>(parent)->setmodified(true);
      setparrentchanged(parent);
    }
}

void tagger::checkparentclean(QTreeWidgetItem *file)
{
  QTreeWidgetItem *parent=file->parent();
  if (parent)
    {
      int modifiedfile=0;
      for (int i=0;i<parent->childCount();i++)
        {
          if (static_cast<filetreeitem *>(parent->child(i))->ismodified()==1)
            {
              modifiedfile=1;
              break;
            }
          else if (static_cast<filetreeitem *>(parent->child(i))->ismodified()==2)
            modifiedfile=2;
        }
      if (modifiedfile!=1)
        {
          static_cast<filetreeitem *>(parent)->setmodified(modifiedfile);
          if (parent->parent())
            checkparentclean(parent);
        }
    }
}

void tagger::saveslot()
{
  QList<QTreeWidgetItem *> selection=tree->selectedItems () ;
  int filecount=0;
  // Generate a datestamp as handshake for progresswidget
  QString datestamp=QDateTime::currentDateTime().toString();
  for (int i=0;i<selection.count();i++)
    {
      filetreeitem *actualitem=static_cast<filetreeitem *>(selection.at(i));
      if (!actualitem->isdir())
        filecount++;
    }
  int displaycount=0;
  for (int i=0;i<selection.count();i++)
    {
      filetreeitem *actualitem=static_cast<filetreeitem *>(selection.at(i));
      if (!actualitem->isdir())
        {
          displaycount++;
          QList<activetagfile *>::Iterator activefileswp=activefileslist.begin();
          bool found=false;
          while (!found&& activefileswp!=activefileslist.end())
            {
              if (!(*activefileswp)->item->getfilename().compare(actualitem->getfilename()))
                found=true;
              else
                activefileswp++;
            }
          if ((*activefileswp)->tagfile->savetags())
            (*activefileswp)->item->setmodified(2);
          else if (QMessageBox::question(this,"K-YAMO-Information",QString(tr("Cant save file<br><b>%1</b> <br>Check your permissions.")).arg((*activefileswp)->item->getfilename()),
                                         QMessageBox::Abort,QMessageBox::Ok)==QMessageBox::Abort)
            break;
        }
      comm_progress transfer={displaycount,filecount,QString::null};
      QVariant variant;
      variant.setValue(transfer);
      comm_if->requestslot(QString("tagger"),QString("saveprocess"),variant);
      checkparentclean(actualitem);
    }
  comm_progress transfer={0,0,QString::null};
  QVariant variant;
  variant.setValue(transfer);
  comm_if->requestslot(QString("tagger"),QString("finished"),variant);
}

void tagger::organiseslot()
{
  QList<QTreeWidgetItem *> selection=tree->selectedItems () ;
  int filecount=0;
  // Generate a datestamp as handshake for progresswidget
  QString datestamp=QDateTime::currentDateTime().toString();
  for (int i=0;i<selection.count();i++)
    {
      filetreeitem *actualitem=static_cast<filetreeitem *>(selection.at(i));
      if (!actualitem->isdir())
        filecount++;
    }
  for (int i=0;i<selection.count();i++)
    {
      filetreeitem *actualitem=static_cast<filetreeitem *>(selection.at(i));
      if (!actualitem->isdir())
        {
          if (settingsfile->getsav_move())
            fileoperationhandle->movefile(actualitem->getfilename());
          else
            fileoperationhandle->copyfile(actualitem->getfilename());
        }
      checkparentclean(actualitem);
    }
}

void tagger::requestslot(QStringList destination,messageobject *message)
{
  if (destination.contains("tagger"))
    {
      qDebug("tagger in list of widgets");
      message->lock();
      if (!message->getsender().compare("discread"))
        {
          if (!message->gettype().compare("finished"))
            {
            }
        }
      else if (!message->getsender().compare("encoder"))
        {
          if (!message->gettype().compare("finished"))
            {
            }
        }
      else if (!message->getsender().compare("filetree"))
        {
          if (!message->gettype().compare("remove"))
            {
              QString filename=message->getdata().toString();
              for (int i=0;i<activefileslist.count();i++)
                {
                  if (!activefileslist.at(i)->item->getfilename().compare(filename))
                    {
                      activefileslist.removeAt(i);
                      break;
                    }
                }
            }
          else if (!message->gettype().compare("reread"))
            {
              activefileslist.clear();
            }
        }
      if (!message->confirm("tagger"))
        adv_comm_if->deletemessage(message->getmessageid());
      message->unlock();
    }
}

void tagger::requestslot(QString sender,QString type, QVariant data)
{
  if (!sender.compare("main"))
    {
      if (data.canConvert<int>())
        {
          if (!type.compare("dirchange"))
            changedirectorytree();
          else if (!type.compare("save"))
            saveslot();
          else if (!type.compare("organise"))
            organiseslot();
        }
    }
}

void tagger::requestslot(QString sender,QString type, QList<QVariant>)
{
// This is not needed in this class. (Myabe this will change later)
}

