/***************************************************************************
 *   Copyright (C) 2007 by Roland Weigert   *
 *   roweigert@t-online.de   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
/**
 *@file tagger.h
 *
 *This file holds the class for the taggersubwindow.<br>
 *It handles all tagging itself, and gets only signals from the mainwindow.
 *@author Roland Weigert
 *@date 06-03-2009
 *@todo Add discogs and musicbrainz-support
 */

#ifndef _TAGGER_H_
#define _TAGGER_H_

#include <QScrollArea>
#include <QTreeView>
#include <QTreeWidgetItem>
#include <QItemSelectionModel>
#include <QDirModel>
#include <QDir>
#include <QSplitter>
#include <QString>
#include <QGroupBox>
#include <QLayout>
#include <QComboBox>
#include <QLabel>
#include <QSpinBox>
#include <QImage>
#include <QList>
#include <QWidget>
#include <QCompleter>
#include <QDateTime>

#include "global-stuff/view.h"
#include "global-stuff/fileoperator.h"
#include "tag-interfaces/mediafile.h"
#include "settings/rcfile.h"
#include "coverbox/coverbox.h"
#include "tagger/filetree.h"
#include "global-stuff/communicationinterface.h"

//prototypedeclaration
struct activetagfile;

/**
*@brief A class for showing taginformation, and changing them
*
*It relies on the mediafile as backend.<br>
*
*@todo Add saving of files.
*/
class tagger : public view
  {
    Q_OBJECT
  public:
    /**
    *@brief Constructor
    *
    *Creates a tagger object, that will take care of itself.
    */
    tagger(QWidget *parent);
    /**
    *@brief Destructor
    *
    *Deletes all used widgets.
    */
    ~tagger();
  private:
    /**
    * This list holds all files, that are already read in by tagger
     */
    QList<activetagfile *> activefileslist;
    QVBoxLayout *dialoglayout;
    QWidget *box;
    QSplitter *splitview;
    QDirModel *model;
    filetree *tree;
    QGroupBox *controlsbox;
    //Artistbox
    QGroupBox *artistbox;
    QLabel *artistlabel;
    QComboBox *artistcombo;
    QCompleter *artistcompleter;
    QHBoxLayout *artistboxlayout;
    //Albumbox
    QGroupBox *albumbox;
    QLabel *albumlabel;
    QComboBox *albumcombo;
    QHBoxLayout *albumboxlayout;
    //Trackbox
    QGroupBox *trackbox;
    QSpinBox *trackspinner;
    QComboBox *trackcombo;
    QHBoxLayout *trackboxlayout;
    //Genrebox
    QGroupBox *genrebox;
    QLabel *genrelabel;
    QComboBox *genrecombo;
    QHBoxLayout *genreboxlayout;
    //Releasebox
    QGroupBox *releasebox;
    QLabel *releaselabel;
    QSpinBox *releasespinner;
    QHBoxLayout *releaseboxlayout;
    //Coverbox
    coverbox *covercontrols;
    //Taglayout
    QVBoxLayout *taglayout;
    void setparrentchanged(QTreeWidgetItem *file);
    void checkparentclean(QTreeWidgetItem *file);
  private slots:
    /**
    *@brief Read in the selected files using mediafile as backend.
    */
    void checkfiles();
    void updatesearchstring(const QString);
    void albumchanged(const QString);
    void artistchanged(const QString);
    void genrechanged(const QString);
    void titlechanged(const QString);
    void trackchanged(const int);
    void yearchanged(const int);
    void coverchanged();
    void saveslot();
    void organiseslot();
    void requestslot(QString sender,QString type, QVariant data);
    void requestslot(QString sender,QString type, QList<QVariant> data);
    void requestslot(QStringList,messageobject *);
    /**
    *@brief Create all editable boxes for the tagger-gui.
    */
    void createtagcontrols();
    /**
    *@brief Create the Treeviewwidget, that shows all files.
    */
    void changedirectorytree();
    void multiselect(bool);
    void allenabled(bool);
  signals:
    void needprogress();
  };

struct activetagfile
  {
    filetreeitem *item;
    mediafile *tagfile;
  };

#endif
